@@ -9,6 +9,7 @@
 #include <utility>
 
 #include "base/atomic_sequence_num.h"
+#include "base/bits.h"
 #include "base/debug/stack_trace.h"
 #include "base/files/file.h"
 #include "base/files/file_path.h"
@@ -25,6 +26,7 @@
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/threading/platform_thread.h"
+#include "build/build_config.h"
 
 namespace base {
 namespace debug {
@@ -51,26 +53,11 @@ const char kProcessPhaseDataKey[] = "process-phase";
 // in the same memory space.
 AtomicSequenceNumber g_next_id;
 
-union ThreadRef {
-  int64_t as_id;
-#if defined(OS_WIN)
-  // On Windows, the handle itself is often a pseudo-handle with a common
-  // value meaning "this thread" and so the thread-id is used. The former
-  // can be converted to a thread-id with a system call.
-  PlatformThreadId as_tid;
-#elif defined(OS_POSIX)
-  // On Posix, the handle is always a unique identifier so no conversion
-  // needs to be done. However, it's value is officially opaque so there
-  // is no one correct way to convert it to a numerical identifier.
-  PlatformThreadHandle::Handle as_handle;
-#endif
-};
-
 // Gets the next non-zero identifier. It is only unique within a process.
 uint32_t GetNextDataId() {
   uint32_t id;
-  while ((id = g_next_id.GetNext()) == 0)
-    ;
+  while ((id = g_next_id.GetNext()) == 0) {
+  }
   return id;
 }
 
@@ -105,12 +92,12 @@ PersistentMemoryAllocator::Reference AllocateFrom(
 
 // Determines the previous aligned index.
 size_t RoundDownToAlignment(size_t index, size_t alignment) {
-  return index & (0 - alignment);
+  return bits::AlignDown(index, alignment);
 }
 
 // Determines the next aligned index.
 size_t RoundUpToAlignment(size_t index, size_t alignment) {
-  return (index + (alignment - 1)) & (0 - alignment);
+  return bits::Align(index, alignment);
 }
 
 // Converts "tick" timing into wall time.
@@ -120,8 +107,23 @@ Time WallTimeFromTickTime(int64_t ticks_start, int64_t ticks, Time time_start) {
 
 }  // namespace
 
-OwningProcess::OwningProcess() {}
-OwningProcess::~OwningProcess() {}
+union ThreadRef {
+  int64_t as_id;
+#if defined(OS_WIN)
+  // On Windows, the handle itself is often a pseudo-handle with a common
+  // value meaning "this thread" and so the thread-id is used. The former
+  // can be converted to a thread-id with a system call.
+  PlatformThreadId as_tid;
+#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+  // On Posix and Fuchsia, the handle is always a unique identifier so no
+  // conversion needs to be done. However, its value is officially opaque so
+  // there is no one correct way to convert it to a numerical identifier.
+  PlatformThreadHandle::Handle as_handle;
+#endif
+};
+
+OwningProcess::OwningProcess() = default;
+OwningProcess::~OwningProcess() = default;
 
 void OwningProcess::Release_Initialize(int64_t pid) {
   uint32_t old_id = data_id.load(std::memory_order_acquire);
@@ -185,7 +187,7 @@ ActivityTrackerMemoryAllocator::ActivityTrackerMemoryAllocator(
   DCHECK(allocator);
 }
 
-ActivityTrackerMemoryAllocator::~ActivityTrackerMemoryAllocator() {}
+ActivityTrackerMemoryAllocator::~ActivityTrackerMemoryAllocator() = default;
 
 ActivityTrackerMemoryAllocator::Reference
 ActivityTrackerMemoryAllocator::GetObjectReference() {
@@ -260,8 +262,9 @@ void Activity::FillFrom(Activity* activity,
   activity->activity_type = type;
   activity->data = data;
 
-#if defined(SYZYASAN)
-  // Create a stacktrace from the current location and get the addresses.
+#if (!defined(OS_NACL) && DCHECK_IS_ON()) || defined(ADDRESS_SANITIZER)
+  // Create a stacktrace from the current location and get the addresses for
+  // improved debuggability.
   StackTrace stack_trace;
   size_t stack_depth;
   const void* const* stack_addrs = stack_trace.Addresses(&stack_depth);
@@ -276,9 +279,9 @@ void Activity::FillFrom(Activity* activity,
 #endif
 }
 
-ActivityUserData::TypedValue::TypedValue() {}
+ActivityUserData::TypedValue::TypedValue() = default;
 ActivityUserData::TypedValue::TypedValue(const TypedValue& other) = default;
-ActivityUserData::TypedValue::~TypedValue() {}
+ActivityUserData::TypedValue::~TypedValue() = default;
 
 StringPiece ActivityUserData::TypedValue::Get() const {
   DCHECK_EQ(RAW_VALUE, type_);
@@ -323,13 +326,13 @@ StringPiece ActivityUserData::TypedValue::GetStringReference() const {
 // These are required because std::atomic is (currently) not a POD type and
 // thus clang requires explicit out-of-line constructors and destructors even
 // when they do nothing.
-ActivityUserData::ValueInfo::ValueInfo() {}
+ActivityUserData::ValueInfo::ValueInfo() = default;
 ActivityUserData::ValueInfo::ValueInfo(ValueInfo&&) = default;
-ActivityUserData::ValueInfo::~ValueInfo() {}
-ActivityUserData::MemoryHeader::MemoryHeader() {}
-ActivityUserData::MemoryHeader::~MemoryHeader() {}
-ActivityUserData::FieldHeader::FieldHeader() {}
-ActivityUserData::FieldHeader::~FieldHeader() {}
+ActivityUserData::ValueInfo::~ValueInfo() = default;
+ActivityUserData::MemoryHeader::MemoryHeader() = default;
+ActivityUserData::MemoryHeader::~MemoryHeader() = default;
+ActivityUserData::FieldHeader::FieldHeader() = default;
+ActivityUserData::FieldHeader::~FieldHeader() = default;
 
 ActivityUserData::ActivityUserData() : ActivityUserData(nullptr, 0, -1) {}
 
@@ -362,7 +365,7 @@ ActivityUserData::ActivityUserData(void* memory, size_t size, int64_t pid)
   ImportExistingData();
 }
 
-ActivityUserData::~ActivityUserData() {}
+ActivityUserData::~ActivityUserData() = default;
 
 bool ActivityUserData::CreateSnapshot(Snapshot* output_snapshot) const {
   DCHECK(output_snapshot);
@@ -638,13 +641,11 @@ struct ThreadActivityTracker::Header {
 
   // A memory location used to indicate if changes have been made to the data
   // that would invalidate an in-progress read of its contents. The active
-  // tracker will zero the value whenever something gets popped from the
-  // stack. A monitoring tracker can write a non-zero value here, copy the
-  // stack contents, and read the value to know, if it is still non-zero, that
-  // the contents didn't change while being copied. This can handle concurrent
-  // snapshot operations only if each snapshot writes a different bit (which
-  // is not the current implementation so no parallel snapshots allowed).
-  std::atomic<uint32_t> data_unchanged;
+  // tracker will increment the value whenever something gets popped from the
+  // stack. A monitoring tracker can check the value before and after access
+  // to know, if it's still the same, that the contents didn't change while
+  // being copied.
+  std::atomic<uint32_t> data_version;
 
   // The last "exception" activity. This can't be stored on the stack because
   // that could get popped as things unwind.
@@ -657,8 +658,8 @@ struct ThreadActivityTracker::Header {
   char thread_name[32];
 };
 
-ThreadActivityTracker::Snapshot::Snapshot() {}
-ThreadActivityTracker::Snapshot::~Snapshot() {}
+ThreadActivityTracker::Snapshot::Snapshot() = default;
+ThreadActivityTracker::Snapshot::~Snapshot() = default;
 
 ThreadActivityTracker::ScopedActivity::ScopedActivity(
     ThreadActivityTracker* tracker,
@@ -687,10 +688,11 @@ ThreadActivityTracker::ThreadActivityTracker(void* base, size_t size)
     : header_(static_cast<Header*>(base)),
       stack_(reinterpret_cast<Activity*>(reinterpret_cast<char*>(base) +
                                          sizeof(Header))),
+#if DCHECK_IS_ON()
+      thread_id_(PlatformThreadRef()),
+#endif
       stack_slots_(
           static_cast<uint32_t>((size - sizeof(Header)) / sizeof(Activity))) {
-  DCHECK(thread_checker_.CalledOnValidThread());
-
   // Verify the parameters but fail gracefully if they're not valid so that
   // production code based on external inputs will not crash.  IsValid() will
   // return false in this case.
@@ -726,7 +728,7 @@ ThreadActivityTracker::ThreadActivityTracker(void* base, size_t size)
     DCHECK_EQ(0, header_->start_ticks);
     DCHECK_EQ(0U, header_->stack_slots);
     DCHECK_EQ(0U, header_->current_depth.load(std::memory_order_relaxed));
-    DCHECK_EQ(0U, header_->data_unchanged.load(std::memory_order_relaxed));
+    DCHECK_EQ(0U, header_->data_version.load(std::memory_order_relaxed));
     DCHECK_EQ(0, stack_[0].time_internal);
     DCHECK_EQ(0U, stack_[0].origin_address);
     DCHECK_EQ(0U, stack_[0].call_stack[0]);
@@ -734,7 +736,7 @@ ThreadActivityTracker::ThreadActivityTracker(void* base, size_t size)
 
 #if defined(OS_WIN)
     header_->thread_ref.as_tid = PlatformThread::CurrentId();
-#elif defined(OS_POSIX)
+#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
     header_->thread_ref.as_handle =
         PlatformThread::CurrentHandle().platform_handle();
 #endif
@@ -758,7 +760,7 @@ ThreadActivityTracker::ThreadActivityTracker(void* base, size_t size)
   }
 }
 
-ThreadActivityTracker::~ThreadActivityTracker() {}
+ThreadActivityTracker::~ThreadActivityTracker() = default;
 
 ThreadActivityTracker::ActivityId ThreadActivityTracker::PushActivity(
     const void* program_counter,
@@ -767,8 +769,7 @@ ThreadActivityTracker::ActivityId ThreadActivityTracker::PushActivity(
     const ActivityData& data) {
   // A thread-checker creates a lock to check the thread-id which means
   // re-entry into this code if lock acquisitions are being tracked.
-  DCHECK(type == Activity::ACT_LOCK_ACQUIRE ||
-         thread_checker_.CalledOnValidThread());
+  DCHECK(type == Activity::ACT_LOCK_ACQUIRE || CalledOnValidThread());
 
   // Get the current depth of the stack. No access to other memory guarded
   // by this variable is done here so a "relaxed" load is acceptable.
@@ -801,7 +802,7 @@ ThreadActivityTracker::ActivityId ThreadActivityTracker::PushActivity(
 void ThreadActivityTracker::ChangeActivity(ActivityId id,
                                            Activity::Type type,
                                            const ActivityData& data) {
-  DCHECK(thread_checker_.CalledOnValidThread());
+  DCHECK(CalledOnValidThread());
   DCHECK(type != Activity::ACT_NULL || &data != &kNullActivityData);
   DCHECK_LT(id, header_->current_depth.load(std::memory_order_acquire));
 
@@ -836,15 +837,14 @@ void ThreadActivityTracker::PopActivity(ActivityId id) {
   // A thread-checker creates a lock to check the thread-id which means
   // re-entry into this code if lock acquisitions are being tracked.
   DCHECK(stack_[depth].activity_type == Activity::ACT_LOCK_ACQUIRE ||
-         thread_checker_.CalledOnValidThread());
+         CalledOnValidThread());
 
   // The stack has shrunk meaning that some other thread trying to copy the
-  // contents for reporting purposes could get bad data. That thread would
-  // have written a non-zero value into |data_unchanged|; clearing it here
-  // will let that thread detect that something did change. This needs to
+  // contents for reporting purposes could get bad data. Increment the data
+  // version so that it con tell that things have changed. This needs to
   // happen after the atomic |depth| operation above so a "release" store
   // is required.
-  header_->data_unchanged.store(0, std::memory_order_release);
+  header_->data_version.fetch_add(1, std::memory_order_release);
 }
 
 std::unique_ptr<ActivityUserData> ThreadActivityTracker::GetUserData(
@@ -853,12 +853,12 @@ std::unique_ptr<ActivityUserData> ThreadActivityTracker::GetUserData(
   // Don't allow user data for lock acquisition as recursion may occur.
   if (stack_[id].activity_type == Activity::ACT_LOCK_ACQUIRE) {
     NOTREACHED();
-    return MakeUnique<ActivityUserData>();
+    return std::make_unique<ActivityUserData>();
   }
 
   // User-data is only stored for activities actually held in the stack.
   if (id >= stack_slots_)
-    return MakeUnique<ActivityUserData>();
+    return std::make_unique<ActivityUserData>();
 
   // Create and return a real UserData object.
   return CreateUserDataForActivity(&stack_[id], allocator);
@@ -885,7 +885,7 @@ void ThreadActivityTracker::RecordExceptionActivity(const void* program_counter,
                                                     const ActivityData& data) {
   // A thread-checker creates a lock to check the thread-id which means
   // re-entry into this code if lock acquisitions are being tracked.
-  DCHECK(thread_checker_.CalledOnValidThread());
+  DCHECK(CalledOnValidThread());
 
   // Fill the reusable exception activity.
   Activity::FillFrom(&header_->last_exception, program_counter, origin, type,
@@ -893,7 +893,7 @@ void ThreadActivityTracker::RecordExceptionActivity(const void* program_counter,
 
   // The data has changed meaning that some other thread trying to copy the
   // contents for reporting purposes could get bad data.
-  header_->data_unchanged.store(0, std::memory_order_relaxed);
+  header_->data_version.fetch_add(1, std::memory_order_relaxed);
 }
 
 bool ThreadActivityTracker::IsValid() const {
@@ -939,12 +939,13 @@ bool ThreadActivityTracker::CreateSnapshot(Snapshot* output_snapshot) const {
     const int64_t starting_process_id = header_->owner.process_id;
     const int64_t starting_thread_id = header_->thread_ref.as_id;
 
-    // Write a non-zero value to |data_unchanged| so it's possible to detect
-    // at the end that nothing has changed since copying the data began. A
-    // "cst" operation is required to ensure it occurs before everything else.
-    // Using "cst" memory ordering is relatively expensive but this is only
-    // done during analysis so doesn't directly affect the worker threads.
-    header_->data_unchanged.store(1, std::memory_order_seq_cst);
+    // Note the current |data_version| so it's possible to detect at the end
+    // that nothing has changed since copying the data began. A "cst" operation
+    // is required to ensure it occurs before everything else. Using "cst"
+    // memory ordering is relatively expensive but this is only done during
+    // analysis so doesn't directly affect the worker threads.
+    const uint32_t pre_version =
+        header_->data_version.load(std::memory_order_seq_cst);
 
     // Fetching the current depth also "acquires" the contents of the stack.
     depth = header_->current_depth.load(std::memory_order_acquire);
@@ -964,7 +965,7 @@ bool ThreadActivityTracker::CreateSnapshot(Snapshot* output_snapshot) const {
 
     // Retry if something changed during the copy. A "cst" operation ensures
     // it must happen after all the above operations.
-    if (!header_->data_unchanged.load(std::memory_order_seq_cst))
+    if (header_->data_version.load(std::memory_order_seq_cst) != pre_version)
       continue;
 
     // Stack copied. Record it's full depth.
@@ -1024,6 +1025,10 @@ const void* ThreadActivityTracker::GetBaseAddress() {
   return header_;
 }
 
+uint32_t ThreadActivityTracker::GetDataVersionForTesting() {
+  return header_->data_version.load(std::memory_order_relaxed);
+}
+
 void ThreadActivityTracker::SetOwningProcessIdForTesting(int64_t pid,
                                                          int64_t stamp) {
   header_->owner.SetOwningProcessIdForTesting(pid, stamp);
@@ -1042,6 +1047,14 @@ size_t ThreadActivityTracker::SizeForStackDepth(int stack_depth) {
   return static_cast<size_t>(stack_depth) * sizeof(Activity) + sizeof(Header);
 }
 
+bool ThreadActivityTracker::CalledOnValidThread() {
+#if DCHECK_IS_ON()
+  return thread_id_ == PlatformThreadRef();
+#else
+  return true;
+#endif
+}
+
 std::unique_ptr<ActivityUserData>
 ThreadActivityTracker::CreateUserDataForActivity(
     Activity* activity,
@@ -1052,14 +1065,14 @@ ThreadActivityTracker::CreateUserDataForActivity(
   void* memory = allocator->GetAsArray<char>(ref, kUserDataSize);
   if (memory) {
     std::unique_ptr<ActivityUserData> user_data =
-        MakeUnique<ActivityUserData>(memory, kUserDataSize);
+        std::make_unique<ActivityUserData>(memory, kUserDataSize);
     activity->user_data_ref = ref;
     activity->user_data_id = user_data->id();
     return user_data;
   }
 
   // Return a dummy object that will still accept (but ignore) Set() calls.
-  return MakeUnique<ActivityUserData>();
+  return std::make_unique<ActivityUserData>();
 }
 
 // The instantiation of the GlobalActivityTracker object.
@@ -1071,18 +1084,18 @@ ThreadActivityTracker::CreateUserDataForActivity(
 // of std::atomic because the latter can create global ctors and dtors.
 subtle::AtomicWord GlobalActivityTracker::g_tracker_ = 0;
 
-GlobalActivityTracker::ModuleInfo::ModuleInfo() {}
+GlobalActivityTracker::ModuleInfo::ModuleInfo() = default;
 GlobalActivityTracker::ModuleInfo::ModuleInfo(ModuleInfo&& rhs) = default;
 GlobalActivityTracker::ModuleInfo::ModuleInfo(const ModuleInfo& rhs) = default;
-GlobalActivityTracker::ModuleInfo::~ModuleInfo() {}
+GlobalActivityTracker::ModuleInfo::~ModuleInfo() = default;
 
 GlobalActivityTracker::ModuleInfo& GlobalActivityTracker::ModuleInfo::operator=(
     ModuleInfo&& rhs) = default;
 GlobalActivityTracker::ModuleInfo& GlobalActivityTracker::ModuleInfo::operator=(
     const ModuleInfo& rhs) = default;
 
-GlobalActivityTracker::ModuleInfoRecord::ModuleInfoRecord() {}
-GlobalActivityTracker::ModuleInfoRecord::~ModuleInfoRecord() {}
+GlobalActivityTracker::ModuleInfoRecord::ModuleInfoRecord() = default;
+GlobalActivityTracker::ModuleInfoRecord::~ModuleInfoRecord() = default;
 
 bool GlobalActivityTracker::ModuleInfoRecord::DecodeTo(
     GlobalActivityTracker::ModuleInfo* info,
@@ -1119,36 +1132,35 @@ bool GlobalActivityTracker::ModuleInfoRecord::DecodeTo(
   return iter.ReadString(&info->file) && iter.ReadString(&info->debug_file);
 }
 
-bool GlobalActivityTracker::ModuleInfoRecord::EncodeFrom(
+GlobalActivityTracker::ModuleInfoRecord*
+GlobalActivityTracker::ModuleInfoRecord::CreateFrom(
     const GlobalActivityTracker::ModuleInfo& info,
-    size_t record_size) {
+    PersistentMemoryAllocator* allocator) {
   Pickle pickler;
-  bool okay =
-      pickler.WriteString(info.file) && pickler.WriteString(info.debug_file);
-  if (!okay) {
-    NOTREACHED();
-    return false;
-  }
-  if (offsetof(ModuleInfoRecord, pickle) + pickler.size() > record_size) {
-    NOTREACHED();
-    return false;
-  }
+  pickler.WriteString(info.file);
+  pickler.WriteString(info.debug_file);
+  size_t required_size = offsetof(ModuleInfoRecord, pickle) + pickler.size();
+  ModuleInfoRecord* record = allocator->New<ModuleInfoRecord>(required_size);
+  if (!record)
+    return nullptr;
 
   // These fields never changes and are done before the record is made
   // iterable so no thread protection is necessary.
-  size = info.size;
-  timestamp = info.timestamp;
-  age = info.age;
-  memcpy(identifier, info.identifier, sizeof(identifier));
-  memcpy(pickle, pickler.data(), pickler.size());
-  pickle_size = pickler.size();
-  changes.store(0, std::memory_order_relaxed);
+  record->size = info.size;
+  record->timestamp = info.timestamp;
+  record->age = info.age;
+  memcpy(record->identifier, info.identifier, sizeof(identifier));
+  memcpy(record->pickle, pickler.data(), pickler.size());
+  record->pickle_size = pickler.size();
+  record->changes.store(0, std::memory_order_relaxed);
 
   // Initialize the owner info.
-  owner.Release_Initialize();
+  record->owner.Release_Initialize();
 
   // Now set those fields that can change.
-  return UpdateFrom(info);
+  bool success = record->UpdateFrom(info);
+  DCHECK(success);
+  return record;
 }
 
 bool GlobalActivityTracker::ModuleInfoRecord::UpdateFrom(
@@ -1176,17 +1188,6 @@ bool GlobalActivityTracker::ModuleInfoRecord::UpdateFrom(
   return true;
 }
 
-// static
-size_t GlobalActivityTracker::ModuleInfoRecord::EncodedSize(
-    const GlobalActivityTracker::ModuleInfo& info) {
-  PickleSizer sizer;
-  sizer.AddString(info.file);
-  sizer.AddString(info.debug_file);
-
-  return offsetof(ModuleInfoRecord, pickle) + sizeof(Pickle::Header) +
-         sizer.payload_size();
-}
-
 GlobalActivityTracker::ScopedThreadActivity::ScopedThreadActivity(
     const void* program_counter,
     const void* origin,
@@ -1215,7 +1216,7 @@ ActivityUserData& GlobalActivityTracker::ScopedThreadActivity::user_data() {
       user_data_ =
           tracker_->GetUserData(activity_id_, &global->user_data_allocator_);
     } else {
-      user_data_ = MakeUnique<ActivityUserData>();
+      user_data_ = std::make_unique<ActivityUserData>();
     }
   }
   return *user_data_;
@@ -1226,7 +1227,7 @@ GlobalActivityTracker::ThreadSafeUserData::ThreadSafeUserData(void* memory,
                                                               int64_t pid)
     : ActivityUserData(memory, size, pid) {}
 
-GlobalActivityTracker::ThreadSafeUserData::~ThreadSafeUserData() {}
+GlobalActivityTracker::ThreadSafeUserData::~ThreadSafeUserData() = default;
 
 void GlobalActivityTracker::ThreadSafeUserData::Set(StringPiece name,
                                                     ValueType type,
@@ -1265,7 +1266,7 @@ void GlobalActivityTracker::CreateWithAllocator(
 
 #if !defined(OS_NACL)
 // static
-void GlobalActivityTracker::CreateWithFile(const FilePath& file_path,
+bool GlobalActivityTracker::CreateWithFile(const FilePath& file_path,
                                            size_t size,
                                            uint64_t id,
                                            StringPiece name,
@@ -1275,28 +1276,48 @@ void GlobalActivityTracker::CreateWithFile(const FilePath& file_path,
 
   // Create and map the file into memory and make it globally available.
   std::unique_ptr<MemoryMappedFile> mapped_file(new MemoryMappedFile());
-  bool success =
-      mapped_file->Initialize(File(file_path,
-                                   File::FLAG_CREATE_ALWAYS | File::FLAG_READ |
-                                   File::FLAG_WRITE | File::FLAG_SHARE_DELETE),
-                              {0, static_cast<int64_t>(size)},
-                              MemoryMappedFile::READ_WRITE_EXTEND);
-  DCHECK(success);
-  CreateWithAllocator(MakeUnique<FilePersistentMemoryAllocator>(
+  bool success = mapped_file->Initialize(
+      File(file_path, File::FLAG_CREATE_ALWAYS | File::FLAG_READ |
+                          File::FLAG_WRITE | File::FLAG_SHARE_DELETE),
+      {0, size}, MemoryMappedFile::READ_WRITE_EXTEND);
+  if (!success)
+    return false;
+  if (!FilePersistentMemoryAllocator::IsFileAcceptable(*mapped_file, false))
+    return false;
+  CreateWithAllocator(std::make_unique<FilePersistentMemoryAllocator>(
                           std::move(mapped_file), size, id, name, false),
                       stack_depth, 0);
+  return true;
 }
 #endif  // !defined(OS_NACL)
 
 // static
-void GlobalActivityTracker::CreateWithLocalMemory(size_t size,
+bool GlobalActivityTracker::CreateWithLocalMemory(size_t size,
                                                   uint64_t id,
                                                   StringPiece name,
                                                   int stack_depth,
                                                   int64_t process_id) {
   CreateWithAllocator(
-      MakeUnique<LocalPersistentMemoryAllocator>(size, id, name), stack_depth,
-      process_id);
+      std::make_unique<LocalPersistentMemoryAllocator>(size, id, name),
+      stack_depth, process_id);
+  return true;
+}
+
+// static
+bool GlobalActivityTracker::CreateWithSharedMemory(
+    base::WritableSharedMemoryMapping mapping,
+    uint64_t id,
+    StringPiece name,
+    int stack_depth) {
+  if (!mapping.IsValid() ||
+      !WritableSharedPersistentMemoryAllocator::IsSharedMemoryAcceptable(
+          mapping)) {
+    return false;
+  }
+  CreateWithAllocator(std::make_unique<WritableSharedPersistentMemoryAllocator>(
+                          std::move(mapping), id, name),
+                      stack_depth, 0);
+  return true;
 }
 
 // static
@@ -1324,6 +1345,10 @@ GlobalActivityTracker::ReleaseForTesting() {
 }
 
 ThreadActivityTracker* GlobalActivityTracker::CreateTrackerForCurrentThread() {
+  // It is not safe to use TLS once TLS has been destroyed.
+  if (base::ThreadLocalStorage::HasBeenDestroyed())
+    return nullptr;
+
   DCHECK(!this_thread_tracker_.Get());
 
   PersistentMemoryAllocator::Reference mem_reference;
@@ -1338,23 +1363,14 @@ ThreadActivityTracker* GlobalActivityTracker::CreateTrackerForCurrentThread() {
     // because the underlying allocator wasn't given enough memory to satisfy
     // to all possible requests.
     NOTREACHED();
-    // Report the thread-count at which the allocator was full so that the
-    // failure can be seen and underlying memory resized appropriately.
-    UMA_HISTOGRAM_COUNTS_1000(
-        "ActivityTracker.ThreadTrackers.MemLimitTrackerCount",
-        thread_tracker_count_.load(std::memory_order_relaxed));
+
     // Return null, just as if tracking wasn't enabled.
     return nullptr;
   }
 
   // Convert the memory block found above into an actual memory address.
   // Doing the conversion as a Header object enacts the 32/64-bit size
-  // consistency checks which would not otherwise be done. Unfortunately,
-  // some older compilers and MSVC don't have standard-conforming definitions
-  // of std::atomic which cause it not to be plain-old-data. Don't check on
-  // those platforms assuming that the checks on other platforms will be
-  // sufficient.
-  // TODO(bcwhite): Review this after major compiler releases.
+  // consistency checks which would not otherwise be done.
   DCHECK(mem_reference);
   void* mem_base;
   mem_base =
@@ -1365,24 +1381,18 @@ ThreadActivityTracker* GlobalActivityTracker::CreateTrackerForCurrentThread() {
 
   // Create a tracker with the acquired memory and set it as the tracker
   // for this particular thread in thread-local-storage.
-  ManagedActivityTracker* tracker =
-      new ManagedActivityTracker(mem_reference, mem_base, stack_memory_size_);
+  auto tracker = std::make_unique<ManagedActivityTracker>(
+      mem_reference, mem_base, stack_memory_size_);
   DCHECK(tracker->IsValid());
-  this_thread_tracker_.Set(tracker);
-  int old_count = thread_tracker_count_.fetch_add(1, std::memory_order_relaxed);
-
-  UMA_HISTOGRAM_ENUMERATION("ActivityTracker.ThreadTrackers.Count",
-                            old_count + 1, kMaxThreadCount);
-  return tracker;
+  auto* tracker_raw = tracker.get();
+  this_thread_tracker_.Set(std::move(tracker));
+  thread_tracker_count_.fetch_add(1, std::memory_order_relaxed);
+  return tracker_raw;
 }
 
 void GlobalActivityTracker::ReleaseTrackerForCurrentThreadForTesting() {
-  ThreadActivityTracker* tracker =
-      reinterpret_cast<ThreadActivityTracker*>(this_thread_tracker_.Get());
-  if (tracker) {
+  if (this_thread_tracker_.Get())
     this_thread_tracker_.Set(nullptr);
-    delete tracker;
-  }
 }
 
 void GlobalActivityTracker::SetBackgroundTaskRunner(
@@ -1405,11 +1415,12 @@ void GlobalActivityTracker::RecordProcessLaunch(
   DCHECK_NE(0, pid);
 
   base::AutoLock lock(global_tracker_lock_);
-  if (base::ContainsKey(known_processes_, pid)) {
+  if (base::Contains(known_processes_, pid)) {
     // TODO(bcwhite): Measure this in UMA.
     NOTREACHED() << "Process #" << process_id
                  << " was previously recorded as \"launched\""
-                 << " with no corresponding exit.";
+                 << " with no corresponding exit.\n"
+                 << known_processes_[pid];
     known_processes_.erase(pid);
   }
 
@@ -1424,12 +1435,12 @@ void GlobalActivityTracker::RecordProcessLaunch(
     ProcessId process_id,
     const FilePath::StringType& exe,
     const FilePath::StringType& args) {
-  const int64_t pid = process_id;
   if (exe.find(FILE_PATH_LITERAL(" "))) {
-    RecordProcessLaunch(pid, FilePath::StringType(FILE_PATH_LITERAL("\"")) +
-                                 exe + FILE_PATH_LITERAL("\" ") + args);
+    RecordProcessLaunch(process_id,
+                        FilePath::StringType(FILE_PATH_LITERAL("\"")) + exe +
+                            FILE_PATH_LITERAL("\" ") + args);
   } else {
-    RecordProcessLaunch(pid, exe + FILE_PATH_LITERAL(' ') + args);
+    RecordProcessLaunch(process_id, exe + FILE_PATH_LITERAL(' ') + args);
   }
 }
 
@@ -1459,11 +1470,11 @@ void GlobalActivityTracker::RecordProcessExit(ProcessId process_id,
 
   // The persistent allocator is thread-safe so run the iteration and
   // adjustments on a worker thread if one was provided.
-  if (task_runner && !task_runner->RunsTasksOnCurrentThread()) {
+  if (task_runner && !task_runner->RunsTasksInCurrentSequence()) {
     task_runner->PostTask(
         FROM_HERE,
         BindOnce(&GlobalActivityTracker::CleanupAfterProcess, Unretained(this),
-                 pid, now_stamp, exit_code, Passed(&command_line)));
+                 pid, now_stamp, exit_code, std::move(command_line)));
     return;
   }
 
@@ -1498,6 +1509,8 @@ void GlobalActivityTracker::CleanupAfterProcess(int64_t process_id,
     while ((ref = iter.GetNextOfType(kTypeIdProcessDataRecord)) != 0) {
       const void* memory = allocator_->GetAsArray<char>(
           ref, kTypeIdProcessDataRecord, PersistentMemoryAllocator::kSizeAny);
+      if (!memory)
+        continue;
       int64_t found_id;
       int64_t create_stamp;
       if (ActivityUserData::GetOwningProcessId(memory, &found_id,
@@ -1535,6 +1548,8 @@ void GlobalActivityTracker::CleanupAfterProcess(int64_t process_id,
       case ModuleInfoRecord::kPersistentTypeId: {
         const void* memory = allocator_->GetAsArray<char>(
             ref, type, PersistentMemoryAllocator::kSizeAny);
+        if (!memory)
+          continue;
         int64_t found_id;
         int64_t create_stamp;
 
@@ -1584,15 +1599,12 @@ void GlobalActivityTracker::RecordModuleInfo(const ModuleInfo& info) {
     return;
   }
 
-  size_t required_size = ModuleInfoRecord::EncodedSize(info);
-  ModuleInfoRecord* record = allocator_->New<ModuleInfoRecord>(required_size);
+  ModuleInfoRecord* record =
+      ModuleInfoRecord::CreateFrom(info, allocator_.get());
   if (!record)
     return;
-
-  bool success = record->EncodeFrom(info, required_size);
-  DCHECK(success);
   allocator_->MakeIterable(record);
-  modules_.insert(std::make_pair(info.file, record));
+  modules_.emplace(info.file, record);
 }
 
 void GlobalActivityTracker::RecordFieldTrial(const std::string& trial_name,
@@ -1618,7 +1630,6 @@ GlobalActivityTracker::GlobalActivityTracker(
     : allocator_(std::move(allocator)),
       stack_memory_size_(ThreadActivityTracker::SizeForStackDepth(stack_depth)),
       process_id_(process_id == 0 ? GetCurrentProcId() : process_id),
-      this_thread_tracker_(&OnTLSDestroy),
       thread_tracker_count_(0),
       thread_tracker_allocator_(allocator_.get(),
                                 kTypeIdActivityTracker,
@@ -1700,11 +1711,6 @@ void GlobalActivityTracker::RecordExceptionImpl(const void* pc,
                                    ActivityData::ForException(code));
 }
 
-// static
-void GlobalActivityTracker::OnTLSDestroy(void* value) {
-  delete reinterpret_cast<ManagedActivityTracker*>(value);
-}
-
 ScopedActivity::ScopedActivity(const void* program_counter,
                                uint8_t action,
                                uint32_t id,
