@@ -5,12 +5,12 @@
 #ifndef BASE_OPTIONAL_H_
 #define BASE_OPTIONAL_H_
 
+#include <functional>
 #include <type_traits>
 #include <utility>
 
 #include "base/logging.h"
 #include "base/template_util.h"
-#include "base/thread_annotations.h"
 
 namespace base {
 
@@ -270,9 +270,7 @@ class OptionalBase {
       storage_.Init(std::forward<U>(value));
   }
 
-  // TODO(lukasza): Figure out how to remove the NO_THREAD_SAFETY_ANALYSIS
-  // annotation below.  See https://crbug.com/881875#c1 for details.
-  void FreeIfNeeded() NO_THREAD_SAFETY_ANALYSIS {
+  void FreeIfNeeded() {
     if (!storage_.is_populated_)
       return;
     storage_.value_.~T();
@@ -429,6 +427,28 @@ class OPTIONAL_DECLSPEC_EMPTY_BASES Optional
                                       std::is_copy_assignable<T>::value>,
       public internal::MoveAssignable<std::is_move_constructible<T>::value &&
                                       std::is_move_assignable<T>::value> {
+ private:
+  // Disable some versions of T that are ill-formed.
+  // See: https://timsong-cpp.github.io/cppwp/n4659/optional#syn-1
+  static_assert(
+      !std::is_same<internal::RemoveCvRefT<T>, in_place_t>::value,
+      "instantiation of base::Optional with in_place_t is ill-formed");
+  static_assert(!std::is_same<internal::RemoveCvRefT<T>, nullopt_t>::value,
+                "instantiation of base::Optional with nullopt_t is ill-formed");
+  static_assert(
+      !std::is_reference<T>::value,
+      "instantiation of base::Optional with a reference type is ill-formed");
+  // See: https://timsong-cpp.github.io/cppwp/n4659/optional#optional-3
+  static_assert(std::is_destructible<T>::value,
+                "instantiation of base::Optional with a non-destructible type "
+                "is ill-formed");
+  // Arrays are explicitly disallowed because for arrays of known bound
+  // is_destructible is of undefined value.
+  // See: https://en.cppreference.com/w/cpp/types/is_destructible
+  static_assert(
+      !std::is_array<T>::value,
+      "instantiation of base::Optional with an array type is ill-formed");
+
  public:
 #undef OPTIONAL_DECLSPEC_EMPTY_BASES
   using value_type = T;
@@ -570,32 +590,32 @@ class OPTIONAL_DECLSPEC_EMPTY_BASES Optional
   }
 
   constexpr const T* operator->() const {
-    DCHECK(storage_.is_populated_);
+    CHECK(storage_.is_populated_);
     return &storage_.value_;
   }
 
   constexpr T* operator->() {
-    DCHECK(storage_.is_populated_);
+    CHECK(storage_.is_populated_);
     return &storage_.value_;
   }
 
   constexpr const T& operator*() const & {
-    DCHECK(storage_.is_populated_);
+    CHECK(storage_.is_populated_);
     return storage_.value_;
   }
 
   constexpr T& operator*() & {
-    DCHECK(storage_.is_populated_);
+    CHECK(storage_.is_populated_);
     return storage_.value_;
   }
 
   constexpr const T&& operator*() const && {
-    DCHECK(storage_.is_populated_);
+    CHECK(storage_.is_populated_);
     return std::move(storage_.value_);
   }
 
   constexpr T&& operator*() && {
-    DCHECK(storage_.is_populated_);
+    CHECK(storage_.is_populated_);
     return std::move(storage_.value_);
   }
 
@@ -890,6 +910,11 @@ constexpr Optional<T> make_optional(std::initializer_list<U> il,
   return Optional<T>(in_place, il, std::forward<Args>(args)...);
 }
 
+template<typename T>
+constexpr Optional<T> pass_optional(const Optional<T> &opt) {
+  return opt ? make_optional(*opt) : nullopt;
+}
+
 // Partial specialization for a function template is not allowed. Also, it is
 // not allowed to add overload function to std namespace, while it is allowed
 // to specialize the template in std. Thus, swap() (kind of) overloading is
