@@ -34,6 +34,22 @@ ScrollOffsetAnimationsImpl::~ScrollOffsetAnimationsImpl() {
   animation_host_->RemoveAnimationTimeline(scroll_offset_timeline_.get());
 }
 
+void ScrollOffsetAnimationsImpl::AutoScrollAnimationCreate(
+    ElementId element_id,
+    const gfx::ScrollOffset& target_offset,
+    const gfx::ScrollOffset& current_offset,
+    float autoscroll_velocity,
+    base::TimeDelta animation_start_offset) {
+  std::unique_ptr<ScrollOffsetAnimationCurve> curve =
+      ScrollOffsetAnimationCurve::Create(
+          target_offset, LinearTimingFunction::Create(),
+          ScrollOffsetAnimationCurve::DurationBehavior::CONSTANT_VELOCITY);
+  curve->SetInitialValue(current_offset, base::TimeDelta(),
+                         autoscroll_velocity);
+  ScrollAnimationCreateInternal(element_id, std::move(curve),
+                                animation_start_offset);
+}
+
 void ScrollOffsetAnimationsImpl::ScrollAnimationCreate(
     ElementId element_id,
     const gfx::ScrollOffset& target_offset,
@@ -42,10 +58,19 @@ void ScrollOffsetAnimationsImpl::ScrollAnimationCreate(
     base::TimeDelta animation_start_offset) {
   std::unique_ptr<ScrollOffsetAnimationCurve> curve =
       ScrollOffsetAnimationCurve::Create(
-          target_offset, CubicBezierTimingFunction::CreatePreset(
-                             CubicBezierTimingFunction::EaseType::EASE_OUT_NATURAL),
+          target_offset,
+          CubicBezierTimingFunction::CreatePreset(
+              CubicBezierTimingFunction::EaseType::EASE_OUT_NATURAL),
           ScrollOffsetAnimationCurve::DurationBehavior::INVERSE_DELTA);
   curve->SetInitialValue(current_offset, delayed_by);
+  ScrollAnimationCreateInternal(element_id, std::move(curve),
+                                animation_start_offset);
+}
+
+void ScrollOffsetAnimationsImpl::ScrollAnimationCreateInternal(
+    ElementId element_id,
+    std::unique_ptr<AnimationCurve> curve,
+    base::TimeDelta animation_start_offset) {
   TRACE_EVENT_INSTANT1("cc", "ScrollAnimationCreate", TRACE_EVENT_SCOPE_THREAD,
                        "Duration", curve->Duration().InMillisecondsF());
 
@@ -59,7 +84,6 @@ void ScrollOffsetAnimationsImpl::ScrollAnimationCreate(
   DCHECK(scroll_offset_animation_->animation_timeline());
 
   ReattachScrollOffsetAnimationIfNeeded(element_id);
-
   scroll_offset_animation_->AddKeyframeModel(std::move(keyframe_model));
 }
 
@@ -183,6 +207,29 @@ void ScrollOffsetAnimationsImpl::NotifyAnimationFinished(
                        TRACE_EVENT_SCOPE_THREAD);
 }
 
+bool ScrollOffsetAnimationsImpl::IsAnimating() const {
+  if (!scroll_offset_animation_->has_element_animations())
+    return false;
+
+  KeyframeModel* keyframe_model =
+      scroll_offset_animation_->GetKeyframeModel(TargetProperty::SCROLL_OFFSET);
+  if (!keyframe_model)
+    return false;
+
+  switch (keyframe_model->run_state()) {
+    case KeyframeModel::WAITING_FOR_TARGET_AVAILABILITY:
+    case KeyframeModel::STARTING:
+    case KeyframeModel::RUNNING:
+    case KeyframeModel::PAUSED:
+      return true;
+    case KeyframeModel::WAITING_FOR_DELETION:
+    case KeyframeModel::FINISHED:
+    case KeyframeModel::ABORTED:
+    case KeyframeModel::ABORTED_BUT_NEEDS_COMPLETION:
+      return false;
+  }
+}
+
 void ScrollOffsetAnimationsImpl::ReattachScrollOffsetAnimationIfNeeded(
     ElementId element_id) {
   if (scroll_offset_animation_->element_id() != element_id) {
