@@ -4,10 +4,11 @@
 
 #include "cc/layers/layer_impl.h"
 
-#include "cc/base/filter_operation.h"
-#include "cc/base/filter_operations.h"
+#include "base/stl_util.h"
 #include "cc/layers/painted_scrollbar_layer_impl.h"
 #include "cc/layers/solid_color_scrollbar_layer_impl.h"
+#include "cc/paint/filter_operation.h"
+#include "cc/paint/filter_operations.h"
 #include "cc/test/animation_test_common.h"
 #include "cc/test/fake_impl_task_runner_provider.h"
 #include "cc/test/fake_layer_tree_frame_sink.h"
@@ -15,7 +16,6 @@
 #include "cc/test/geometry_test_utils.h"
 #include "cc/test/test_task_graph_runner.h"
 #include "cc/trees/layer_tree_impl.h"
-#include "cc/trees/mutable_properties.h"
 #include "cc/trees/single_thread_proxy.h"
 #include "cc/trees/tree_synchronizer.h"
 #include "testing/gmock/include/gmock/gmock.h"
@@ -25,72 +25,34 @@
 namespace cc {
 namespace {
 
-#define EXECUTE_AND_VERIFY_SUBTREE_CHANGED(code_to_test)                    \
-  root->layer_tree_impl()->ResetAllChangeTracking();                        \
-  code_to_test;                                                             \
-  EXPECT_TRUE(                                                              \
-      root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting(root));   \
-  EXPECT_FALSE(                                                             \
-      root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting(child));  \
-  EXPECT_FALSE(root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting( \
-      grand_child));                                                        \
-  EXPECT_TRUE(root->LayerPropertyChanged());                                \
-  EXPECT_TRUE(child->LayerPropertyChanged());                               \
-  EXPECT_TRUE(grand_child->LayerPropertyChanged());
-
-#define EXECUTE_AND_VERIFY_SUBTREE_DID_NOT_CHANGE(code_to_test)             \
-  root->layer_tree_impl()->ResetAllChangeTracking();                        \
-  code_to_test;                                                             \
-  EXPECT_FALSE(                                                             \
-      root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting(root));   \
-  EXPECT_FALSE(                                                             \
-      root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting(child));  \
-  EXPECT_FALSE(root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting( \
-      grand_child));                                                        \
-  EXPECT_FALSE(root->LayerPropertyChanged());                               \
-  EXPECT_FALSE(child->LayerPropertyChanged());                              \
+#define EXECUTE_AND_VERIFY_SUBTREE_DID_NOT_CHANGE(code_to_test)           \
+  root->layer_tree_impl()->ResetAllChangeTracking();                      \
+  code_to_test;                                                           \
+  EXPECT_FALSE(root->LayerPropertyChanged());                             \
+  EXPECT_FALSE(child->LayerPropertyChanged());                            \
   EXPECT_FALSE(grand_child->LayerPropertyChanged());
 
-#define EXECUTE_AND_VERIFY_NEEDS_PUSH_PROPERTIES_AND_SUBTREE_DID_NOT_CHANGE( \
-    code_to_test)                                                            \
-  root->layer_tree_impl()->ResetAllChangeTracking();                         \
-  code_to_test;                                                              \
-  EXPECT_TRUE(                                                               \
-      root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting(root));    \
-  EXPECT_FALSE(                                                              \
-      root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting(child));   \
-  EXPECT_FALSE(root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting(  \
-      grand_child));                                                         \
-  EXPECT_FALSE(root->LayerPropertyChanged());                                \
-  EXPECT_FALSE(child->LayerPropertyChanged());                               \
-  EXPECT_FALSE(grand_child->LayerPropertyChanged());
+#define EXECUTE_AND_VERIFY_SUBTREE_CHANGED(code_to_test)             \
+  root->layer_tree_impl()->ResetAllChangeTracking();                 \
+  code_to_test;                                                      \
+  EXPECT_TRUE(root->LayerPropertyChanged());                         \
+  EXPECT_TRUE(root->LayerPropertyChangedFromPropertyTrees());        \
+  EXPECT_FALSE(root->LayerPropertyChangedNotFromPropertyTrees());    \
+  EXPECT_TRUE(child->LayerPropertyChanged());                        \
+  EXPECT_TRUE(child->LayerPropertyChangedFromPropertyTrees());       \
+  EXPECT_FALSE(child->LayerPropertyChangedNotFromPropertyTrees());   \
+  EXPECT_TRUE(grand_child->LayerPropertyChanged());                  \
+  EXPECT_TRUE(grand_child->LayerPropertyChangedFromPropertyTrees()); \
+  EXPECT_FALSE(grand_child->LayerPropertyChangedNotFromPropertyTrees());
 
-#define EXECUTE_AND_VERIFY_NO_NEED_TO_PUSH_PROPERTIES_AND_SUBTREE_CHANGED(  \
-    code_to_test)                                                           \
-  root->layer_tree_impl()->ResetAllChangeTracking();                        \
-  code_to_test;                                                             \
-  EXPECT_FALSE(                                                             \
-      root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting(root));   \
-  EXPECT_FALSE(                                                             \
-      root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting(child));  \
-  EXPECT_FALSE(root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting( \
-      grand_child));                                                        \
-  EXPECT_TRUE(root->LayerPropertyChanged());                                \
-  EXPECT_TRUE(child->LayerPropertyChanged());                               \
-  EXPECT_TRUE(grand_child->LayerPropertyChanged());
-
-#define EXECUTE_AND_VERIFY_ONLY_LAYER_CHANGED(code_to_test)                 \
-  root->layer_tree_impl()->ResetAllChangeTracking();                        \
-  root->layer_tree_impl()->property_trees()->full_tree_damaged = false;     \
-  code_to_test;                                                             \
-  EXPECT_TRUE(                                                              \
-      root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting(root));   \
-  EXPECT_FALSE(                                                             \
-      root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting(child));  \
-  EXPECT_FALSE(root->layer_tree_impl()->LayerNeedsPushPropertiesForTesting( \
-      grand_child));                                                        \
-  EXPECT_TRUE(root->LayerPropertyChanged());                                \
-  EXPECT_FALSE(child->LayerPropertyChanged());                              \
+#define EXECUTE_AND_VERIFY_ONLY_LAYER_CHANGED(code_to_test)               \
+  root->layer_tree_impl()->ResetAllChangeTracking();                      \
+  root->layer_tree_impl()->property_trees()->full_tree_damaged = false;   \
+  code_to_test;                                                           \
+  EXPECT_TRUE(root->LayerPropertyChanged());                              \
+  EXPECT_FALSE(root->LayerPropertyChangedFromPropertyTrees());            \
+  EXPECT_TRUE(root->LayerPropertyChangedNotFromPropertyTrees());          \
+  EXPECT_FALSE(child->LayerPropertyChanged());                            \
   EXPECT_FALSE(grand_child->LayerPropertyChanged());
 
 #define VERIFY_NEEDS_UPDATE_DRAW_PROPERTIES(code_to_test)                \
@@ -128,7 +90,7 @@ TEST(LayerImplTest, VerifyPendingLayerChangesAreTrackedProperly) {
       FakeLayerTreeFrameSink::Create3d();
   FakeLayerTreeHostImpl host_impl(&task_runner_provider, &task_graph_runner);
   host_impl.SetVisible(true);
-  EXPECT_TRUE(host_impl.InitializeRenderer(layer_tree_frame_sink.get()));
+  EXPECT_TRUE(host_impl.InitializeFrameSink(layer_tree_frame_sink.get()));
   host_impl.CreatePendingTree();
   std::unique_ptr<LayerImpl> root_ptr =
       LayerImpl::Create(host_impl.pending_tree(), 2);
@@ -153,7 +115,6 @@ TEST(LayerImplTest, VerifyPendingLayerChangesAreTrackedProperly) {
   EXPECT_FALSE(child->LayerPropertyChanged());
   EXPECT_FALSE(grand_child->LayerPropertyChanged());
 
-  gfx::PointF arbitrary_point_f = gfx::PointF(0.125f, 0.25f);
   float arbitrary_number = 0.352f;
   gfx::Size arbitrary_size = gfx::Size(111, 222);
   gfx::Point arbitrary_point = gfx::Point(333, 444);
@@ -167,23 +128,21 @@ TEST(LayerImplTest, VerifyPendingLayerChangesAreTrackedProperly) {
 
   // These properties are internal, and should not be considered "change" when
   // they are used.
-  EXECUTE_AND_VERIFY_NEEDS_PUSH_PROPERTIES_AND_SUBTREE_DID_NOT_CHANGE(
+  EXECUTE_AND_VERIFY_SUBTREE_DID_NOT_CHANGE(
       root->SetUpdateRect(arbitrary_rect));
   EXECUTE_AND_VERIFY_ONLY_LAYER_CHANGED(root->SetBounds(arbitrary_size));
   host_impl.pending_tree()->property_trees()->needs_rebuild = true;
   host_impl.pending_tree()->BuildLayerListAndPropertyTreesForTesting();
 
   // Changing these properties affects the entire subtree of layers.
-  EXECUTE_AND_VERIFY_NO_NEED_TO_PUSH_PROPERTIES_AND_SUBTREE_CHANGED(
-      host_impl.pending_tree()->SetFilterMutated(root->element_id(),
-                                                 arbitrary_filters));
-  EXECUTE_AND_VERIFY_NO_NEED_TO_PUSH_PROPERTIES_AND_SUBTREE_CHANGED(
-      host_impl.pending_tree()->SetFilterMutated(root->element_id(),
-                                                 FilterOperations()));
-  EXECUTE_AND_VERIFY_NO_NEED_TO_PUSH_PROPERTIES_AND_SUBTREE_CHANGED(
+  EXECUTE_AND_VERIFY_SUBTREE_CHANGED(host_impl.pending_tree()->SetFilterMutated(
+      root->element_id(), arbitrary_filters));
+  EXECUTE_AND_VERIFY_SUBTREE_CHANGED(host_impl.pending_tree()->SetFilterMutated(
+      root->element_id(), FilterOperations()));
+  EXECUTE_AND_VERIFY_SUBTREE_CHANGED(
       host_impl.pending_tree()->SetOpacityMutated(root->element_id(),
                                                   arbitrary_number));
-  EXECUTE_AND_VERIFY_NO_NEED_TO_PUSH_PROPERTIES_AND_SUBTREE_CHANGED(
+  EXECUTE_AND_VERIFY_SUBTREE_CHANGED(
       host_impl.pending_tree()->SetTransformMutated(root->element_id(),
                                                     arbitrary_transform));
 
@@ -194,17 +153,11 @@ TEST(LayerImplTest, VerifyPendingLayerChangesAreTrackedProperly) {
 
   // Changing these properties does not cause the layer to be marked as changed
   // but does cause the layer to need to push properties.
-  EXECUTE_AND_VERIFY_NEEDS_PUSH_PROPERTIES_AND_SUBTREE_DID_NOT_CHANGE(
-      root->SetElementId(ElementId(2)));
-  EXECUTE_AND_VERIFY_NEEDS_PUSH_PROPERTIES_AND_SUBTREE_DID_NOT_CHANGE(
-      root->SetMutableProperties(MutableProperty::kOpacity);
-      root->SetNeedsPushProperties());
+  EXECUTE_AND_VERIFY_SUBTREE_DID_NOT_CHANGE(root->SetElementId(ElementId(2)));
 
   // After setting all these properties already, setting to the exact same
   // values again should not cause any change.
   EXECUTE_AND_VERIFY_SUBTREE_DID_NOT_CHANGE(root->SetMasksToBounds(true));
-  EXECUTE_AND_VERIFY_SUBTREE_DID_NOT_CHANGE(
-      root->SetPosition(arbitrary_point_f));
   EXECUTE_AND_VERIFY_SUBTREE_DID_NOT_CHANGE(root->SetContentsOpaque(true));
   EXECUTE_AND_VERIFY_SUBTREE_DID_NOT_CHANGE(root->SetDrawsContent(true));
   EXECUTE_AND_VERIFY_SUBTREE_DID_NOT_CHANGE(root->SetBounds(root->bounds()));
@@ -217,7 +170,7 @@ TEST(LayerImplTest, VerifyActiveLayerChangesAreTrackedProperly) {
       FakeLayerTreeFrameSink::Create3d();
   FakeLayerTreeHostImpl host_impl(&task_runner_provider, &task_graph_runner);
   host_impl.SetVisible(true);
-  EXPECT_TRUE(host_impl.InitializeRenderer(layer_tree_frame_sink.get()));
+  EXPECT_TRUE(host_impl.InitializeFrameSink(layer_tree_frame_sink.get()));
   std::unique_ptr<LayerImpl> root_ptr =
       LayerImpl::Create(host_impl.active_tree(), 2);
   LayerImpl* root = root_ptr.get();
@@ -229,10 +182,10 @@ TEST(LayerImplTest, VerifyActiveLayerChangesAreTrackedProperly) {
   root->SetScrollable(gfx::Size(100, 100));
   host_impl.active_tree()->BuildLayerListAndPropertyTreesForTesting();
 
-  // Make root the inner viewport container layer. This ensures the later call
+  // Make root the outer viewport container layer. This ensures the later call
   // to |SetViewportBoundsDelta| will be on a viewport layer.
   LayerTreeImpl::ViewportLayerIds viewport_ids;
-  viewport_ids.inner_viewport_container = root->id();
+  viewport_ids.outer_viewport_container = root->id();
   host_impl.active_tree()->SetViewportLayersFromIds(viewport_ids);
 
   root->SetMasksToBounds(true);
@@ -243,6 +196,8 @@ TEST(LayerImplTest, VerifyActiveLayerChangesAreTrackedProperly) {
   // SetViewportBoundsDelta changes subtree only when masks_to_bounds is true.
   root->SetViewportBoundsDelta(gfx::Vector2d(222, 333));
   EXPECT_TRUE(root->LayerPropertyChanged());
+  EXPECT_TRUE(root->LayerPropertyChangedFromPropertyTrees());
+  EXPECT_FALSE(root->LayerPropertyChangedNotFromPropertyTrees());
   EXPECT_TRUE(host_impl.active_tree()->property_trees()->full_tree_damaged);
 
   root->SetMasksToBounds(false);
@@ -253,18 +208,20 @@ TEST(LayerImplTest, VerifyActiveLayerChangesAreTrackedProperly) {
   // SetViewportBoundsDelta does not change the subtree without masks_to_bounds.
   root->SetViewportBoundsDelta(gfx::Vector2d(333, 444));
   EXPECT_TRUE(root->LayerPropertyChanged());
+  EXPECT_FALSE(root->LayerPropertyChangedFromPropertyTrees());
+  EXPECT_TRUE(root->LayerPropertyChangedNotFromPropertyTrees());
   EXPECT_FALSE(host_impl.active_tree()->property_trees()->full_tree_damaged);
 
   host_impl.active_tree()->property_trees()->needs_rebuild = true;
   host_impl.active_tree()->BuildLayerListAndPropertyTreesForTesting();
   root->layer_tree_impl()->ResetAllChangeTracking();
 
-  // Ensure some node is affected by the inner viewport bounds delta. This
+  // Ensure some node is affected by the outer viewport bounds delta. This
   // ensures the later call to |SetViewportBoundsDelta| will require a
   // transform tree update.
   TransformTree& transform_tree =
       host_impl.active_tree()->property_trees()->transform_tree;
-  transform_tree.AddNodeAffectedByInnerViewportBoundsDelta(
+  transform_tree.AddNodeAffectedByOuterViewportBoundsDelta(
       child->transform_tree_index());
   EXPECT_FALSE(transform_tree.needs_update());
   root->SetViewportBoundsDelta(gfx::Vector2d(111, 222));
@@ -278,6 +235,8 @@ TEST(LayerImplTest, VerifyActiveLayerChangesAreTrackedProperly) {
   root->ScrollBy(gfx::Vector2d(7, 9));
   EXPECT_TRUE(transform_tree.needs_update());
   EXPECT_TRUE(root->LayerPropertyChanged());
+  EXPECT_TRUE(root->LayerPropertyChangedFromPropertyTrees());
+  EXPECT_FALSE(root->LayerPropertyChangedNotFromPropertyTrees());
   EXPECT_FALSE(host_impl.active_tree()->property_trees()->full_tree_damaged);
 }
 
@@ -288,7 +247,7 @@ TEST(LayerImplTest, VerifyNeedsUpdateDrawProperties) {
       FakeLayerTreeFrameSink::Create3d();
   FakeLayerTreeHostImpl host_impl(&task_runner_provider, &task_graph_runner);
   host_impl.SetVisible(true);
-  EXPECT_TRUE(host_impl.InitializeRenderer(layer_tree_frame_sink.get()));
+  EXPECT_TRUE(host_impl.InitializeFrameSink(layer_tree_frame_sink.get()));
   host_impl.active_tree()->SetRootLayerForTesting(
       LayerImpl::Create(host_impl.active_tree(), 1));
   LayerImpl* root = host_impl.active_tree()->root_layer_for_testing();
@@ -304,7 +263,6 @@ TEST(LayerImplTest, VerifyNeedsUpdateDrawProperties) {
   host_impl.active_tree()->BuildLayerListAndPropertyTreesForTesting();
   DCHECK(host_impl.CanDraw());
 
-  gfx::PointF arbitrary_point_f = gfx::PointF(0.125f, 0.25f);
   float arbitrary_number = 0.352f;
   gfx::Size arbitrary_size = gfx::Size(111, 222);
   gfx::Vector2d arbitrary_vector2d = gfx::Vector2d(111, 222);
@@ -363,8 +321,6 @@ TEST(LayerImplTest, VerifyNeedsUpdateDrawProperties) {
                                       layer->NoteLayerPropertyChanged());
   VERIFY_NEEDS_UPDATE_DRAW_PROPERTIES(layer->SetContentsOpaque(true);
                                       layer->NoteLayerPropertyChanged());
-  VERIFY_NEEDS_UPDATE_DRAW_PROPERTIES(layer2->SetPosition(arbitrary_point_f);
-                                      layer->NoteLayerPropertyChanged());
   VERIFY_NEEDS_UPDATE_DRAW_PROPERTIES(
       layer->SetBackgroundColor(arbitrary_color));
   VERIFY_NEEDS_UPDATE_DRAW_PROPERTIES(
@@ -384,15 +340,11 @@ TEST(LayerImplTest, VerifyNeedsUpdateDrawProperties) {
                                                 arbitrary_filters));
   VERIFY_NO_NEEDS_UPDATE_DRAW_PROPERTIES(layer->SetMasksToBounds(true));
   VERIFY_NO_NEEDS_UPDATE_DRAW_PROPERTIES(layer->SetContentsOpaque(true));
-  VERIFY_NO_NEEDS_UPDATE_DRAW_PROPERTIES(
-      layer2->SetPosition(arbitrary_point_f));
   VERIFY_NO_NEEDS_UPDATE_DRAW_PROPERTIES(layer->SetDrawsContent(true));
   VERIFY_NO_NEEDS_UPDATE_DRAW_PROPERTIES(
       layer->SetBackgroundColor(arbitrary_color));
   VERIFY_NO_NEEDS_UPDATE_DRAW_PROPERTIES(layer->SetBounds(arbitrary_size));
   VERIFY_NO_NEEDS_UPDATE_DRAW_PROPERTIES(layer->SetElementId(ElementId(2)));
-  VERIFY_NO_NEEDS_UPDATE_DRAW_PROPERTIES(
-      layer->SetMutableProperties(MutableProperty::kTransform));
 }
 
 TEST(LayerImplTest, SafeOpaqueBackgroundColor) {
@@ -402,7 +354,7 @@ TEST(LayerImplTest, SafeOpaqueBackgroundColor) {
       FakeLayerTreeFrameSink::Create3d();
   FakeLayerTreeHostImpl host_impl(&task_runner_provider, &task_graph_runner);
   host_impl.SetVisible(true);
-  EXPECT_TRUE(host_impl.InitializeRenderer(layer_tree_frame_sink.get()));
+  EXPECT_TRUE(host_impl.InitializeFrameSink(layer_tree_frame_sink.get()));
   host_impl.active_tree()->SetRootLayerForTesting(
       LayerImpl::Create(host_impl.active_tree(), 1));
   LayerImpl* layer = host_impl.active_tree()->root_layer_for_testing();
@@ -439,7 +391,6 @@ TEST(LayerImplTest, PerspectiveTransformHasReasonableScale) {
   std::unique_ptr<LayerTreeFrameSink> layer_tree_frame_sink =
       FakeLayerTreeFrameSink::Create3d();
   LayerTreeSettings settings;
-  settings.layer_transforms_should_scale_layer_contents = true;
   FakeLayerTreeHostImpl host_impl(settings, &task_runner_provider,
                                   &task_graph_runner);
   auto owned_layer = LayerImpl::Create(host_impl.active_tree(), 1);
@@ -479,14 +430,25 @@ TEST(LayerImplTest, PerspectiveTransformHasReasonableScale) {
     ASSERT_TRUE(layer->ScreenSpaceTransform().HasPerspective());
     EXPECT_FLOAT_EQ(127.f, layer->GetIdealContentsScale());
   }
+  // Test case from crbug.com/766021.
+  {
+    gfx::Transform transform(-0.9397f, -0.7019f, 0.2796f, 2383.4521f,   // row 1
+                             -0.0038f, 0.0785f, 1.0613f, 1876.4553f,    // row 2
+                             -0.0835f, 0.9081f, -0.4105f, -2208.3035f,  // row 3
+                             0.0001f, -0.0008f, 0.0003f, 2.8435f);      // row 4
+    layer->draw_properties().screen_space_transform = transform;
+
+    ASSERT_TRUE(layer->ScreenSpaceTransform().HasPerspective());
+    EXPECT_FLOAT_EQ(1.f, layer->GetIdealContentsScale());
+  }
 }
 
 class LayerImplScrollTest : public testing::Test {
  public:
-  LayerImplScrollTest()
-      : host_impl_(settings(),
-                   &task_runner_provider_,
-                   &task_graph_runner_),
+  LayerImplScrollTest() : LayerImplScrollTest(LayerTreeSettings()) {}
+
+  explicit LayerImplScrollTest(const LayerTreeSettings& settings)
+      : host_impl_(settings, &task_runner_provider_, &task_graph_runner_),
         root_id_(7) {
     host_impl_.active_tree()->SetRootLayerForTesting(
         LayerImpl::Create(host_impl_.active_tree(), root_id_));
@@ -520,11 +482,6 @@ class LayerImplScrollTest : public testing::Test {
 
   LayerTreeImpl* tree() { return host_impl_.active_tree(); }
 
-  LayerTreeSettings settings() {
-    LayerTreeSettings settings;
-    return settings;
-  }
-
  private:
   FakeImplTaskRunnerProvider task_runner_provider_;
   TestTaskGraphRunner task_graph_runner_;
@@ -532,6 +489,17 @@ class LayerImplScrollTest : public testing::Test {
   int root_id_;
 };
 
+class CommitToPendingTreeLayerImplScrollTest : public LayerImplScrollTest {
+ public:
+  CommitToPendingTreeLayerImplScrollTest() : LayerImplScrollTest(settings()) {}
+
+  LayerTreeSettings settings() {
+    LayerTreeSettings tree_settings;
+    tree_settings.commit_to_active_tree = false;
+    return tree_settings;
+  }
+};
+
 TEST_F(LayerImplScrollTest, ScrollByWithZeroOffset) {
   // Test that LayerImpl::ScrollBy only affects ScrollDelta and total scroll
   // offset is bounded by the range [0, max scroll offset].
@@ -635,7 +603,33 @@ TEST_F(LayerImplScrollTest, ScrollUserUnscrollableLayer) {
   EXPECT_VECTOR_EQ(gfx::Vector2dF(30.5f, 5), layer()->CurrentScrollOffset());
 }
 
-TEST_F(LayerImplScrollTest, PushPropertiesToMirrorsCurrentScrollOffset) {
+// |LayerImpl::all_touch_action_regions_| is a cache of all regions on
+// |LayerImpl::touch_action_region_| and must be invalidated on changes.
+TEST_F(LayerImplScrollTest, TouchActionRegionCacheInvalidation) {
+  host_impl().CreatePendingTree();
+  std::unique_ptr<LayerImpl> pending_layer =
+      LayerImpl::Create(host_impl().pending_tree(), 2);
+
+  TouchActionRegion region;
+  region.Union(kTouchActionNone, gfx::Rect(0, 0, 50, 50));
+  pending_layer->SetTouchActionRegion(region);
+
+  // The values for GetAllTouchActionRegions should be correct on both layers.
+  // Note that querying GetAllTouchActionRegions will update the cached value
+  // in |LayerImpl::all_touch_action_regions_|.
+  EXPECT_EQ(pending_layer->GetAllTouchActionRegions(), region.GetAllRegions());
+  EXPECT_EQ(layer()->GetAllTouchActionRegions(), Region());
+
+  pending_layer->PushPropertiesTo(layer());
+
+  // After pushing properties, the value for GetAllTouchActionRegions should
+  // not be stale.
+  EXPECT_EQ(pending_layer->GetAllTouchActionRegions(), region.GetAllRegions());
+  EXPECT_EQ(layer()->GetAllTouchActionRegions(), region.GetAllRegions());
+}
+
+TEST_F(CommitToPendingTreeLayerImplScrollTest,
+       PushPropertiesToMirrorsCurrentScrollOffset) {
   gfx::ScrollOffset scroll_offset(10, 5);
   gfx::Vector2dF scroll_delta(12, 18);
 
