@@ -8,31 +8,34 @@
 
 #include "base/bind.h"
 #include "base/location.h"
+#include "base/memory/ptr_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/values.h"
 #include "chrome/browser/chromeos/login/users/fake_chrome_user_manager.h"
-#include "chrome/browser/chromeos/login/users/scoped_user_manager_enabler.h"
+#include "chrome/browser/chromeos/settings/cros_settings.h"
 #include "chrome/browser/chromeos/settings/scoped_cros_settings_test_helper.h"
-#include "chrome/browser/chromeos/settings/stub_install_attributes.h"
 #include "chrome/browser/extensions/extension_function_test_utils.h"
-#include "chrome/browser/signin/signin_manager_factory.h"
+#include "chrome/browser/signin/identity_manager_factory.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/test/base/browser_with_test_window_test.h"
 #include "chrome/test/base/testing_browser_process.h"
 #include "chrome/test/base/testing_profile_manager.h"
-#include "chromeos/attestation/attestation_constants.h"
 #include "chromeos/attestation/mock_attestation_flow.h"
 #include "chromeos/cryptohome/async_method_caller.h"
+#include "chromeos/cryptohome/cryptohome_parameters.h"
 #include "chromeos/cryptohome/mock_async_method_caller.h"
+#include "chromeos/dbus/constants/attestation_constants.h"
+#include "chromeos/dbus/cryptohome/fake_cryptohome_client.h"
 #include "chromeos/dbus/dbus_method_call_status.h"
-#include "chromeos/dbus/fake_cryptohome_client.h"
+#include "chromeos/tpm/stub_install_attributes.h"
+#include "components/account_id/account_id.h"
 #include "components/policy/core/common/cloud/cloud_policy_constants.h"
 #include "components/prefs/pref_service.h"
-#include "components/signin/core/account_id/account_id.h"
-#include "components/signin/core/browser/signin_manager.h"
-#include "extensions/common/test_util.h"
+#include "components/signin/public/identity_manager/identity_test_utils.h"
+#include "components/user_manager/scoped_user_manager.h"
+#include "extensions/common/extension_builder.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/cros_system_api/dbus/service_constants.h"
@@ -108,7 +111,9 @@ void GetCertificateCallbackTrue(
     const chromeos::attestation::AttestationFlow::CertificateCallback&
         callback) {
   base::ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE, base::BindOnce(callback, true, "certificate"));
+      FROM_HERE,
+      base::BindOnce(callback, chromeos::attestation::ATTESTATION_SUCCESS,
+                     "certificate"));
 }
 
 void GetCertificateCallbackFalse(
@@ -119,17 +124,19 @@ void GetCertificateCallbackFalse(
     const chromeos::attestation::AttestationFlow::CertificateCallback&
         callback) {
   base::ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE, base::BindOnce(callback, false, ""));
+      FROM_HERE,
+      base::BindOnce(callback,
+                     chromeos::attestation::ATTESTATION_UNSPECIFIED_FAILURE,
+                     ""));
 }
 
 class EPKChallengeKeyTestBase : public BrowserWithTestWindowTest {
  protected:
   EPKChallengeKeyTestBase()
       : settings_helper_(false),
-        extension_(test_util::CreateEmptyExtension()),
-        profile_manager_(TestingBrowserProcess::GetGlobal()),
+        extension_(ExtensionBuilder("Test").Build()),
         fake_user_manager_(new chromeos::FakeChromeUserManager),
-        user_manager_enabler_(fake_user_manager_) {
+        user_manager_enabler_(base::WrapUnique(fake_user_manager_)) {
     // Set up the default behavior of mocks.
     ON_CALL(mock_async_method_caller_, TpmAttestationRegisterKey(_, _, _, _))
         .WillByDefault(Invoke(RegisterKeyCallbackTrue));
@@ -141,13 +148,11 @@ class EPKChallengeKeyTestBase : public BrowserWithTestWindowTest {
 
     stub_install_attributes_.SetCloudManaged("google.com", "device_id");
 
-    settings_helper_.ReplaceProvider(chromeos::kDeviceAttestationEnabled);
+    settings_helper_.ReplaceDeviceSettingsProviderWithStub();
     settings_helper_.SetBoolean(chromeos::kDeviceAttestationEnabled, true);
   }
 
   void SetUp() override {
-    ASSERT_TRUE(profile_manager_.SetUp());
-
     BrowserWithTestWindowTest::SetUp();
 
     // Set the user preferences.
@@ -163,20 +168,15 @@ class EPKChallengeKeyTestBase : public BrowserWithTestWindowTest {
   TestingProfile* CreateProfile() override {
     fake_user_manager_->AddUserWithAffiliation(
         AccountId::FromUserEmail(kUserEmail), true);
-    return profile_manager_.CreateTestingProfile(kUserEmail);
-  }
-
-  void DestroyProfile(TestingProfile* profile) override {
-    profile_manager_.DeleteTestingProfile(profile->GetProfileUserName());
-    // Profile itself will be destroyed later in
-    // ProfileManager::ProfileInfo::~ProfileInfo() .
+    return profile_manager()->CreateTestingProfile(kUserEmail);
   }
 
   // Derived classes can override this method to set the required authenticated
-  // user in the SigninManager class.
+  // user in the IdentityManager class.
   virtual void SetAuthenticatedUser() {
-    SigninManagerFactory::GetForProfile(browser()->profile())
-        ->SetAuthenticatedAccountInfo("12345", kUserEmail);
+    signin::MakePrimaryAccountAvailable(
+        IdentityManagerFactory::GetForProfile(browser()->profile()),
+        kUserEmail);
   }
 
   // Like extension_function_test_utils::RunFunctionAndReturnError but with an
@@ -184,7 +184,8 @@ class EPKChallengeKeyTestBase : public BrowserWithTestWindowTest {
   std::string RunFunctionAndReturnError(UIThreadExtensionFunction* function,
                                         std::unique_ptr<base::ListValue> args,
                                         Browser* browser) {
-    utils::RunFunction(function, std::move(args), browser, utils::NONE);
+    utils::RunFunction(function, std::move(args), browser,
+                       extensions::api_test_utils::NONE);
     EXPECT_EQ(ExtensionFunction::FAILED, *function->response_type());
     return function->GetError();
   }
@@ -198,7 +199,8 @@ class EPKChallengeKeyTestBase : public BrowserWithTestWindowTest {
     scoped_refptr<ExtensionFunction> function_owner(function);
     // Without a callback the function will not generate a result.
     function->set_has_callback(true);
-    utils::RunFunction(function, std::move(args), browser, utils::NONE);
+    utils::RunFunction(function, std::move(args), browser,
+                       extensions::api_test_utils::NONE);
     EXPECT_TRUE(function->GetError().empty()) << "Unexpected error: "
                                               << function->GetError();
     const base::Value* single_result = NULL;
@@ -213,12 +215,11 @@ class EPKChallengeKeyTestBase : public BrowserWithTestWindowTest {
   NiceMock<cryptohome::MockAsyncMethodCaller> mock_async_method_caller_;
   NiceMock<chromeos::attestation::MockAttestationFlow> mock_attestation_flow_;
   chromeos::ScopedCrosSettingsTestHelper settings_helper_;
-  scoped_refptr<extensions::Extension> extension_;
+  scoped_refptr<const extensions::Extension> extension_;
   chromeos::StubInstallAttributes stub_install_attributes_;
-  TestingProfileManager profile_manager_;
   // fake_user_manager_ is owned by user_manager_enabler_.
   chromeos::FakeChromeUserManager* fake_user_manager_;
-  chromeos::ScopedUserManagerEnabler user_manager_enabler_;
+  user_manager::ScopedUserManager user_manager_enabler_;
   PrefService* prefs_ = nullptr;
 };
 
@@ -244,11 +245,11 @@ class EPKChallengeMachineKeyTest : public EPKChallengeKeyTestBase {
   }
 
   std::unique_ptr<base::ListValue> CreateArgsNoRegister() {
-    return CreateArgsInternal(base::MakeUnique<bool>(false));
+    return CreateArgsInternal(std::make_unique<bool>(false));
   }
 
   std::unique_ptr<base::ListValue> CreateArgsRegister() {
-    return CreateArgsInternal(base::MakeUnique<bool>(true));
+    return CreateArgsInternal(std::make_unique<bool>(true));
   }
 
   std::unique_ptr<base::ListValue> CreateArgsInternal(
@@ -328,16 +329,16 @@ TEST_F(EPKChallengeMachineKeyTest, KeyExists) {
   // GetCertificate must not be called if the key exists.
   EXPECT_CALL(mock_attestation_flow_, GetCertificate(_, _, _, _, _)).Times(0);
 
-  EXPECT_TRUE(
-      utils::RunFunction(func_.get(), CreateArgs(), browser(), utils::NONE));
+  EXPECT_TRUE(utils::RunFunction(func_.get(), CreateArgs(), browser(),
+                                 extensions::api_test_utils::NONE));
 }
 
 TEST_F(EPKChallengeMachineKeyTest, KeyNotRegisteredByDefault) {
   EXPECT_CALL(mock_async_method_caller_, TpmAttestationRegisterKey(_, _, _, _))
       .Times(0);
 
-  EXPECT_TRUE(
-      utils::RunFunction(func_.get(), CreateArgs(), browser(), utils::NONE));
+  EXPECT_TRUE(utils::RunFunction(func_.get(), CreateArgs(), browser(),
+                                 extensions::api_test_utils::NONE));
 }
 
 TEST_F(EPKChallengeMachineKeyTest, KeyNotRegistered) {
@@ -345,7 +346,7 @@ TEST_F(EPKChallengeMachineKeyTest, KeyNotRegistered) {
       .Times(0);
 
   EXPECT_TRUE(utils::RunFunction(func_.get(), CreateArgsNoRegister(), browser(),
-                                 utils::NONE));
+                                 extensions::api_test_utils::NONE));
 }
 
 TEST_F(EPKChallengeMachineKeyTest, Success) {
@@ -368,7 +369,7 @@ TEST_F(EPKChallengeMachineKeyTest, Success) {
 
   ASSERT_TRUE(value->is_blob());
   EXPECT_EQ("response",
-            std::string(value->GetBlob().data(), value->GetBlob().size()));
+            std::string(value->GetBlob().begin(), value->GetBlob().end()));
 }
 
 TEST_F(EPKChallengeMachineKeyTest, KeyRegisteredSuccess) {
@@ -397,7 +398,7 @@ TEST_F(EPKChallengeMachineKeyTest, KeyRegisteredSuccess) {
 
   ASSERT_TRUE(value->is_blob());
   EXPECT_EQ("response",
-            std::string(value->GetBlob().data(), value->GetBlob().size()));
+            std::string(value->GetBlob().begin(), value->GetBlob().end()));
 }
 
 TEST_F(EPKChallengeMachineKeyTest, AttestationNotPrepared) {
@@ -513,13 +514,14 @@ TEST_F(EPKChallengeUserKeyTest, KeyRegistrationFailed) {
 
 TEST_F(EPKChallengeUserKeyTest, KeyExists) {
   cryptohome_client_.SetTpmAttestationUserCertificate(
-      cryptohome::Identification(AccountId::FromUserEmail(kUserEmail)),
+      cryptohome::CreateAccountIdentifierFromAccountId(
+          AccountId::FromUserEmail(kUserEmail)),
       "attest-ent-user", std::string());
   // GetCertificate must not be called if the key exists.
   EXPECT_CALL(mock_attestation_flow_, GetCertificate(_, _, _, _, _)).Times(0);
 
-  EXPECT_TRUE(
-      utils::RunFunction(func_.get(), CreateArgs(), browser(), utils::NONE));
+  EXPECT_TRUE(utils::RunFunction(func_.get(), CreateArgs(), browser(),
+                                 extensions::api_test_utils::NONE));
 }
 
 TEST_F(EPKChallengeUserKeyTest, KeyNotRegistered) {
@@ -527,7 +529,7 @@ TEST_F(EPKChallengeUserKeyTest, KeyNotRegistered) {
       .Times(0);
 
   EXPECT_TRUE(utils::RunFunction(func_.get(), CreateArgsNoRegister(), browser(),
-                                 utils::NONE));
+                                 extensions::api_test_utils::NONE));
 }
 
 TEST_F(EPKChallengeUserKeyTest, PersonalDevice) {
@@ -565,7 +567,7 @@ TEST_F(EPKChallengeUserKeyTest, Success) {
 
   ASSERT_TRUE(value->is_blob());
   EXPECT_EQ("response",
-            std::string(value->GetBlob().data(), value->GetBlob().size()));
+            std::string(value->GetBlob().begin(), value->GetBlob().end()));
 }
 
 TEST_F(EPKChallengeUserKeyTest, AttestationNotPrepared) {
@@ -586,20 +588,20 @@ class EPKChallengeMachineKeyUnmanagedUserTest
     : public EPKChallengeMachineKeyTest {
  protected:
   void SetAuthenticatedUser() override {
-    SigninManagerFactory::GetForProfile(browser()->profile())
-        ->SetAuthenticatedAccountInfo(account_id_.GetGaiaId(),
-                                      account_id_.GetUserEmail());
+    signin::MakePrimaryAccountAvailable(
+        IdentityManagerFactory::GetForProfile(browser()->profile()),
+        account_id_.GetUserEmail());
   }
 
   TestingProfile* CreateProfile() override {
     fake_user_manager_->AddUser(account_id_);
-    TestingProfile* profile =
-        profile_manager_.CreateTestingProfile(account_id_.GetUserEmail());
-    return profile;
+    return profile_manager()->CreateTestingProfile(account_id_.GetUserEmail());
   }
 
+  const std::string email = "test@chromium.com";
   const AccountId account_id_ =
-      AccountId::FromUserEmailGaiaId("test@chromium.com", "12345");
+      AccountId::FromUserEmailGaiaId(email,
+                                     signin::GetTestGaiaIdForEmail(email));
 };
 
 TEST_F(EPKChallengeMachineKeyUnmanagedUserTest, UserNotManaged) {
@@ -610,20 +612,20 @@ TEST_F(EPKChallengeMachineKeyUnmanagedUserTest, UserNotManaged) {
 class EPKChallengeUserKeyUnmanagedUserTest : public EPKChallengeUserKeyTest {
  protected:
   void SetAuthenticatedUser() override {
-    SigninManagerFactory::GetForProfile(browser()->profile())
-        ->SetAuthenticatedAccountInfo(account_id_.GetGaiaId(),
-                                      account_id_.GetUserEmail());
+    signin::MakePrimaryAccountAvailable(
+        IdentityManagerFactory::GetForProfile(browser()->profile()),
+        account_id_.GetUserEmail());
   }
 
   TestingProfile* CreateProfile() override {
     fake_user_manager_->AddUser(account_id_);
-    TestingProfile* profile =
-        profile_manager_.CreateTestingProfile(account_id_.GetUserEmail());
-    return profile;
+    return profile_manager()->CreateTestingProfile(account_id_.GetUserEmail());
   }
 
+  const std::string email = "test@chromium.com";
   const AccountId account_id_ =
-      AccountId::FromUserEmailGaiaId("test@chromium.com", "12345");
+      AccountId::FromUserEmailGaiaId(email,
+                                     signin::GetTestGaiaIdForEmail(email));
 };
 
 TEST_F(EPKChallengeUserKeyUnmanagedUserTest, UserNotManaged) {
