@@ -7,18 +7,20 @@
 #include <stdint.h>
 #include <utility>
 
-#include "ash/shell.h"
-#include "ash/system/bluetooth/tray_bluetooth_helper.h"
 #include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/macros.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/values.h"
 #include "chrome/browser/chromeos/system/fake_input_device_settings.h"
 #include "chrome/browser/chromeos/system/input_device_settings.h"
+#include "chrome/browser/ui/webui/chromeos/bluetooth_pairing_dialog.h"
+#include "chromeos/dbus/audio/fake_cras_audio_client.h"
 #include "chromeos/dbus/dbus_thread_manager.h"
-#include "chromeos/dbus/fake_cras_audio_client.h"
-#include "chromeos/dbus/fake_power_manager_client.h"
+#include "chromeos/dbus/power/fake_power_manager_client.h"
 #include "content/public/browser/web_ui.h"
+#include "device/bluetooth/bluetooth_adapter_factory.h"
+#include "device/bluetooth/bluetooth_device.h"
 #include "device/bluetooth/dbus/bluez_dbus_manager.h"
 #include "device/bluetooth/dbus/fake_bluetooth_adapter_client.h"
 #include "device/bluetooth/dbus/fake_bluetooth_device_client.h"
@@ -119,8 +121,8 @@ void DeviceEmulatorMessageHandler::BluetoothObserver::DeviceAdded(
 }
 
 void DeviceEmulatorMessageHandler::BluetoothObserver::DevicePropertyChanged(
-  const dbus::ObjectPath& object_path,
-  const std::string& property_name) {
+    const dbus::ObjectPath& object_path,
+    const std::string& property_name) {
   if (property_name == kPairedPropertyName) {
     owner_->web_ui()->CallJavascriptFunctionUnsafe(
         kDevicePairedFromTrayJSCallback, base::Value(object_path.value()));
@@ -139,30 +141,26 @@ class DeviceEmulatorMessageHandler::CrasAudioObserver
  public:
   explicit CrasAudioObserver(DeviceEmulatorMessageHandler* owner)
       : owner_(owner) {
-    owner_->fake_cras_audio_client_->AddObserver(this);
+    chromeos::FakeCrasAudioClient::Get()->AddObserver(this);
   }
 
   ~CrasAudioObserver() override {
-    owner_->fake_cras_audio_client_->RemoveObserver(this);
+    chromeos::FakeCrasAudioClient::Get()->RemoveObserver(this);
   }
 
   // chromeos::CrasAudioClient::Observer.
-  void NodesChanged() override;
+  void NodesChanged() override { owner_->HandleRequestAudioNodes(nullptr); }
 
  private:
   DeviceEmulatorMessageHandler* owner_;
+
   DISALLOW_COPY_AND_ASSIGN(CrasAudioObserver);
 };
 
-void DeviceEmulatorMessageHandler::CrasAudioObserver::NodesChanged() {
-  owner_->HandleRequestAudioNodes(nullptr);
-}
-
 class DeviceEmulatorMessageHandler::PowerObserver
     : public PowerManagerClient::Observer {
  public:
-  explicit PowerObserver(DeviceEmulatorMessageHandler* owner)
-      : owner_(owner) {
+  explicit PowerObserver(DeviceEmulatorMessageHandler* owner) : owner_(owner) {
     owner_->fake_power_manager_client_->AddObserver(this);
   }
 
@@ -170,13 +168,12 @@ class DeviceEmulatorMessageHandler::PowerObserver
     owner_->fake_power_manager_client_->RemoveObserver(this);
   }
 
-  void PowerChanged(
-      const power_manager::PowerSupplyProperties& proto) override;
+  void PowerChanged(const power_manager::PowerSupplyProperties& proto) override;
 
  private:
-   DeviceEmulatorMessageHandler* owner_;
+  DeviceEmulatorMessageHandler* owner_;
 
-   DISALLOW_COPY_AND_ASSIGN(PowerObserver);
+  DISALLOW_COPY_AND_ASSIGN(PowerObserver);
 };
 
 void DeviceEmulatorMessageHandler::PowerObserver::PowerChanged(
@@ -200,23 +197,29 @@ void DeviceEmulatorMessageHandler::PowerObserver::PowerChanged(
 DeviceEmulatorMessageHandler::DeviceEmulatorMessageHandler()
     : fake_bluetooth_device_client_(
           static_cast<bluez::FakeBluetoothDeviceClient*>(
-              bluez::BluezDBusManager::Get()
-                  ->GetBluetoothDeviceClient())),
-      fake_cras_audio_client_(static_cast<chromeos::FakeCrasAudioClient*>(
-          chromeos::DBusThreadManager::Get()
-              ->GetCrasAudioClient())),
-      fake_power_manager_client_(static_cast<chromeos::FakePowerManagerClient*>(
-          chromeos::DBusThreadManager::Get()
-              ->GetPowerManagerClient())),
-      weak_ptr_factory_(this) {}
-
-DeviceEmulatorMessageHandler::~DeviceEmulatorMessageHandler() {
+              bluez::BluezDBusManager::Get()->GetBluetoothDeviceClient())),
+      fake_power_manager_client_(chromeos::FakePowerManagerClient::Get()),
+      weak_ptr_factory_(this) {
+  device::BluetoothAdapterFactory::GetAdapter(
+      base::BindOnce(&DeviceEmulatorMessageHandler::BluetoothDeviceAdapterReady,
+                     weak_ptr_factory_.GetWeakPtr()));
 }
 
+DeviceEmulatorMessageHandler::~DeviceEmulatorMessageHandler() {}
+
 void DeviceEmulatorMessageHandler::Init(const base::ListValue* args) {
   AllowJavascript();
 }
 
+void DeviceEmulatorMessageHandler::BluetoothDeviceAdapterReady(
+    scoped_refptr<device::BluetoothAdapter> adapter) {
+  if (!adapter) {
+    LOG(ERROR) << "Bluetooth adapter not available";
+    return;
+  }
+  bluetooth_adapter_ = adapter;
+}
+
 void DeviceEmulatorMessageHandler::RequestPowerInfo(
     const base::ListValue* args) {
   fake_power_manager_client_->RequestStatusUpdate();
@@ -287,8 +290,7 @@ void DeviceEmulatorMessageHandler::HandleRequestBluetoothPair(
 
   // Try to pair the device with the main adapter. The device is identified
   // by its device ID, which, in this case is the same as its address.
-  ash::Shell::Get()->tray_bluetooth_helper()->ConnectToBluetoothDevice(
-      props->address.value());
+  ConnectToBluetoothDevice(props->address.value());
   if (!props->paired.value()) {
     web_ui()->CallJavascriptFunctionUnsafe(kPairFailedJSCallback,
                                            base::Value(path));
@@ -300,12 +302,13 @@ void DeviceEmulatorMessageHandler::HandleRequestAudioNodes(
   // Get every active audio node and create a dictionary to
   // send it to JavaScript.
   base::ListValue audio_nodes;
-  for (const AudioNode& node : fake_cras_audio_client_->node_list()) {
+  for (const AudioNode& node :
+       chromeos::FakeCrasAudioClient::Get()->node_list()) {
     std::unique_ptr<base::DictionaryValue> audio_node(
         new base::DictionaryValue());
 
     audio_node->SetBoolean("isInput", node.is_input);
-    audio_node->SetString("id", base::Uint64ToString(node.id));
+    audio_node->SetString("id", base::NumberToString(node.id));
     audio_node->SetString("deviceName", node.device_name);
     audio_node->SetString("type", node.type);
     audio_node->SetString("name", node.name);
@@ -332,7 +335,7 @@ void DeviceEmulatorMessageHandler::HandleInsertAudioNode(
   CHECK(device_dict->GetString("id", &tmp_id));
   CHECK(base::StringToUint64(tmp_id, &audio_node.id));
 
-  fake_cras_audio_client_->InsertAudioNodeToList(audio_node);
+  chromeos::FakeCrasAudioClient::Get()->InsertAudioNodeToList(audio_node);
 }
 
 void DeviceEmulatorMessageHandler::HandleRemoveAudioNode(
@@ -342,7 +345,7 @@ void DeviceEmulatorMessageHandler::HandleRemoveAudioNode(
   CHECK(args->GetString(0, &tmp_id));
   CHECK(base::StringToUint64(tmp_id, &id));
 
-  fake_cras_audio_client_->RemoveAudioNodeFromList(id);
+  chromeos::FakeCrasAudioClient::Get()->RemoveAudioNodeFromList(id);
 }
 
 void DeviceEmulatorMessageHandler::HandleSetHasTouchpad(
@@ -368,7 +371,7 @@ void DeviceEmulatorMessageHandler::UpdateBatteryPercent(
   int new_percent;
   if (args->GetInteger(0, &new_percent)) {
     power_manager::PowerSupplyProperties props =
-        fake_power_manager_client_->props();
+        *fake_power_manager_client_->GetLastStatus();
     props.set_battery_percent(new_percent);
     fake_power_manager_client_->UpdatePowerProperties(props);
   }
@@ -379,7 +382,7 @@ void DeviceEmulatorMessageHandler::UpdateBatteryState(
   int battery_state;
   if (args->GetInteger(0, &battery_state)) {
     power_manager::PowerSupplyProperties props =
-        fake_power_manager_client_->props();
+        *fake_power_manager_client_->GetLastStatus();
     props.set_battery_state(
         static_cast<power_manager::PowerSupplyProperties_BatteryState>(
             battery_state));
@@ -392,7 +395,7 @@ void DeviceEmulatorMessageHandler::UpdateTimeToEmpty(
   int new_time;
   if (args->GetInteger(0, &new_time)) {
     power_manager::PowerSupplyProperties props =
-        fake_power_manager_client_->props();
+        *fake_power_manager_client_->GetLastStatus();
     props.set_battery_time_to_empty_sec(new_time);
     fake_power_manager_client_->UpdatePowerProperties(props);
   }
@@ -403,7 +406,7 @@ void DeviceEmulatorMessageHandler::UpdateTimeToFull(
   int new_time;
   if (args->GetInteger(0, &new_time)) {
     power_manager::PowerSupplyProperties props =
-        fake_power_manager_client_->props();
+        *fake_power_manager_client_->GetLastStatus();
     props.set_battery_time_to_full_sec(new_time);
     fake_power_manager_client_->UpdatePowerProperties(props);
   }
@@ -414,7 +417,7 @@ void DeviceEmulatorMessageHandler::UpdatePowerSources(
   const base::ListValue* sources;
   CHECK(args->GetList(0, &sources));
   power_manager::PowerSupplyProperties props =
-      fake_power_manager_client_->props();
+      *fake_power_manager_client_->GetLastStatus();
 
   std::string selected_id = props.external_power_source_id();
 
@@ -445,8 +448,8 @@ void DeviceEmulatorMessageHandler::UpdatePowerSources(
             port));
     std::string power_level;
     CHECK(dict->GetString("power", &power_level));
-    source->set_max_power(
-        power_level == "high" ? kPowerLevelHigh : kPowerLevelLow);
+    source->set_max_power(power_level == "high" ? kPowerLevelHigh
+                                                : kPowerLevelLow);
     if (id == selected_id)
       selected_source = source;
   }
@@ -476,72 +479,77 @@ void DeviceEmulatorMessageHandler::UpdatePowerSourceId(
 
 void DeviceEmulatorMessageHandler::RegisterMessages() {
   web_ui()->RegisterMessageCallback(
-      kInitialize,
-      base::Bind(&DeviceEmulatorMessageHandler::Init, base::Unretained(this)));
+      kInitialize, base::BindRepeating(&DeviceEmulatorMessageHandler::Init,
+                                       base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kRequestPowerInfo,
-      base::Bind(&DeviceEmulatorMessageHandler::RequestPowerInfo,
-                 base::Unretained(this)));
+      base::BindRepeating(&DeviceEmulatorMessageHandler::RequestPowerInfo,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kUpdateBatteryPercent,
-      base::Bind(&DeviceEmulatorMessageHandler::UpdateBatteryPercent,
-                 base::Unretained(this)));
+      base::BindRepeating(&DeviceEmulatorMessageHandler::UpdateBatteryPercent,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kUpdateBatteryState,
-      base::Bind(&DeviceEmulatorMessageHandler::UpdateBatteryState,
-                 base::Unretained(this)));
+      base::BindRepeating(&DeviceEmulatorMessageHandler::UpdateBatteryState,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kUpdateTimeToEmpty,
-      base::Bind(&DeviceEmulatorMessageHandler::UpdateTimeToEmpty,
-                 base::Unretained(this)));
+      base::BindRepeating(&DeviceEmulatorMessageHandler::UpdateTimeToEmpty,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kUpdateTimeToFull,
-      base::Bind(&DeviceEmulatorMessageHandler::UpdateTimeToFull,
-                 base::Unretained(this)));
+      base::BindRepeating(&DeviceEmulatorMessageHandler::UpdateTimeToFull,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kUpdatePowerSources,
-      base::Bind(&DeviceEmulatorMessageHandler::UpdatePowerSources,
-                 base::Unretained(this)));
+      base::BindRepeating(&DeviceEmulatorMessageHandler::UpdatePowerSources,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kUpdatePowerSourceId,
-      base::Bind(&DeviceEmulatorMessageHandler::UpdatePowerSourceId,
-                 base::Unretained(this)));
+      base::BindRepeating(&DeviceEmulatorMessageHandler::UpdatePowerSourceId,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kRequestAudioNodes,
-      base::Bind(&DeviceEmulatorMessageHandler::HandleRequestAudioNodes,
-                 base::Unretained(this)));
+      base::BindRepeating(
+          &DeviceEmulatorMessageHandler::HandleRequestAudioNodes,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kInsertAudioNode,
-      base::Bind(&DeviceEmulatorMessageHandler::HandleInsertAudioNode,
-                 base::Unretained(this)));
+      base::BindRepeating(&DeviceEmulatorMessageHandler::HandleInsertAudioNode,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kRemoveAudioNode,
-      base::Bind(&DeviceEmulatorMessageHandler::HandleRemoveAudioNode,
-                 base::Unretained(this)));
+      base::BindRepeating(&DeviceEmulatorMessageHandler::HandleRemoveAudioNode,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kBluetoothDiscoverFunction,
-      base::Bind(&DeviceEmulatorMessageHandler::HandleRequestBluetoothDiscover,
-                 base::Unretained(this)));
+      base::BindRepeating(
+          &DeviceEmulatorMessageHandler::HandleRequestBluetoothDiscover,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kBluetoothPairFunction,
-      base::Bind(&DeviceEmulatorMessageHandler::HandleRequestBluetoothPair,
-                 base::Unretained(this)));
+      base::BindRepeating(
+          &DeviceEmulatorMessageHandler::HandleRequestBluetoothPair,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kRequestBluetoothInfo,
-      base::Bind(&DeviceEmulatorMessageHandler::HandleRequestBluetoothInfo,
-                 base::Unretained(this)));
+      base::BindRepeating(
+          &DeviceEmulatorMessageHandler::HandleRequestBluetoothInfo,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kRemoveBluetoothDevice,
-      base::Bind(&DeviceEmulatorMessageHandler::HandleRemoveBluetoothDevice,
-                 base::Unretained(this)));
+      base::BindRepeating(
+          &DeviceEmulatorMessageHandler::HandleRemoveBluetoothDevice,
+          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kSetHasTouchpad,
-      base::Bind(&DeviceEmulatorMessageHandler::HandleSetHasTouchpad,
-                 base::Unretained(this)));
+      base::BindRepeating(&DeviceEmulatorMessageHandler::HandleSetHasTouchpad,
+                          base::Unretained(this)));
   web_ui()->RegisterMessageCallback(
       kSetHasMouse,
-      base::Bind(&DeviceEmulatorMessageHandler::HandleSetHasMouse,
-                 base::Unretained(this)));
+      base::BindRepeating(&DeviceEmulatorMessageHandler::HandleSetHasMouse,
+                          base::Unretained(this)));
 }
 
 void DeviceEmulatorMessageHandler::OnJavascriptAllowed() {
@@ -625,6 +633,29 @@ DeviceEmulatorMessageHandler::GetDeviceInfo(
   return device;
 }
 
+void DeviceEmulatorMessageHandler::ConnectToBluetoothDevice(
+    const std::string& address) {
+  if (!bluetooth_adapter_) {
+    LOG(ERROR) << "Bluetooth adapter not ready";
+    return;
+  }
+  device::BluetoothDevice* device = bluetooth_adapter_->GetDevice(address);
+  if (!device || device->IsConnecting() ||
+      (device->IsPaired() &&
+       (device->IsConnected() || !device->IsConnectable()))) {
+    return;
+  }
+  if (!device->IsPaired() && device->IsPairable()) {
+    // Show pairing dialog for the unpaired device.
+    chromeos::BluetoothPairingDialog::ShowDialog(
+        device->GetAddress(), device->GetNameForDisplay(), device->IsPaired(),
+        device->IsConnected());
+  } else {
+    // Attempt to connect to the device.
+    device->Connect(nullptr, base::DoNothing(), base::DoNothing());
+  }
+}
+
 void DeviceEmulatorMessageHandler::TouchpadExists(bool exists) {
   if (!IsJavascriptAllowed())
     return;
