@@ -15,12 +15,12 @@
 #include "components/autofill/core/browser/autofill_driver.h"
 #include "components/autofill/core/browser/autofill_manager.h"
 #include "components/autofill/core/browser/autofill_test_utils.h"
-#include "components/autofill/core/browser/credit_card.h"
+#include "components/autofill/core/browser/data_model/credit_card.h"
 #include "components/autofill/core/browser/form_structure.h"
 #include "components/autofill/core/browser/mock_autocomplete_history_manager.h"
+#include "components/autofill/core/browser/payments/test_credit_card_save_manager.h"
 #include "components/autofill/core/browser/test_autofill_client.h"
 #include "components/autofill/core/browser/test_autofill_driver.h"
-#include "components/autofill/core/browser/test_credit_card_save_manager.h"
 #include "components/autofill/core/browser/test_personal_data_manager.h"
 #include "components/autofill/core/common/autofill_constants.h"
 #include "components/autofill/core/common/autofill_features.h"
@@ -70,9 +70,9 @@ class AutofillAssistantTest : public testing::Test {
 
   void SetUp() {
     payments::TestPaymentsClient* payments_client =
-        new payments::TestPaymentsClient(
-            autofill_driver_.GetURLLoaderFactory(), autofill_client_.GetPrefs(),
-            autofill_client_.GetIdentityManager(), &pdm_);
+        new payments::TestPaymentsClient(autofill_driver_.GetURLLoaderFactory(),
+                                         autofill_client_.GetIdentityManager(),
+                                         &pdm_);
     autofill_client_.set_test_payments_client(
         std::unique_ptr<payments::TestPaymentsClient>(payments_client));
     TestCreditCardSaveManager* credit_card_save_manager =
@@ -101,7 +101,7 @@ class AutofillAssistantTest : public testing::Test {
   // Returns a valid credit card form.
   FormData CreateValidCreditCardFormData() {
     FormData form;
-    form.origin = GURL("https://myform.com");
+    form.url = GURL("https://myform.com");
     form.action = GURL("https://myform.com/submit");
 
     FormFieldData field;
@@ -139,6 +139,15 @@ class AutofillAssistantTest : public testing::Test {
     return form_structure;
   }
 
+  // Convenience method to cast the FullCardRequest into a CardUnmaskDelegate.
+  CardUnmaskDelegate* full_card_unmask_delegate() {
+    payments::FullCardRequest* full_card_request =
+        autofill_manager_->credit_card_access_manager_->cvc_authenticator_
+            ->full_card_request_.get();
+    DCHECK(full_card_request);
+    return static_cast<CardUnmaskDelegate*>(full_card_request);
+  }
+
   base::test::ScopedTaskEnvironment task_environment_;
   TestAutofillClient autofill_client_;
   testing::NiceMock<TestAutofillDriver> autofill_driver_;
@@ -202,7 +211,7 @@ TEST_F(AutofillAssistantTest, CanShowCreditCardAssist_FeatureOn_NotSecure) {
 
   // Cannot be shown if the context is not secure.
   FormData form = CreateValidCreditCardFormData();
-  form.origin = GURL("http://myform.com");
+  form.url = GURL("http://myform.com");
   form.action = GURL("http://myform.com/submit");
   auto form_structure = std::make_unique<FormStructure>(form);
   form_structure->DetermineHeuristicTypes();
@@ -279,9 +288,7 @@ TEST_F(AutofillAssistantTest, ShowAssistForCreditCard_ValidCard_CancelCvc) {
   EXPECT_CALL(*autofill_manager_, FillCreditCardForm(_, _, _, _, _)).Times(0);
 
   autofill_assistant_->ShowAssistForCreditCard(card);
-  static_cast<CardUnmaskDelegate*>(
-      autofill_manager_->GetOrCreateFullCardRequest())
-      ->OnUnmaskPromptClosed();
+  full_card_unmask_delegate()->OnUnmaskPromptClosed();
 }
 
 TEST_F(AutofillAssistantTest, ShowAssistForCreditCard_ValidCard_SubmitCvc) {
@@ -310,9 +317,7 @@ TEST_F(AutofillAssistantTest, ShowAssistForCreditCard_ValidCard_SubmitCvc) {
 
   CardUnmaskDelegate::UnmaskResponse unmask_response;
   unmask_response.cvc = base::ASCIIToUTF16("123");
-  static_cast<CardUnmaskDelegate*>(
-      autofill_manager_->GetOrCreateFullCardRequest())
-      ->OnUnmaskResponse(unmask_response);
+  full_card_unmask_delegate()->OnUnmaskResponse(unmask_response);
 }
 
 }  // namespace autofill
