@@ -7,18 +7,19 @@
 #include <stddef.h>
 #include <stdint.h>
 
+#include <memory>
 #include <utility>
 #include <vector>
 
 #include "base/base64.h"
 #include "base/location.h"
 #include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/numerics/safe_conversions.h"
 #include "base/time/time.h"
-#include "components/autofill/core/browser/autofill_profile.h"
-#include "components/autofill/core/browser/credit_card.h"
+#include "components/autofill/core/browser/data_model/autofill_profile.h"
+#include "components/autofill/core/browser/data_model/credit_card.h"
 #include "components/autofill/core/browser/webdata/autofill_webdata_backend.h"
+#include "components/autofill/core/browser/webdata/mock_autofill_webdata_backend.h"
 #include "components/sync/model/sync_change.h"
 #include "components/sync/model/sync_change_processor_wrapper_for_test.h"
 #include "components/sync/model/sync_error_factory_mock.h"
@@ -70,13 +71,13 @@ ACTION_P2(GetCopiesOf, profiles, cards) {
   for (const auto& profile : *profiles) {
     std::string utf8_server_id;
     base::Base64Encode(profile.server_id(), &utf8_server_id);
-    (*arg0)[utf8_server_id] = base::MakeUnique<AutofillProfile>(profile);
+    (*arg0)[utf8_server_id] = std::make_unique<AutofillProfile>(profile);
   }
 
   for (const auto& card : *cards) {
     std::string utf8_server_id;
     base::Base64Encode(card.server_id(), &utf8_server_id);
-    (*arg1)[utf8_server_id] = base::MakeUnique<CreditCard>(card);
+    (*arg1)[utf8_server_id] = std::make_unique<CreditCard>(card);
   }
 }
 
@@ -145,35 +146,21 @@ class MockService : public AutofillWalletMetadataSyncableService {
   DISALLOW_COPY_AND_ASSIGN(MockService);
 };
 
-class NoOpWebData : public AutofillWebDataBackend {
- public:
-  NoOpWebData() {}
-  ~NoOpWebData() override {}
-
- private:
-  // AutofillWebDataBackend implementation.
-  WebDatabase* GetDatabase() override { return nullptr; }
-  void AddObserver(
-      AutofillWebDataServiceObserverOnDBThread* observer) override {}
-  void RemoveObserver(
-      AutofillWebDataServiceObserverOnDBThread* observer) override {}
-  void RemoveExpiredFormElements() override {}
-  void NotifyOfMultipleAutofillChanges() override {}
-  void NotifyThatSyncHasStarted(syncer::ModelType /* model_type */) override {}
-
-  DISALLOW_COPY_AND_ASSIGN(NoOpWebData);
-};
-
 class AutofillWalletMetadataSyncableServiceTest : public Test {
  public:
   AutofillWalletMetadataSyncableServiceTest()
-      : local_(&no_op_web_data_), remote_(&no_op_web_data_) {}
+      : local_(&backend_), remote_(&backend_) {}
   ~AutofillWalletMetadataSyncableServiceTest() override {}
 
-  // Outlives local_ and remote_.
-  NoOpWebData no_op_web_data_;
+  void SetUp() {
+    local_.OnWalletDataTrackingStateChanged(true);
+    remote_.OnWalletDataTrackingStateChanged(true);
+  }
 
-  // Outlived by no_op_web_data_.
+  // Outlives local_ and remote_.
+  NiceMock<MockAutofillWebDataBackend> backend_;
+
+  // Outlived by backend_.
   NiceMock<MockService> local_;
   NiceMock<MockService> remote_;
 
@@ -317,6 +304,45 @@ TEST_F(AutofillWalletMetadataSyncableServiceTest, DeleteFromServerOnMerge) {
   MergeMetadata(&local_, &remote_);
 }
 
+// Verify that remote data without local counterpart is kept when we're not
+// tracking wallet data.
+TEST_F(AutofillWalletMetadataSyncableServiceTest,
+       DeleteFromServerOnMerge_NotWhenNotTracking) {
+  local_.OnWalletDataTrackingStateChanged(false);
+
+  remote_.UpdateAddressStats(BuildAddress(kAddr1, 1, 2, true));
+  remote_.UpdateCardStats(BuildCard(kCard1, 3, 4, kAddr1));
+
+  EXPECT_CALL(local_, UpdateAddressStats(_)).Times(0);
+  EXPECT_CALL(local_, UpdateCardStats(_)).Times(0);
+  EXPECT_CALL(local_, SendChangesToSyncServer(_)).Times(0);
+
+  MergeMetadata(&local_, &remote_);
+}
+
+// Verify that remote data without local counterpart is deleted when we start
+// tracking wallet data after the initial merge happened.
+TEST_F(AutofillWalletMetadataSyncableServiceTest,
+       DeleteFromServerOnMerge_MergeBeforeTracking) {
+  local_.OnWalletDataTrackingStateChanged(false);
+
+  remote_.UpdateAddressStats(BuildAddress(kAddr1, 1, 2, true));
+  remote_.UpdateCardStats(BuildCard(kCard1, 3, 4, kAddr1));
+
+  EXPECT_CALL(local_, UpdateAddressStats(_)).Times(0);
+  EXPECT_CALL(local_, UpdateCardStats(_)).Times(0);
+  EXPECT_CALL(
+      local_,
+      SendChangesToSyncServer(UnorderedElementsAre(
+          SyncChangeMatches(syncer::SyncChange::ACTION_DELETE, kAddr1SyncTag),
+          SyncChangeMatches(syncer::SyncChange::ACTION_DELETE,
+                            kCard1SyncTag))));
+
+  MergeMetadata(&local_, &remote_);
+
+  local_.OnWalletDataTrackingStateChanged(true);
+}
+
 MATCHER_P7(SyncAddressChangeAndDataMatch,
            change_type,
            sync_tag,
@@ -450,8 +476,8 @@ TEST_F(AutofillWalletMetadataSyncableServiceTest,
   MergeMetadata(&local_, &remote_);
 }
 
-// Verify that lower values of metadata are not sent to the sync server when
-// local metadata is updated.
+// Verify that lower or equal values of metadata are not sent to the sync server
+// when local metadata is updated.
 TEST_F(AutofillWalletMetadataSyncableServiceTest,
        DontSendLowerValueToServerOnSingleChange) {
   local_.UpdateAddressStats(BuildAddress(kAddr1, 1, 2, true));
@@ -459,8 +485,8 @@ TEST_F(AutofillWalletMetadataSyncableServiceTest,
   remote_.UpdateAddressStats(BuildAddress(kAddr1, 1, 2, true));
   remote_.UpdateCardStats(BuildCard(kCard1, 3, 4, kAddr1));
   MergeMetadata(&local_, &remote_);
-  AutofillProfile address = BuildAddress(kAddr1, 0, 0, false);
-  CreditCard card = BuildCard(kCard1, 0, 0, kAddr2);
+  AutofillProfile address = BuildAddress(kAddr1, 0, 0, true);
+  CreditCard card = BuildCard(kCard1, 3, 4, kAddr1);
 
   EXPECT_CALL(local_, SendChangesToSyncServer(_)).Times(0);
 
@@ -499,6 +525,35 @@ TEST_F(AutofillWalletMetadataSyncableServiceTest,
       CreditCardChange(CreditCardChange::UPDATE, card.guid(), &card));
 }
 
+// Verify that other changed metadata elements are sent to the sync server when
+// local metadata is updated.
+TEST_F(AutofillWalletMetadataSyncableServiceTest,
+       SendChangedMetadataToServerOnLocalSingleChange) {
+  local_.UpdateAddressStats(BuildAddress(kAddr1, 1, 2, false));
+  local_.UpdateCardStats(BuildCard(kCard1, 3, 4, kAddr1));
+  remote_.UpdateAddressStats(BuildAddress(kAddr1, 1, 2, false));
+  remote_.UpdateCardStats(BuildCard(kCard1, 3, 4, kAddr1));
+  MergeMetadata(&local_, &remote_);
+  AutofillProfile address = BuildAddress(kAddr1, 1, 2, true);
+  CreditCard card = BuildCard(kCard1, 3, 4, kAddr2);
+
+  EXPECT_CALL(
+      local_,
+      SendChangesToSyncServer(ElementsAre(SyncAddressChangeAndDataMatch(
+          syncer::SyncChange::ACTION_UPDATE, kAddr1SyncTag,
+          sync_pb::WalletMetadataSpecifics::ADDRESS, kAddr1Utf8, 1, 2, true))));
+  EXPECT_CALL(local_,
+              SendChangesToSyncServer(ElementsAre(SyncCardChangeAndDataMatch(
+                  syncer::SyncChange::ACTION_UPDATE, kCard1SyncTag,
+                  sync_pb::WalletMetadataSpecifics::CARD, kCard1Utf8, 3, 4,
+                  kAddr2Utf8))));
+
+  local_.AutofillProfileChanged(AutofillProfileChange(
+      AutofillProfileChange::UPDATE, address.guid(), &address));
+  local_.CreditCardChanged(
+      CreditCardChange(CreditCardChange::UPDATE, card.guid(), &card));
+}
+
 // Verify that one-off addition of metadata is not sent to the sync
 // server. Metadata add and delete trigger multiple changes notification
 // instead.
@@ -542,7 +597,7 @@ TEST_F(AutofillWalletMetadataSyncableServiceTest, AddToServerOnMultiChange) {
                               sync_pb::WalletMetadataSpecifics::CARD,
                               kCard2Utf8, 7, 8, kAddr2Utf8))));
 
-  local_.AutofillMultipleChanged();
+  local_.AutofillMultipleChangedBySync();
 }
 
 // Verify that higher values of existing metadata are sent to the sync server
@@ -568,7 +623,7 @@ TEST_F(AutofillWalletMetadataSyncableServiceTest,
                               sync_pb::WalletMetadataSpecifics::CARD,
                               kCard1Utf8, 7, 8, kAddr2Utf8))));
 
-  local_.AutofillMultipleChanged();
+  local_.AutofillMultipleChangedBySync();
 }
 
 // Verify that lower values of existing metadata are not sent to the sync server
@@ -586,7 +641,7 @@ TEST_F(AutofillWalletMetadataSyncableServiceTest,
 
   EXPECT_CALL(local_, SendChangesToSyncServer(_)).Times(0);
 
-  local_.AutofillMultipleChanged();
+  local_.AutofillMultipleChangedBySync();
 }
 
 // Verify that erased local metadata is also erased from the sync server when
@@ -608,7 +663,51 @@ TEST_F(AutofillWalletMetadataSyncableServiceTest,
           SyncChangeMatches(syncer::SyncChange::ACTION_DELETE,
                             kCard1SyncTag))));
 
-  local_.AutofillMultipleChanged();
+  local_.AutofillMultipleChangedBySync();
+}
+
+// Verify that erased local metadata is not erased from the sync server when
+// the service is not tracking Wallet data.
+TEST_F(AutofillWalletMetadataSyncableServiceTest,
+       DeleteFromServerOnMultiChange_NotWhenNotTracking) {
+  local_.OnWalletDataTrackingStateChanged(false);
+
+  local_.UpdateAddressStats(BuildAddress(kAddr1, 1, 2, true));
+  local_.UpdateCardStats(BuildCard(kCard1, 3, 4, kAddr1));
+  remote_.UpdateAddressStats(BuildAddress(kAddr1, 1, 2, true));
+  remote_.UpdateCardStats(BuildCard(kCard1, 3, 4, kAddr1));
+  MergeMetadata(&local_, &remote_);
+  // This method dooes not trigger notifications or sync:
+  local_.ClearServerData();
+
+  EXPECT_CALL(local_, SendChangesToSyncServer(_)).Times(0);
+
+  local_.AutofillMultipleChangedBySync();
+}
+
+// Verify that erased local metadata is also erased from the sync server when
+// we start tracking Wallet data after multiple metadata values change at once.
+TEST_F(AutofillWalletMetadataSyncableServiceTest,
+       DeleteFromServerOnMultiChange_ChangeBeforeTracking) {
+  local_.OnWalletDataTrackingStateChanged(false);
+
+  local_.UpdateAddressStats(BuildAddress(kAddr1, 1, 2, true));
+  local_.UpdateCardStats(BuildCard(kCard1, 3, 4, kAddr1));
+  remote_.UpdateAddressStats(BuildAddress(kAddr1, 1, 2, true));
+  remote_.UpdateCardStats(BuildCard(kCard1, 3, 4, kAddr1));
+  MergeMetadata(&local_, &remote_);
+  // This method dooes not trigger notifications or sync:
+  local_.ClearServerData();
+
+  EXPECT_CALL(
+      local_,
+      SendChangesToSyncServer(UnorderedElementsAre(
+          SyncChangeMatches(syncer::SyncChange::ACTION_DELETE, kAddr1SyncTag),
+          SyncChangeMatches(syncer::SyncChange::ACTION_DELETE,
+                            kCard1SyncTag))));
+
+  local_.AutofillMultipleChangedBySync();
+  local_.OnWalletDataTrackingStateChanged(true);
 }
 
 // Verify that empty sync change from the sync server does not trigger writing
@@ -902,7 +1001,7 @@ TEST_F(AutofillWalletMetadataSyncableServiceTest,
           SyncChangeMatches(syncer::SyncChange::ACTION_DELETE,
                             kCard2SyncTag))));
 
-  local_.AutofillMultipleChanged();
+  local_.AutofillMultipleChangedBySync();
 }
 
 // Verify that Wallet data arriving after metadata will not send lower metadata
@@ -929,7 +1028,7 @@ TEST_F(AutofillWalletMetadataSyncableServiceTest,
                           AutofillCardMetadataMatches(kCard1, 7, 8, kAddr2)));
   EXPECT_CALL(local_, SendChangesToSyncServer(_)).Times(0);
 
-  local_.AutofillMultipleChanged();
+  local_.AutofillMultipleChangedBySync();
 }
 
 // Verify that processing a small subset of metadata changes before any Wallet
@@ -973,7 +1072,7 @@ TEST_F(AutofillWalletMetadataSyncableServiceTest,
                           AutofillCardMetadataMatches(kCard2, 15, 16, kAddr1)));
   EXPECT_CALL(local_, SendChangesToSyncServer(_)).Times(0);
 
-  local_.AutofillMultipleChanged();
+  local_.AutofillMultipleChangedBySync();
 }
 
 // Verify that the merge logic keeps the best data on a field by field basis.
@@ -1231,5 +1330,37 @@ TEST_F(AutofillWalletMetadataSyncableServiceTest, NewLocalCard) {
   MergeMetadata(&local_, &remote_);
 }
 
+// Tests that processing remote changes does not trigger a merge. This is a
+// specific test for reflection blocking in notifications.
+TEST_F(AutofillWalletMetadataSyncableServiceTest,
+       RemoteChangesDoNotTriggerMerge) {
+  // Make the backend broadcast back the notifications it receives
+  ON_CALL(backend_, NotifyOfMultipleAutofillChanges())
+      .WillByDefault(
+          DoAll(Invoke(&local_, &MockService::AutofillMultipleChangedBySync),
+                Invoke(&remote_, &MockService::AutofillMultipleChangedBySync)));
+
+  // Get initial data from |remote_| into |local_|.
+  local_.UpdateAddressStats(BuildAddress(kAddr1, 2, 2, true));
+  local_.UpdateCardStats(BuildCard(kCard1, 5, 6, kAddr1));
+  remote_.UpdateAddressStats(BuildAddress(kAddr1, 2, 2, true));
+  remote_.UpdateCardStats(BuildCard(kCard1, 5, 6, kAddr1));
+  MergeMetadata(&local_, &remote_);
+
+  // Silently update local card in the DB.
+  local_.UpdateCardStats(BuildCard(kCard1, 7, 8, kAddr1));
+
+  // Receive a remote update of the address.
+  syncer::SyncChangeList changes;
+  changes.push_back(BuildAddressChange(
+      syncer::SyncChange::ACTION_UPDATE, kAddr1SyncTag,
+      sync_pb::WalletMetadataSpecifics::ADDRESS, kAddr1Utf8, 10, 20, true));
+
+  // Processing remote change should not trigger a commit of the local change.
+  EXPECT_CALL(local_, SendChangesToSyncServer(_)).Times(0);
+
+  local_.ProcessSyncChanges(FROM_HERE, changes);
+}
+
 }  // namespace
 }  // namespace autofill
