@@ -270,7 +270,7 @@ TEST_F(BookmarkUtilsTest, PasteBookmarkFromURL) {
 
   // Write blank text to clipboard.
   {
-    ui::ScopedClipboardWriter clipboard_writer(ui::CLIPBOARD_TYPE_COPY_PASTE);
+    ui::ScopedClipboardWriter clipboard_writer(ui::ClipboardType::kCopyPaste);
     clipboard_writer.WriteText(base::string16());
   }
   // Now we shouldn't be able to paste from the clipboard.
@@ -278,17 +278,18 @@ TEST_F(BookmarkUtilsTest, PasteBookmarkFromURL) {
 
   // Write some valid url to the clipboard.
   {
-    ui::ScopedClipboardWriter clipboard_writer(ui::CLIPBOARD_TYPE_COPY_PASTE);
+    ui::ScopedClipboardWriter clipboard_writer(ui::ClipboardType::kCopyPaste);
     clipboard_writer.WriteText(url_text);
   }
   // Now we should be able to paste from the clipboard.
   EXPECT_TRUE(CanPasteFromClipboard(model.get(), new_folder));
 
   PasteFromClipboard(model.get(), new_folder, 0);
-  ASSERT_EQ(1, new_folder->child_count());
+  ASSERT_EQ(1u, new_folder->children().size());
 
   // Url for added node should be same as url_text.
-  EXPECT_EQ(url_text, ASCIIToUTF16(new_folder->GetChild(0)->url().spec()));
+  EXPECT_EQ(url_text,
+            ASCIIToUTF16(new_folder->children().front()->url().spec()));
 }
 
 TEST_F(BookmarkUtilsTest, CopyPaste) {
@@ -308,8 +309,7 @@ TEST_F(BookmarkUtilsTest, CopyPaste) {
 
   // Write some text to the clipboard.
   {
-    ui::ScopedClipboardWriter clipboard_writer(
-        ui::CLIPBOARD_TYPE_COPY_PASTE);
+    ui::ScopedClipboardWriter clipboard_writer(ui::ClipboardType::kCopyPaste);
     clipboard_writer.WriteText(ASCIIToUTF16("foo"));
   }
 
@@ -329,8 +329,8 @@ TEST_F(BookmarkUtilsTest, MakeTitleUnique) {
       model->AddURL(bookmark_bar_node, 0, title_text, GURL(url_text));
 
   EXPECT_EQ(url_text,
-            ASCIIToUTF16(bookmark_bar_node->GetChild(0)->url().spec()));
-  EXPECT_EQ(title_text, bookmark_bar_node->GetChild(0)->GetTitle());
+            ASCIIToUTF16(bookmark_bar_node->children()[0]->url().spec()));
+  EXPECT_EQ(title_text, bookmark_bar_node->children()[0]->GetTitle());
 
   // Copy a node to the clipboard.
   std::vector<const BookmarkNode*> nodes;
@@ -341,15 +341,15 @@ TEST_F(BookmarkUtilsTest, MakeTitleUnique) {
   EXPECT_TRUE(CanPasteFromClipboard(model.get(), bookmark_bar_node));
 
   PasteFromClipboard(model.get(), bookmark_bar_node, 1);
-  ASSERT_EQ(2, bookmark_bar_node->child_count());
+  ASSERT_EQ(2u, bookmark_bar_node->children().size());
 
   // Url for added node should be same as url_text.
   EXPECT_EQ(url_text,
-            ASCIIToUTF16(bookmark_bar_node->GetChild(1)->url().spec()));
+            ASCIIToUTF16(bookmark_bar_node->children()[1]->url().spec()));
   // Title for added node should be numeric subscript suffix with copied node
   // title.
   EXPECT_EQ(ASCIIToUTF16("foobar (1)"),
-            bookmark_bar_node->GetChild(1)->GetTitle());
+            bookmark_bar_node->children()[1]->GetTitle());
 }
 
 TEST_F(BookmarkUtilsTest, CopyPasteMetaInfo) {
@@ -369,16 +369,16 @@ TEST_F(BookmarkUtilsTest, CopyPasteMetaInfo) {
   // Paste node to a different folder.
   const BookmarkNode* folder =
       model->AddFolder(model->bookmark_bar_node(), 0, ASCIIToUTF16("Folder"));
-  EXPECT_EQ(0, folder->child_count());
+  EXPECT_EQ(0u, folder->children().size());
 
   // And make sure we can paste a bookmark from the clipboard.
   EXPECT_TRUE(CanPasteFromClipboard(model.get(), folder));
 
   PasteFromClipboard(model.get(), folder, 0);
-  ASSERT_EQ(1, folder->child_count());
+  ASSERT_EQ(1u, folder->children().size());
 
   // Verify that the pasted node contains the same meta info.
-  const BookmarkNode* pasted = folder->GetChild(0);
+  const BookmarkNode* pasted = folder->children().front().get();
   ASSERT_TRUE(pasted->GetMetaInfoMap());
   EXPECT_EQ(2u, pasted->GetMetaInfoMap()->size());
   std::string value;
@@ -410,7 +410,7 @@ TEST_F(BookmarkUtilsTest, MAYBE_CutToClipboard) {
   CopyToClipboard(model.get(), nodes, true);
 
   // Make sure the nodes were removed.
-  EXPECT_EQ(0, model->other_node()->child_count());
+  EXPECT_EQ(0u, model->other_node()->children().size());
 
   // Make sure observers were notified the set of changes should be grouped.
   ExpectGroupedChangeCount(1, 1);
@@ -420,12 +420,12 @@ TEST_F(BookmarkUtilsTest, MAYBE_CutToClipboard) {
 }
 
 TEST_F(BookmarkUtilsTest, PasteNonEditableNodes) {
-  // Load a model with an extra node that is not editable.
+  // Load a model with an managed node that is not editable.
   std::unique_ptr<TestBookmarkClient> client(new TestBookmarkClient());
-  BookmarkPermanentNodeList extra_nodes;
-  extra_nodes.push_back(std::make_unique<BookmarkPermanentNode>(100));
-  BookmarkPermanentNode* extra_node = extra_nodes.back().get();
-  client->SetExtraNodesToLoad(std::move(extra_nodes));
+  auto owned_managed_node =
+      std::make_unique<BookmarkPermanentNode>(100, BookmarkNode::FOLDER);
+  BookmarkPermanentNode* managed_node = owned_managed_node.get();
+  client->SetManagedNodeToLoad(std::move(owned_managed_node));
 
   std::unique_ptr<BookmarkModel> model(
       TestBookmarkClient::CreateModelWithClient(std::move(client)));
@@ -444,8 +444,8 @@ TEST_F(BookmarkUtilsTest, PasteNonEditableNodes) {
 
   // But it can't be pasted into a non-editable folder.
   BookmarkClient* upcast = model->client();
-  EXPECT_FALSE(upcast->CanBeEditedByUser(extra_node));
-  EXPECT_FALSE(CanPasteFromClipboard(model.get(), extra_node));
+  EXPECT_FALSE(upcast->CanBeEditedByUser(managed_node));
+  EXPECT_FALSE(CanPasteFromClipboard(model.get(), managed_node));
 }
 #endif  // !defined(OS_IOS)
 
@@ -455,11 +455,11 @@ TEST_F(BookmarkUtilsTest, GetParentForNewNodes) {
   // folder.
   std::vector<const BookmarkNode*> nodes;
   nodes.push_back(model->bookmark_bar_node());
-  int index = -1;
+  size_t index = size_t{-1};
   const BookmarkNode* real_parent =
       GetParentForNewNodes(model->bookmark_bar_node(), nodes, &index);
   EXPECT_EQ(real_parent, model->bookmark_bar_node());
-  EXPECT_EQ(0, index);
+  EXPECT_EQ(0u, index);
 
   nodes.clear();
 
@@ -472,7 +472,7 @@ TEST_F(BookmarkUtilsTest, GetParentForNewNodes) {
   nodes.push_back(page1);
   real_parent = GetParentForNewNodes(model->bookmark_bar_node(), nodes, &index);
   EXPECT_EQ(real_parent, model->bookmark_bar_node());
-  EXPECT_EQ(1, index);
+  EXPECT_EQ(1u, index);
 
   // This tests the case where selection has more than one item.
   const BookmarkNode* folder1 =
@@ -480,13 +480,13 @@ TEST_F(BookmarkUtilsTest, GetParentForNewNodes) {
   nodes.push_back(folder1);
   real_parent = GetParentForNewNodes(model->bookmark_bar_node(), nodes, &index);
   EXPECT_EQ(real_parent, model->bookmark_bar_node());
-  EXPECT_EQ(2, index);
+  EXPECT_EQ(2u, index);
 
   // This tests the case where selection doesn't contain any items.
   nodes.clear();
   real_parent = GetParentForNewNodes(model->bookmark_bar_node(), nodes, &index);
   EXPECT_EQ(real_parent, model->bookmark_bar_node());
-  EXPECT_EQ(2, index);
+  EXPECT_EQ(2u, index);
 }
 
 // Verifies that meta info is copied when nodes are cloned.
@@ -506,12 +506,12 @@ TEST_F(BookmarkUtilsTest, CloneMetaInfo) {
   std::vector<BookmarkNodeData::Element> elements;
   BookmarkNodeData::Element node_data(node);
   elements.push_back(node_data);
-  EXPECT_EQ(0, folder->child_count());
+  EXPECT_EQ(0u, folder->children().size());
   CloneBookmarkNode(model.get(), elements, folder, 0, false);
-  ASSERT_EQ(1, folder->child_count());
+  ASSERT_EQ(1u, folder->children().size());
 
   // Verify that the cloned node contains the same meta info.
-  const BookmarkNode* clone = folder->GetChild(0);
+  const BookmarkNode* clone = folder->children().front().get();
   ASSERT_TRUE(clone->GetMetaInfoMap());
   EXPECT_EQ(2u, clone->GetMetaInfoMap()->size());
   std::string value;
@@ -537,12 +537,12 @@ TEST_F(BookmarkUtilsTest, CloneBookmarkResetsNonClonedKey) {
 
   // Cloning a bookmark should clear the non cloned key.
   CloneBookmarkNode(model.get(), elements, parent, 0, true);
-  ASSERT_EQ(2, parent->child_count());
+  ASSERT_EQ(2u, parent->children().size());
   std::string value;
-  EXPECT_FALSE(parent->GetChild(0)->GetMetaInfo("foo", &value));
+  EXPECT_FALSE(parent->children().front()->GetMetaInfo("foo", &value));
 
   // Other keys should still be cloned.
-  EXPECT_TRUE(parent->GetChild(0)->GetMetaInfo("bar", &value));
+  EXPECT_TRUE(parent->children().front()->GetMetaInfo("bar", &value));
   EXPECT_EQ("kept value", value);
 }
 
@@ -561,36 +561,36 @@ TEST_F(BookmarkUtilsTest, CloneFolderResetsNonClonedKey) {
 
   // Cloning a folder should clear the non cloned key.
   CloneBookmarkNode(model.get(), elements, parent, 0, true);
-  ASSERT_EQ(2, parent->child_count());
+  ASSERT_EQ(2u, parent->children().size());
   std::string value;
-  EXPECT_FALSE(parent->GetChild(0)->GetMetaInfo("foo", &value));
+  EXPECT_FALSE(parent->children().front()->GetMetaInfo("foo", &value));
 
   // Other keys should still be cloned.
-  EXPECT_TRUE(parent->GetChild(0)->GetMetaInfo("bar", &value));
+  EXPECT_TRUE(parent->children().front()->GetMetaInfo("bar", &value));
   EXPECT_EQ("kept value", value);
 }
 
 TEST_F(BookmarkUtilsTest, RemoveAllBookmarks) {
-  // Load a model with an extra node that is not editable.
+  // Load a model with an managed node that is not editable.
   std::unique_ptr<TestBookmarkClient> client(new TestBookmarkClient());
-  BookmarkPermanentNodeList extra_nodes;
-  extra_nodes.push_back(std::make_unique<BookmarkPermanentNode>(100));
-  BookmarkPermanentNode* extra_node = extra_nodes.back().get();
-  client->SetExtraNodesToLoad(std::move(extra_nodes));
+  auto owned_managed_node =
+      std::make_unique<BookmarkPermanentNode>(100, BookmarkNode::FOLDER);
+  BookmarkPermanentNode* managed_node = owned_managed_node.get();
+  client->SetManagedNodeToLoad(std::move(owned_managed_node));
 
   std::unique_ptr<BookmarkModel> model(
       TestBookmarkClient::CreateModelWithClient(std::move(client)));
-  EXPECT_TRUE(model->bookmark_bar_node()->empty());
-  EXPECT_TRUE(model->other_node()->empty());
-  EXPECT_TRUE(model->mobile_node()->empty());
-  EXPECT_TRUE(extra_node->empty());
+  EXPECT_TRUE(model->bookmark_bar_node()->children().empty());
+  EXPECT_TRUE(model->other_node()->children().empty());
+  EXPECT_TRUE(model->mobile_node()->children().empty());
+  EXPECT_TRUE(managed_node->children().empty());
 
   const base::string16 title = base::ASCIIToUTF16("Title");
   const GURL url("http://google.com");
   model->AddURL(model->bookmark_bar_node(), 0, title, url);
   model->AddURL(model->other_node(), 0, title, url);
   model->AddURL(model->mobile_node(), 0, title, url);
-  model->AddURL(extra_node, 0, title, url);
+  model->AddURL(managed_node, 0, title, url);
 
   std::vector<const BookmarkNode*> nodes;
   model->GetNodesByURL(url, &nodes);
@@ -601,10 +601,10 @@ TEST_F(BookmarkUtilsTest, RemoveAllBookmarks) {
   nodes.clear();
   model->GetNodesByURL(url, &nodes);
   ASSERT_EQ(1u, nodes.size());
-  EXPECT_TRUE(model->bookmark_bar_node()->empty());
-  EXPECT_TRUE(model->other_node()->empty());
-  EXPECT_TRUE(model->mobile_node()->empty());
-  EXPECT_EQ(1, extra_node->child_count());
+  EXPECT_TRUE(model->bookmark_bar_node()->children().empty());
+  EXPECT_TRUE(model->other_node()->children().empty());
+  EXPECT_TRUE(model->mobile_node()->children().empty());
+  EXPECT_EQ(1u, managed_node->children().size());
 }
 
 }  // namespace
