@@ -37,6 +37,7 @@
 #include "components/content_settings/core/browser/user_modifiable_provider.h"
 #include "components/content_settings/core/browser/website_settings_registry.h"
 #include "components/content_settings/core/common/content_settings_pattern.h"
+#include "components/content_settings/core/common/content_settings_types.h"
 #include "components/content_settings/core/common/content_settings_utils.h"
 #include "components/content_settings/core/common/pref_names.h"
 #include "components/pref_registry/pref_registry_syncable.h"
@@ -68,6 +69,8 @@ constexpr ProviderNamesSourceMapEntry kProviderNamesSourceMap[] = {
     {"policy", content_settings::SETTING_SOURCE_POLICY},
     {"supervised_user", content_settings::SETTING_SOURCE_SUPERVISED},
     {"extension", content_settings::SETTING_SOURCE_EXTENSION},
+    {"installed_webapp_provider",
+      content_settings::SETTING_SOURCE_INSTALLED_WEBAPP},
     {"notification_android", content_settings::SETTING_SOURCE_USER},
     {"ephemeral", content_settings::SETTING_SOURCE_USER},
     {"preference", content_settings::SETTING_SOURCE_USER},
@@ -238,8 +241,7 @@ const char* ContentSettingToString(ContentSetting setting) {
 
 HostContentSettingsMap::HostContentSettingsMap(
     PrefService* prefs,
-    bool is_incognito_profile,
-    bool is_guest_profile,
+    bool is_off_the_record,
     bool store_last_modified,
     bool migrate_requesting_and_top_level_origin_settings)
     : RefcountedKeyedService(base::ThreadTaskRunnerHandle::Get()),
@@ -247,11 +249,9 @@ HostContentSettingsMap::HostContentSettingsMap(
       used_from_thread_id_(base::PlatformThread::CurrentId()),
 #endif
       prefs_(prefs),
-      is_incognito_(is_incognito_profile || is_guest_profile),
-      store_last_modified_(store_last_modified),
-      weak_ptr_factory_(this) {
+      is_off_the_record_(is_off_the_record),
+      store_last_modified_(store_last_modified) {
   TRACE_EVENT0("startup", "HostContentSettingsMap::HostContentSettingsMap");
-  DCHECK(!(is_incognito_profile && is_guest_profile));
 
   content_settings::PolicyProvider* policy_provider =
       new content_settings::PolicyProvider(prefs_);
@@ -259,18 +259,12 @@ HostContentSettingsMap::HostContentSettingsMap(
       base::WrapUnique(policy_provider);
   policy_provider->AddObserver(this);
 
-  pref_provider_ = new content_settings::PrefProvider(prefs_, is_incognito_,
-                                                      store_last_modified_);
+  pref_provider_ = new content_settings::PrefProvider(
+      prefs_, is_off_the_record_, store_last_modified_);
   content_settings_providers_[PREF_PROVIDER] = base::WrapUnique(pref_provider_);
   user_modifiable_providers_.push_back(pref_provider_);
   pref_provider_->AddObserver(this);
 
-  // This ensures that content settings are cleared for the guest profile. This
-  // wouldn't be needed except that we used to allow settings to be stored for
-  // the guest profile and so we need to ensure those get cleared.
-  if (is_guest_profile)
-    pref_provider_->ClearPrefs();
-
   content_settings::EphemeralProvider* ephemeral_provider =
       new content_settings::EphemeralProvider(store_last_modified_);
   content_settings_providers_[EPHEMERAL_PROVIDER] =
@@ -279,7 +273,7 @@ HostContentSettingsMap::HostContentSettingsMap(
   ephemeral_provider->AddObserver(this);
 
   auto default_provider = std::make_unique<content_settings::DefaultProvider>(
-      prefs_, is_incognito_);
+      prefs_, is_off_the_record_);
   default_provider->AddObserver(this);
   content_settings_providers_[DEFAULT_PROVIDER] = std::move(default_provider);
 
@@ -360,7 +354,7 @@ ContentSetting HostContentSettingsMap::GetDefaultContentSettingInternal(
       continue;
     ContentSetting default_setting = GetDefaultContentSettingFromProvider(
         content_type, provider_pair.second.get());
-    if (is_incognito_) {
+    if (is_off_the_record_) {
       default_setting = content_settings::ValueToContentSetting(
           ProcessIncognitoInheritanceBehavior(
               content_type,
@@ -425,7 +419,7 @@ void HostContentSettingsMap::GetSettingsForOneType(
   for (const auto& provider_pair : content_settings_providers_) {
     // For each provider, iterate first the incognito-specific rules, then the
     // normal rules.
-    if (is_incognito_) {
+    if (is_off_the_record_) {
       AddSettingsForOneType(provider_pair.second.get(), provider_pair.first,
                             content_type, resource_identifier, settings, true);
     }
@@ -491,9 +485,8 @@ void HostContentSettingsMap::SetWebsiteSettingCustomScope(
   for (const auto& provider_pair : content_settings_providers_) {
     if (provider_pair.second->SetWebsiteSetting(
             primary_pattern, secondary_pattern, content_type,
-            resource_identifier, value.get())) {
+            resource_identifier, std::move(value))) {
       // If successful then ownership is passed to the provider.
-      ignore_result(value.release());
       return;
     }
   }
@@ -588,8 +581,8 @@ void HostContentSettingsMap::SetContentSettingCustomScope(
     ContentSettingsPattern temp_patterns[2];
     std::unique_ptr<base::Value> value(GetContentSettingValueAndPatterns(
         content_settings_providers_[PREF_PROVIDER].get(), url, url,
-        CONTENT_SETTINGS_TYPE_PLUGINS_DATA, resource_identifier, is_incognito_,
-        temp_patterns, temp_patterns + 1));
+        CONTENT_SETTINGS_TYPE_PLUGINS_DATA, resource_identifier,
+        is_off_the_record_, temp_patterns, temp_patterns + 1));
 
     UMA_HISTOGRAM_ENUMERATION(
         "ContentSettings.EphemeralFlashPermission",
@@ -645,6 +638,7 @@ void HostContentSettingsMap::RecordExceptionMetrics() {
     ContentSettingsForOneType settings;
     GetSettingsForOneType(content_type, std::string(), &settings);
     size_t num_exceptions = 0;
+    size_t num_third_party_cookie_allow_exceptions = 0;
     base::flat_map<ContentSetting, size_t> num_exceptions_with_setting;
     const content_settings::ContentSettingsInfo* content_info =
         content_setting_registry->Get(content_type);
@@ -684,6 +678,12 @@ void HostContentSettingsMap::RecordExceptionMetrics() {
         if (content_info)
           ++num_exceptions_with_setting[setting_entry.GetContentSetting()];
         ++num_exceptions;
+        if (content_type == CONTENT_SETTINGS_TYPE_COOKIES &&
+            setting_entry.primary_pattern.MatchesAllHosts() &&
+            !setting_entry.secondary_pattern.MatchesAllHosts() &&
+            setting_entry.GetContentSetting() == CONTENT_SETTING_ALLOW) {
+          num_third_party_cookie_allow_exceptions++;
+        }
       }
     }
 
@@ -706,6 +706,11 @@ void HostContentSettingsMap::RecordExceptionMetrics() {
             1, 1000, 30);
       }
     }
+    if (content_type == CONTENT_SETTINGS_TYPE_COOKIES) {
+      base::UmaHistogramCustomCounts(
+          "ContentSettings.Exceptions.cookies.AllowThirdParty",
+          num_third_party_cookie_allow_exceptions, 1, 1000, 30);
+    }
   }
 }
 
@@ -898,7 +903,8 @@ std::unique_ptr<base::Value> HostContentSettingsMap::GetWebsiteSettingInternal(
   for (; it != content_settings_providers_.end(); ++it) {
     std::unique_ptr<base::Value> value = GetContentSettingValueAndPatterns(
         it->second.get(), primary_url, secondary_url, content_type,
-        resource_identifier, is_incognito_, primary_pattern, secondary_pattern);
+        resource_identifier, is_off_the_record_, primary_pattern,
+        secondary_pattern);
     if (value) {
       if (info)
         info->source = kProviderNamesSourceMap[it->first].provider_source;
