@@ -24,6 +24,8 @@
 #include "base/strings/stringprintf.h"
 #include "base/strings/sys_string_conversions.h"
 #include "base/strings/utf_string_conversions.h"
+#include "base/system/sys_info.h"
+#include "base/time/time.h"
 #include "build/build_config.h"
 #include "components/crash/content/app/crash_reporter_client.h"
 #include "third_party/crashpad/crashpad/client/annotation.h"
@@ -164,11 +166,14 @@ void InitializeCrashpadImpl(bool initial_client,
 
   static crashpad::StringAnnotation<12> pid_key("pid");
 #if defined(OS_POSIX)
-  pid_key.Set(base::IntToString(getpid()));
+  pid_key.Set(base::NumberToString(getpid()));
 #elif defined(OS_WIN)
-  pid_key.Set(base::IntToString(::GetCurrentProcessId()));
+  pid_key.Set(base::NumberToString(::GetCurrentProcessId()));
 #endif
 
+  static crashpad::StringAnnotation<24> osarch_key("osarch");
+  osarch_key.Set(base::SysInfo::OperatingSystemArchitecture());
+
   logging::SetLogMessageHandler(LogMessageHandler);
 
   // If clients called CRASHPAD_SIMULATE_CRASH() instead of
@@ -325,6 +330,14 @@ base::FilePath GetCrashpadDatabasePath() {
 #endif
 }
 
+void ClearReportsBetween(const base::Time& begin, const base::Time& end) {
+#if defined(OS_WIN)
+  ClearReportsBetween_ExportThunk(begin.ToTimeT(), end.ToTimeT());
+#else
+  ClearReportsBetweenImpl(begin.ToTimeT(), end.ToTimeT());
+#endif
+}
+
 void GetReportsImpl(std::vector<Report>* reports) {
   reports->clear();
 
@@ -400,6 +413,21 @@ base::FilePath::StringType::const_pointer GetCrashpadDatabasePathImpl() {
   return g_database_path->value().c_str();
 }
 
+void ClearReportsBetweenImpl(time_t begin, time_t end) {
+  std::vector<Report> reports;
+  GetReports(&reports);
+  for (const Report& report : reports) {
+    // Delete if either time lies in the range, as they both reveal that the
+    // browser was open.
+    if ((begin <= report.capture_time && report.capture_time <= end) ||
+        (begin <= report.upload_time && report.upload_time <= end)) {
+      crashpad::UUID uuid;
+      uuid.InitializeFromString(report.local_id);
+      g_database->DeleteReport(uuid);
+    }
+  }
+}
+
 namespace internal {
 
 crashpad::CrashReportDatabase* GetCrashReportDatabase() {
