@@ -13,6 +13,7 @@
 #include "base/format_macros.h"
 #include "base/logging.h"
 #include "base/macros.h"
+#include "base/metrics/histogram_functions.h"
 #include "base/numerics/safe_conversions.h"
 #include "base/pickle.h"
 #include "base/strings/stringprintf.h"
@@ -24,6 +25,16 @@
 #include "components/download/public/common/download_stats.h"
 #include "components/download/quarantine/quarantine.h"
 #include "crypto/secure_hash.h"
+#include "services/service_manager/public/cpp/connector.h"
+
+#if defined(OS_WIN)
+#include "components/services/quarantine/public/cpp/quarantine_features_win.h"
+#endif  // defined(OS_WIN)
+
+#if defined(OS_ANDROID)
+#include "base/android/content_uri_utils.h"
+#include "components/download/internal/common/android/download_collection_bridge.h"
+#endif  // defined(OS_ANDROID)
 
 #define CONDITIONAL_TRACE(trace)                  \
   do {                                            \
@@ -62,6 +73,29 @@ class FileErrorData : public base::trace_event::ConvertableToTraceFormat {
   DownloadInterruptReason interrupt_reason_;
   DISALLOW_COPY_AND_ASSIGN(FileErrorData);
 };
+
+void InitializeFile(base::File* file, const base::FilePath& file_path) {
+#if defined(OS_ANDROID)
+  if (file_path.IsContentUri()) {
+    *file = DownloadCollectionBridge::OpenIntermediateUri(file_path);
+    return;
+  }
+#endif  // defined(OS_ANDROID)
+  file->Initialize(file_path, base::File::FLAG_OPEN_ALWAYS |
+                                  base::File::FLAG_WRITE |
+                                  base::File::FLAG_READ);
+}
+
+void DeleteFile(const base::FilePath& file_path) {
+#if defined(OS_ANDROID)
+  if (file_path.IsContentUri()) {
+    DownloadCollectionBridge::DeleteIntermediateUri(file_path);
+    return;
+  }
+#endif  // defined(OS_ANDROID)
+  base::DeleteFile(file_path, false);
+}
+
 }  // namespace
 
 BaseFile::BaseFile(uint32_t download_id) : download_id_(download_id) {
@@ -126,9 +160,10 @@ DownloadInterruptReason BaseFile::WriteDataToFile(int64_t offset,
   if (detached_)
     RecordDownloadCount(APPEND_TO_DETACHED_FILE_COUNT);
 
-  if (!file_.IsValid())
+  if (!file_.IsValid()) {
     return LogInterruptReason("No file stream on append", 0,
                               DOWNLOAD_INTERRUPT_REASON_FILE_FAILED);
+  }
 
   // TODO(phajdan.jr): get rid of this check.
   if (data_len == 0)
@@ -176,6 +211,27 @@ DownloadInterruptReason BaseFile::WriteDataToFile(int64_t offset,
   return DOWNLOAD_INTERRUPT_REASON_NONE;
 }
 
+bool BaseFile::ValidateDataInFile(int64_t offset,
+                                  const char* data,
+                                  size_t data_len) {
+  if (!file_.IsValid())
+    return false;
+
+  // Only validate the first chunk of the file. So |offset| cannot be
+  // larger than bytes received.
+  if (offset > bytes_so_far_)
+    return false;
+
+  if (data_len <= 0)
+    return true;
+
+  std::unique_ptr<char[]> buffer(new char[data_len]);
+  if (file_.Read(offset, buffer.get(), data_len) <= 0)
+    return false;
+
+  return memcmp(data, buffer.get(), data_len) == 0;
+}
+
 DownloadInterruptReason BaseFile::Rename(const base::FilePath& new_path) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DownloadInterruptReason rename_result = DOWNLOAD_INTERRUPT_REASON_NONE;
@@ -194,12 +250,21 @@ DownloadInterruptReason BaseFile::Rename(const base::FilePath& new_path) {
   CONDITIONAL_TRACE(BEGIN2("download", "DownloadFileRename", "old_filename",
                            full_path_.AsUTF8Unsafe(), "new_filename",
                            new_path.AsUTF8Unsafe()));
+  bool need_to_move_file = true;
+#if defined(OS_ANDROID)
+  if (new_path.IsContentUri()) {
+    rename_result = DownloadCollectionBridge::MoveFileToIntermediateUri(
+        full_path_, new_path);
+    need_to_move_file = false;
+  }
+#endif
+  if (need_to_move_file) {
+    base::CreateDirectory(new_path.DirName());
 
-  base::CreateDirectory(new_path.DirName());
-
-  // A simple rename wouldn't work here since we want the file to have
-  // permissions / security descriptors that makes sense in the new directory.
-  rename_result = MoveFileAndAdjustPermissions(new_path);
+    // A simple rename wouldn't work here since we want the file to have
+    // permissions / security descriptors that makes sense in the new directory.
+    rename_result = MoveFileAndAdjustPermissions(new_path);
+  }
 
   CONDITIONAL_TRACE(END0("download", "DownloadFileRename"));
 
@@ -219,6 +284,7 @@ DownloadInterruptReason BaseFile::Rename(const base::FilePath& new_path) {
 }
 
 void BaseFile::Detach() {
+  weak_factory_.InvalidateWeakPtrs();
   detached_ = true;
   CONDITIONAL_TRACE(
       INSTANT0("download", "DownloadFileDetached", TRACE_EVENT_SCOPE_THREAD));
@@ -236,7 +302,7 @@ void BaseFile::Cancel() {
   if (!full_path_.empty()) {
     CONDITIONAL_TRACE(
         INSTANT0("download", "DownloadFileDeleted", TRACE_EVENT_SCOPE_THREAD));
-    base::DeleteFile(full_path_, false);
+    DeleteFile(full_path_);
   }
 
   Detach();
@@ -334,9 +400,7 @@ DownloadInterruptReason BaseFile::Open(const std::string& hash_so_far,
 
   // Create a new file if it is not provided.
   if (!file_.IsValid()) {
-    file_.Initialize(full_path_, base::File::FLAG_OPEN_ALWAYS |
-                                     base::File::FLAG_WRITE |
-                                     base::File::FLAG_READ);
+    InitializeFile(&file_, full_path_);
     if (!file_.IsValid()) {
       return LogNetError("Open/Initialize File",
                          net::FileErrorToNetError(file_.error_details()));
@@ -446,10 +510,59 @@ DownloadInterruptReason BaseFile::LogInterruptReason(
   return reason;
 }
 
-#if defined(OS_WIN) || defined(OS_MACOSX) || defined(OS_LINUX)
+#if defined(OS_ANDROID)
+DownloadInterruptReason BaseFile::PublishDownload() {
+  Close();
+  base::FilePath new_path =
+      DownloadCollectionBridge::PublishDownload(full_path_);
+  if (!new_path.empty()) {
+    full_path_ = new_path;
+    return DOWNLOAD_INTERRUPT_REASON_NONE;
+  }
+  return DOWNLOAD_INTERRUPT_REASON_FILE_FAILED;
+}
+#endif  // defined(OS_ANDROID)
 
 namespace {
 
+DownloadInterruptReason QuarantineFileResultToReason(
+    quarantine::mojom::QuarantineFileResult result) {
+  switch (result) {
+    case quarantine::mojom::QuarantineFileResult::OK:
+      return DOWNLOAD_INTERRUPT_REASON_NONE;
+    case quarantine::mojom::QuarantineFileResult::VIRUS_INFECTED:
+      return DOWNLOAD_INTERRUPT_REASON_FILE_VIRUS_INFECTED;
+    case quarantine::mojom::QuarantineFileResult::SECURITY_CHECK_FAILED:
+      return DOWNLOAD_INTERRUPT_REASON_FILE_SECURITY_CHECK_FAILED;
+    case quarantine::mojom::QuarantineFileResult::BLOCKED_BY_POLICY:
+      return DOWNLOAD_INTERRUPT_REASON_FILE_BLOCKED;
+    case quarantine::mojom::QuarantineFileResult::ACCESS_DENIED:
+      return DOWNLOAD_INTERRUPT_REASON_FILE_ACCESS_DENIED;
+
+    case quarantine::mojom::QuarantineFileResult::FILE_MISSING:
+      // Don't have a good interrupt reason here. This return code means that
+      // the file at |full_path_| went missing before QuarantineFile got to
+      // look at it. Not expected to happen, but we've seen instances where a
+      // file goes missing immediately after BaseFile closes the handle.
+      //
+      // Intentionally using a different error message than
+      // SECURITY_CHECK_FAILED in order to distinguish the two.
+      return DOWNLOAD_INTERRUPT_REASON_FILE_FAILED;
+
+    case quarantine::mojom::QuarantineFileResult::ANNOTATION_FAILED:
+      // This means that the mark-of-the-web couldn't be applied. The file is
+      // already on the file system under its final target name.
+      //
+      // Causes of failed annotations typically aren't transient. E.g. the
+      // target file system may not support extended attributes or alternate
+      // streams. We are going to allow these downloads to progress on the
+      // assumption that failures to apply MOTW can't reliably be introduced
+      // remotely.
+      return DOWNLOAD_INTERRUPT_REASON_NONE;
+  }
+  return DOWNLOAD_INTERRUPT_REASON_FILE_FAILED;
+}
+
 // Given a source and a referrer, determines the "safest" URL that can be used
 // to determine the authority of the download source. Returns an empty URL if no
 // HTTP/S URL can be determined for the <|source_url|, |referrer_url|> pair.
@@ -481,7 +594,9 @@ GURL GetEffectiveAuthorityURL(const GURL& source_url,
 
 }  // namespace
 
-DownloadInterruptReason BaseFile::AnnotateWithSourceInformation(
+#if defined(OS_WIN) || defined(OS_MACOSX) || defined(OS_LINUX)
+
+DownloadInterruptReason BaseFile::AnnotateWithSourceInformationSync(
     const std::string& client_guid,
     const GURL& source_url,
     const GURL& referrer_url) {
@@ -495,43 +610,10 @@ DownloadInterruptReason BaseFile::AnnotateWithSourceInformation(
       referrer_url, client_guid);
   CONDITIONAL_TRACE(END0("download", "DownloadFileAnnotate"));
 
-  switch (result) {
-    case QuarantineFileResult::OK:
-      return DOWNLOAD_INTERRUPT_REASON_NONE;
-    case QuarantineFileResult::VIRUS_INFECTED:
-      return DOWNLOAD_INTERRUPT_REASON_FILE_VIRUS_INFECTED;
-    case QuarantineFileResult::SECURITY_CHECK_FAILED:
-      return DOWNLOAD_INTERRUPT_REASON_FILE_SECURITY_CHECK_FAILED;
-    case QuarantineFileResult::BLOCKED_BY_POLICY:
-      return DOWNLOAD_INTERRUPT_REASON_FILE_BLOCKED;
-    case QuarantineFileResult::ACCESS_DENIED:
-      return DOWNLOAD_INTERRUPT_REASON_FILE_ACCESS_DENIED;
-
-    case QuarantineFileResult::FILE_MISSING:
-      // Don't have a good interrupt reason here. This return code means that
-      // the file at |full_path_| went missing before QuarantineFile got to look
-      // at it. Not expected to happen, but we've seen instances where a file
-      // goes missing immediately after BaseFile closes the handle.
-      //
-      // Intentionally using a different error message than
-      // SECURITY_CHECK_FAILED in order to distinguish the two.
-      return DOWNLOAD_INTERRUPT_REASON_FILE_FAILED;
-
-    case QuarantineFileResult::ANNOTATION_FAILED:
-      // This means that the mark-of-the-web couldn't be applied. The file is
-      // already on the file system under its final target name.
-      //
-      // Causes of failed annotations typically aren't transient. E.g. the
-      // target file system may not support extended attributes or alternate
-      // streams. We are going to allow these downloads to progress on the
-      // assumption that failures to apply MOTW can't reliably be introduced
-      // remotely.
-      return DOWNLOAD_INTERRUPT_REASON_NONE;
-  }
-  return DOWNLOAD_INTERRUPT_REASON_FILE_FAILED;
+  return QuarantineFileResultToReason(result);
 }
 #else  // !OS_WIN && !OS_MACOSX && !OS_LINUX
-DownloadInterruptReason BaseFile::AnnotateWithSourceInformation(
+DownloadInterruptReason BaseFile::AnnotateWithSourceInformationSync(
     const std::string& client_guid,
     const GURL& source_url,
     const GURL& referrer_url) {
@@ -539,4 +621,63 @@ DownloadInterruptReason BaseFile::AnnotateWithSourceInformation(
 }
 #endif
 
+void BaseFile::OnFileQuarantined(
+    bool connection_error,
+    quarantine::mojom::QuarantineFileResult result) {
+  base::UmaHistogramBoolean("Download.QuarantineService.ConnectionError",
+                            connection_error);
+
+  DCHECK(on_annotation_done_callback_);
+  quarantine_service_.reset();
+  std::move(on_annotation_done_callback_)
+      .Run(QuarantineFileResultToReason(result));
+}
+
+void BaseFile::OnQuarantineServiceError(const GURL& source_url,
+                                        const GURL& referrer_url) {
+#if defined(OS_WIN)
+  if (base::FeatureList::IsEnabled(quarantine::kOutOfProcessQuarantine)) {
+    OnFileQuarantined(/*connection_error=*/true,
+                      quarantine::SetInternetZoneIdentifierDirectly(
+                          full_path_, source_url, referrer_url));
+    return;
+  }
+#endif  // defined(OS_WIN)
+
+  CHECK(false) << "In-process quarantine service should not have failed.";
+}
+
+void BaseFile::AnnotateWithSourceInformation(
+    const std::string& client_guid,
+    const GURL& source_url,
+    const GURL& referrer_url,
+    std::unique_ptr<service_manager::Connector> connector,
+    OnAnnotationDoneCallback on_annotation_done_callback) {
+  GURL authority_url = GetEffectiveAuthorityURL(source_url, referrer_url);
+  if (!connector) {
+#if defined(OS_WIN)
+    QuarantineFileResult result = quarantine::SetInternetZoneIdentifierDirectly(
+        full_path_, authority_url, referrer_url);
+#else
+    QuarantineFileResult result = QuarantineFileResult::ANNOTATION_FAILED;
+#endif
+    std::move(on_annotation_done_callback)
+        .Run(QuarantineFileResultToReason(result));
+  } else {
+    connector->BindInterface(quarantine::mojom::kServiceName,
+                             mojo::MakeRequest(&quarantine_service_));
+
+    on_annotation_done_callback_ = std::move(on_annotation_done_callback);
+
+    quarantine_service_.set_connection_error_handler(base::BindOnce(
+        &BaseFile::OnQuarantineServiceError, weak_factory_.GetWeakPtr(),
+        authority_url, referrer_url));
+
+    quarantine_service_->QuarantineFile(
+        full_path_, authority_url, referrer_url, client_guid,
+        base::BindOnce(&BaseFile::OnFileQuarantined, weak_factory_.GetWeakPtr(),
+                       false));
+  }
+}
+
 }  // namespace download
