@@ -11,11 +11,14 @@
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted_memory.h"
+#include "base/strings/stringprintf.h"
 #include "base/task/cancelable_task_tracker.h"
 #include "base/test/metrics/histogram_tester.h"
 #include "base/test/mock_callback.h"
+#include "base/test/scoped_feature_list.h"
 #include "base/test/scoped_task_environment.h"
 #include "base/threading/thread_task_runner_handle.h"
+#include "build/build_config.h"
 #include "components/favicon/core/favicon_client.h"
 #include "components/favicon/core/favicon_server_fetcher_params.h"
 #include "components/favicon/core/test/mock_favicon_service.h"
@@ -24,7 +27,6 @@
 #include "components/image_fetcher/core/image_fetcher.h"
 #include "components/image_fetcher/core/mock_image_fetcher.h"
 #include "components/image_fetcher/core/request_metadata.h"
-#include "components/variations/variations_params_manager.h"
 #include "net/traffic_annotation/network_traffic_annotation_test_helper.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -41,41 +43,36 @@ namespace favicon {
 namespace {
 
 using image_fetcher::MockImageFetcher;
+using testing::_;
 using testing::ElementsAre;
-using testing::IsEmpty;
-using testing::IsNull;
 using testing::Eq;
 using testing::HasSubstr;
+using testing::IsEmpty;
+using testing::IsNull;
 using testing::NiceMock;
 using testing::Not;
 using testing::Property;
 using testing::Return;
 using testing::SaveArg;
-using testing::_;
 
 const char kDummyUrl[] = "http://www.example.com";
 const char kDummyIconUrl[] = "http://www.example.com/touch_icon.png";
 const SkColor kTestColor = SK_ColorRED;
+#if defined(OS_IOS)
+const int kMobileSizeInDip = 32;
+#else
+const int kMobileSizeInDip = 24;
+#endif
 
 ACTION_P(PostFetchReply, p0) {
   base::ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE, base::BindOnce(std::move(*arg3), arg0, p0,
-                                image_fetcher::RequestMetadata()));
+      FROM_HERE,
+      base::BindOnce(std::move(*arg2), p0, image_fetcher::RequestMetadata()));
 }
 
 ACTION_P2(PostFetchReplyWithMetadata, p0, p1) {
   base::ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE, base::BindOnce(std::move(*arg3), arg0, p0, p1));
-}
-
-ACTION_P(PostBoolReplyToArg4, p0) {
-  base::ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE, base::BindOnce(std::move(*arg4), p0));
-}
-
-ACTION_P(PostBoolReplyToArg2, p0) {
-  base::ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE, base::BindOnce(std::move(*arg2), p0));
+      FROM_HERE, base::BindOnce(std::move(*arg2), p0, p1));
 }
 
 SkBitmap CreateTestSkBitmap(int w, int h, SkColor color) {
@@ -148,35 +145,40 @@ class LargeIconServiceTest : public testing::Test {
 };
 
 TEST_F(LargeIconServiceTest, ShouldGetFromGoogleServer) {
-  const GURL kExpectedServerUrl(
+  const GURL kExpectedServerUrl(base::StringPrintf(
       "https://t0.gstatic.com/faviconV2?client=chrome&nfrp=2"
-      "&check_seen=true&size=61&min_size=42&max_size=256"
-      "&fallback_opts=TYPE,SIZE,URL&url=http://www.example.com/");
+      "&check_seen=true&size=%d&min_size=16&max_size=256"
+      "&fallback_opts=TYPE,SIZE,URL&url=http://www.example.com/",
+      kMobileSizeInDip * 2));
 
   EXPECT_CALL(mock_favicon_service_, UnableToDownloadFavicon(_)).Times(0);
   EXPECT_CALL(mock_favicon_service_,
               CanSetOnDemandFavicons(GURL(kDummyUrl),
                                      favicon_base::IconType::kTouchIcon, _))
-      .WillOnce(PostBoolReplyToArg2(true));
+      .WillOnce([](auto, auto, base::OnceCallback<void(bool)> callback) {
+        return base::ThreadTaskRunnerHandle::Get()->PostTask(
+            FROM_HERE, base::BindOnce(std::move(callback), true));
+      });
 
   base::MockCallback<favicon_base::GoogleFaviconServerCallback> callback;
   EXPECT_CALL(*mock_image_fetcher_,
-              FetchImageAndData_(_, kExpectedServerUrl, _, _, _))
+              FetchImageAndData_(kExpectedServerUrl, _, _, _))
       .WillOnce(PostFetchReply(gfx::Image::CreateFrom1xBitmap(
           CreateTestSkBitmap(64, 64, kTestColor))));
   EXPECT_CALL(mock_favicon_service_,
               SetOnDemandFavicons(GURL(kDummyUrl), kExpectedServerUrl,
                                   favicon_base::IconType::kTouchIcon, _, _))
-      .WillOnce(PostBoolReplyToArg4(true));
+      .WillOnce(
+          [](auto, auto, auto, auto, base::OnceCallback<void(bool)> callback) {
+            return base::ThreadTaskRunnerHandle::Get()->PostTask(
+                FROM_HERE, base::BindOnce(std::move(callback), true));
+          });
 
   large_icon_service_
       .GetLargeIconOrFallbackStyleFromGoogleServerSkippingLocalCache(
-          favicon::FaviconServerFetcherParams::CreateForMobile(
-              GURL(kDummyUrl),
-              /*min_source_size_in_pixel=*/42,
-              /*desired_size_in_pixel=*/61),
-          /*may_page_url_be_private=*/true, TRAFFIC_ANNOTATION_FOR_TESTS,
-          callback.Get());
+          favicon::FaviconServerFetcherParams::CreateForMobile(GURL(kDummyUrl)),
+          /*may_page_url_be_private=*/true, /*should_trim_page_url_path=*/false,
+          TRAFFIC_ANNOTATION_FOR_TESTS, callback.Get());
 
   EXPECT_CALL(callback,
               Run(favicon_base::GoogleFaviconServerRequestStatus::SUCCESS));
@@ -188,31 +190,38 @@ TEST_F(LargeIconServiceTest, ShouldGetFromGoogleServer) {
 TEST_F(LargeIconServiceTest, ShouldGetFromGoogleServerForDesktop) {
   const GURL kExpectedServerUrl(
       "https://t0.gstatic.com/faviconV2?client=chrome_desktop"
-      "&nfrp=2&check_seen=true&size=32&min_size=32&max_size=256"
+      "&nfrp=2&check_seen=true&size=32&min_size=16&max_size=256"
       "&fallback_opts=TYPE,SIZE,URL&url=http://www.example.com/");
 
   EXPECT_CALL(mock_favicon_service_, UnableToDownloadFavicon(_)).Times(0);
   EXPECT_CALL(mock_favicon_service_,
               CanSetOnDemandFavicons(GURL(kDummyUrl),
                                      favicon_base::IconType::kFavicon, _))
-      .WillOnce(PostBoolReplyToArg2(true));
+      .WillOnce([](auto, auto, base::OnceCallback<void(bool)> callback) {
+        return base::ThreadTaskRunnerHandle::Get()->PostTask(
+            FROM_HERE, base::BindOnce(std::move(callback), true));
+      });
 
   base::MockCallback<favicon_base::GoogleFaviconServerCallback> callback;
   EXPECT_CALL(*mock_image_fetcher_,
-              FetchImageAndData_(_, kExpectedServerUrl, _, _, _))
+              FetchImageAndData_(kExpectedServerUrl, _, _, _))
       .WillOnce(PostFetchReply(gfx::Image::CreateFrom1xBitmap(
           CreateTestSkBitmap(32, 32, kTestColor))));
   EXPECT_CALL(mock_favicon_service_,
               SetOnDemandFavicons(GURL(kDummyUrl), kExpectedServerUrl,
                                   favicon_base::IconType::kFavicon, _, _))
-      .WillOnce(PostBoolReplyToArg4(true));
+      .WillOnce(
+          [](auto, auto, auto, auto, base::OnceCallback<void(bool)> callback) {
+            return base::ThreadTaskRunnerHandle::Get()->PostTask(
+                FROM_HERE, base::BindOnce(std::move(callback), true));
+          });
 
   large_icon_service_
       .GetLargeIconOrFallbackStyleFromGoogleServerSkippingLocalCache(
           favicon::FaviconServerFetcherParams::CreateForDesktop(
               GURL(kDummyUrl)),
-          /*may_page_url_be_private=*/true, TRAFFIC_ANNOTATION_FOR_TESTS,
-          callback.Get());
+          /*may_page_url_be_private=*/true, /*should_trim_page_url_path=*/false,
+          TRAFFIC_ANNOTATION_FOR_TESTS, callback.Get());
 
   EXPECT_CALL(callback,
               Run(favicon_base::GoogleFaviconServerRequestStatus::SUCCESS));
@@ -221,65 +230,26 @@ TEST_F(LargeIconServiceTest, ShouldGetFromGoogleServerForDesktop) {
       "Favicons.LargeIconService.DownloadedSize", 32, /*expected_count=*/1);
 }
 
-TEST_F(LargeIconServiceTest, ShouldGetFromGoogleServerWithCustomUrl) {
-  variations::testing::VariationParamsManager variation_params(
-      "LargeIconServiceFetching",
-      {{"request_format",
-        "https://t0.gstatic.com/"
-        "faviconV2?%ssize=%d&min_size=%d&max_size=%d&url=%s"},
-       {"enforced_min_size_in_pixel", "43"},
-       {"desired_to_max_size_factor", "6.5"}},
-      {"LargeIconServiceFetching"});
-  const GURL kExpectedServerUrl(
-      "https://t0.gstatic.com/faviconV2?check_seen=true&"
-      "size=61&min_size=43&max_size=396&url=http://www.example.com/");
-
-  EXPECT_CALL(mock_favicon_service_, UnableToDownloadFavicon(_)).Times(0);
-  EXPECT_CALL(mock_favicon_service_,
-              CanSetOnDemandFavicons(GURL(kDummyUrl),
-                                     favicon_base::IconType::kTouchIcon, _))
-      .WillOnce(PostBoolReplyToArg2(true));
-
-  base::MockCallback<favicon_base::GoogleFaviconServerCallback> callback;
-  EXPECT_CALL(*mock_image_fetcher_,
-              FetchImageAndData_(_, kExpectedServerUrl, _, _, _))
-      .WillOnce(PostFetchReply(gfx::Image::CreateFrom1xBitmap(
-          CreateTestSkBitmap(64, 64, kTestColor))));
-  EXPECT_CALL(mock_favicon_service_,
-              SetOnDemandFavicons(GURL(kDummyUrl), kExpectedServerUrl,
-                                  favicon_base::IconType::kTouchIcon, _, _))
-      .WillOnce(PostBoolReplyToArg4(true));
-
-  large_icon_service_
-      .GetLargeIconOrFallbackStyleFromGoogleServerSkippingLocalCache(
-          favicon::FaviconServerFetcherParams::CreateForMobile(
-              GURL(kDummyUrl),
-              /*min_source_size_in_pixel=*/42,
-              /*desired_size_in_pixel=*/61),
-          /*may_page_url_be_private=*/true, TRAFFIC_ANNOTATION_FOR_TESTS,
-          callback.Get());
-
-  EXPECT_CALL(callback,
-              Run(favicon_base::GoogleFaviconServerRequestStatus::SUCCESS));
-  scoped_task_environment_.RunUntilIdle();
-}
-
 TEST_F(LargeIconServiceTest, ShouldGetFromGoogleServerWithOriginalUrl) {
-  const GURL kExpectedServerUrl(
+  const GURL kExpectedServerUrl(base::StringPrintf(
       "https://t0.gstatic.com/faviconV2?client=chrome&nfrp=2"
-      "&check_seen=true&size=61&min_size=42&max_size=256"
-      "&fallback_opts=TYPE,SIZE,URL&url=http://www.example.com/");
+      "&check_seen=true&size=%d&min_size=16&max_size=256"
+      "&fallback_opts=TYPE,SIZE,URL&url=http://www.example.com/",
+      kMobileSizeInDip * 2));
   const GURL kExpectedOriginalUrl("http://www.example.com/favicon.png");
 
   EXPECT_CALL(mock_favicon_service_,
               CanSetOnDemandFavicons(GURL(kDummyUrl),
                                      favicon_base::IconType::kTouchIcon, _))
-      .WillOnce(PostBoolReplyToArg2(true));
+      .WillOnce([](auto, auto, base::OnceCallback<void(bool)> callback) {
+        return base::ThreadTaskRunnerHandle::Get()->PostTask(
+            FROM_HERE, base::BindOnce(std::move(callback), true));
+      });
 
   image_fetcher::RequestMetadata expected_metadata;
   expected_metadata.content_location_header = kExpectedOriginalUrl.spec();
   EXPECT_CALL(*mock_image_fetcher_,
-              FetchImageAndData_(_, kExpectedServerUrl, _, _, _))
+              FetchImageAndData_(kExpectedServerUrl, _, _, _))
       .WillOnce(PostFetchReplyWithMetadata(
           gfx::Image::CreateFrom1xBitmap(
               CreateTestSkBitmap(64, 64, kTestColor)),
@@ -287,17 +257,18 @@ TEST_F(LargeIconServiceTest, ShouldGetFromGoogleServerWithOriginalUrl) {
   EXPECT_CALL(mock_favicon_service_,
               SetOnDemandFavicons(GURL(kDummyUrl), kExpectedOriginalUrl,
                                   favicon_base::IconType::kTouchIcon, _, _))
-      .WillOnce(PostBoolReplyToArg4(true));
+      .WillOnce(
+          [](auto, auto, auto, auto, base::OnceCallback<void(bool)> callback) {
+            return base::ThreadTaskRunnerHandle::Get()->PostTask(
+                FROM_HERE, base::BindOnce(std::move(callback), true));
+          });
 
   base::MockCallback<favicon_base::GoogleFaviconServerCallback> callback;
   large_icon_service_
       .GetLargeIconOrFallbackStyleFromGoogleServerSkippingLocalCache(
-          favicon::FaviconServerFetcherParams::CreateForMobile(
-              GURL(kDummyUrl),
-              /*min_source_size_in_pixel=*/42,
-              /*desired_size_in_pixel=*/61),
-          /*may_page_url_be_private=*/true, TRAFFIC_ANNOTATION_FOR_TESTS,
-          callback.Get());
+          favicon::FaviconServerFetcherParams::CreateForMobile(GURL(kDummyUrl)),
+          /*may_page_url_be_private=*/true, /*should_trim_page_url_path=*/false,
+          TRAFFIC_ANNOTATION_FOR_TESTS, callback.Get());
 
   EXPECT_CALL(callback,
               Run(favicon_base::GoogleFaviconServerRequestStatus::SUCCESS));
@@ -306,18 +277,22 @@ TEST_F(LargeIconServiceTest, ShouldGetFromGoogleServerWithOriginalUrl) {
 
 TEST_F(LargeIconServiceTest, ShouldTrimQueryParametersForGoogleServer) {
   const GURL kDummyUrlWithQuery("http://www.example.com?foo=1");
-  const GURL kExpectedServerUrl(
+  const GURL kExpectedServerUrl(base::StringPrintf(
       "https://t0.gstatic.com/faviconV2?client=chrome&nfrp=2"
-      "&check_seen=true&size=61&min_size=42&max_size=256"
-      "&fallback_opts=TYPE,SIZE,URL&url=http://www.example.com/");
+      "&check_seen=true&size=%d&min_size=16&max_size=256"
+      "&fallback_opts=TYPE,SIZE,URL&url=http://www.example.com/",
+      kMobileSizeInDip * 2));
 
   EXPECT_CALL(mock_favicon_service_,
               CanSetOnDemandFavicons(GURL(kDummyUrlWithQuery),
                                      favicon_base::IconType::kTouchIcon, _))
-      .WillOnce(PostBoolReplyToArg2(true));
+      .WillOnce([](auto, auto, base::OnceCallback<void(bool)> callback) {
+        return base::ThreadTaskRunnerHandle::Get()->PostTask(
+            FROM_HERE, base::BindOnce(std::move(callback), true));
+      });
 
   EXPECT_CALL(*mock_image_fetcher_,
-              FetchImageAndData_(_, kExpectedServerUrl, _, _, _))
+              FetchImageAndData_(kExpectedServerUrl, _, _, _))
       .WillOnce(PostFetchReply(gfx::Image::CreateFrom1xBitmap(
           CreateTestSkBitmap(64, 64, kTestColor))));
   // Verify that the non-trimmed page URL is used when writing to the database.
@@ -327,10 +302,9 @@ TEST_F(LargeIconServiceTest, ShouldTrimQueryParametersForGoogleServer) {
   large_icon_service_
       .GetLargeIconOrFallbackStyleFromGoogleServerSkippingLocalCache(
           favicon::FaviconServerFetcherParams::CreateForMobile(
-              GURL(kDummyUrlWithQuery),
-              /*min_source_size_in_pixel=*/42,
-              /*desired_size_in_pixel=*/61),
-          /*may_page_url_be_private=*/true, TRAFFIC_ANNOTATION_FOR_TESTS,
+              GURL(kDummyUrlWithQuery)),
+          /*may_page_url_be_private=*/true, /*should_trim_page_url_path=*/false,
+          TRAFFIC_ANNOTATION_FOR_TESTS,
           favicon_base::GoogleFaviconServerCallback());
 
   scoped_task_environment_.RunUntilIdle();
@@ -340,23 +314,25 @@ TEST_F(LargeIconServiceTest, ShouldNotCheckOnPublicUrls) {
   EXPECT_CALL(mock_favicon_service_,
               CanSetOnDemandFavicons(GURL(kDummyUrl),
                                      favicon_base::IconType::kTouchIcon, _))
-      .WillOnce(PostBoolReplyToArg2(true));
+      .WillOnce([](auto, auto, base::OnceCallback<void(bool)> callback) {
+        return base::ThreadTaskRunnerHandle::Get()->PostTask(
+            FROM_HERE, base::BindOnce(std::move(callback), true));
+      });
+
   // The request has no "check_seen=true"; full URL is tested elsewhere.
-  EXPECT_CALL(*mock_image_fetcher_,
-              FetchImageAndData_(
-                  _, Property(&GURL::query, Not(HasSubstr("check_seen=true"))),
-                  _, _, _))
+  EXPECT_CALL(
+      *mock_image_fetcher_,
+      FetchImageAndData_(
+          Property(&GURL::query, Not(HasSubstr("check_seen=true"))), _, _, _))
       .WillOnce(PostFetchReply(gfx::Image()));
 
   base::MockCallback<favicon_base::GoogleFaviconServerCallback> callback;
 
   large_icon_service_
       .GetLargeIconOrFallbackStyleFromGoogleServerSkippingLocalCache(
-          favicon::FaviconServerFetcherParams::CreateForMobile(
-              GURL(kDummyUrl),
-              /*min_source_size_in_pixel=*/42,
-              /*desired_size_in_pixel=*/61),
-          /*may_page_url_be_private=*/false, TRAFFIC_ANNOTATION_FOR_TESTS,
+          favicon::FaviconServerFetcherParams::CreateForMobile(GURL(kDummyUrl)),
+          /*may_page_url_be_private=*/false,
+          /*should_trim_page_url_path=*/false, TRAFFIC_ANNOTATION_FOR_TESTS,
           callback.Get());
 
   EXPECT_CALL(callback, Run(favicon_base::GoogleFaviconServerRequestStatus::
@@ -367,18 +343,16 @@ TEST_F(LargeIconServiceTest, ShouldNotCheckOnPublicUrls) {
 TEST_F(LargeIconServiceTest, ShouldNotQueryGoogleServerIfInvalidScheme) {
   const GURL kDummyFtpUrl("ftp://www.example.com");
 
-  EXPECT_CALL(*mock_image_fetcher_, FetchImageAndData_(_, _, _, _, _)).Times(0);
+  EXPECT_CALL(*mock_image_fetcher_, FetchImageAndData_(_, _, _, _)).Times(0);
 
   base::MockCallback<favicon_base::GoogleFaviconServerCallback> callback;
 
   large_icon_service_
       .GetLargeIconOrFallbackStyleFromGoogleServerSkippingLocalCache(
           favicon::FaviconServerFetcherParams::CreateForMobile(
-              GURL(kDummyFtpUrl),
-              /*min_source_size_in_pixel=*/42,
-              /*desired_size_in_pixel=*/61),
-          /*may_page_url_be_private=*/true, TRAFFIC_ANNOTATION_FOR_TESTS,
-          callback.Get());
+              GURL(kDummyFtpUrl)),
+          /*may_page_url_be_private=*/true, /*should_trim_page_url_path=*/false,
+          TRAFFIC_ANNOTATION_FOR_TESTS, callback.Get());
 
   EXPECT_CALL(callback, Run(favicon_base::GoogleFaviconServerRequestStatus::
                                 FAILURE_TARGET_URL_SKIPPED));
@@ -391,18 +365,16 @@ TEST_F(LargeIconServiceTest, ShouldNotQueryGoogleServerIfInvalidScheme) {
 TEST_F(LargeIconServiceTest, ShouldNotQueryGoogleServerIfInvalidURL) {
   const GURL kDummyInvalidUrl("htt");
 
-  EXPECT_CALL(*mock_image_fetcher_, FetchImageAndData_(_, _, _, _, _)).Times(0);
+  EXPECT_CALL(*mock_image_fetcher_, FetchImageAndData_(_, _, _, _)).Times(0);
 
   base::MockCallback<favicon_base::GoogleFaviconServerCallback> callback;
 
   large_icon_service_
       .GetLargeIconOrFallbackStyleFromGoogleServerSkippingLocalCache(
           favicon::FaviconServerFetcherParams::CreateForMobile(
-              GURL(kDummyInvalidUrl),
-              /*min_source_size_in_pixel=*/42,
-              /*desired_size_in_pixel=*/61),
-          /*may_page_url_be_private=*/true, TRAFFIC_ANNOTATION_FOR_TESTS,
-          callback.Get());
+              GURL(kDummyInvalidUrl)),
+          /*may_page_url_be_private=*/true, /*should_trim_page_url_path=*/false,
+          TRAFFIC_ANNOTATION_FOR_TESTS, callback.Get());
 
   EXPECT_CALL(callback, Run(favicon_base::GoogleFaviconServerRequestStatus::
                                 FAILURE_TARGET_URL_INVALID));
@@ -413,33 +385,34 @@ TEST_F(LargeIconServiceTest, ShouldNotQueryGoogleServerIfInvalidURL) {
 }
 
 TEST_F(LargeIconServiceTest, ShouldReportUnavailableIfFetchFromServerFails) {
-  const GURL kExpectedServerUrl(
+  const GURL kExpectedServerUrl(base::StringPrintf(
       "https://t0.gstatic.com/faviconV2?client=chrome&nfrp=2"
-      "&check_seen=true&size=61&min_size=42&max_size=256"
-      "&fallback_opts=TYPE,SIZE,URL&url=http://www.example.com/");
+      "&check_seen=true&size=%d&min_size=16&max_size=256"
+      "&fallback_opts=TYPE,SIZE,URL&url=http://www.example.com/",
+      kMobileSizeInDip * 2));
 
   EXPECT_CALL(mock_favicon_service_,
               CanSetOnDemandFavicons(GURL(kDummyUrl),
                                      favicon_base::IconType::kTouchIcon, _))
-      .WillOnce(PostBoolReplyToArg2(true));
+      .WillOnce([](auto, auto, base::OnceCallback<void(bool)> callback) {
+        return base::ThreadTaskRunnerHandle::Get()->PostTask(
+            FROM_HERE, base::BindOnce(std::move(callback), true));
+      });
   EXPECT_CALL(mock_favicon_service_, SetOnDemandFavicons(_, _, _, _, _))
       .Times(0);
 
   base::MockCallback<favicon_base::GoogleFaviconServerCallback> callback;
   EXPECT_CALL(*mock_image_fetcher_,
-              FetchImageAndData_(_, kExpectedServerUrl, _, _, _))
+              FetchImageAndData_(kExpectedServerUrl, _, _, _))
       .WillOnce(PostFetchReply(gfx::Image()));
   EXPECT_CALL(mock_favicon_service_,
               UnableToDownloadFavicon(kExpectedServerUrl));
 
   large_icon_service_
       .GetLargeIconOrFallbackStyleFromGoogleServerSkippingLocalCache(
-          favicon::FaviconServerFetcherParams::CreateForMobile(
-              GURL(kDummyUrl),
-              /*min_source_size_in_pixel=*/42,
-              /*desired_size_in_pixel=*/61),
-          /*may_page_url_be_private=*/true, TRAFFIC_ANNOTATION_FOR_TESTS,
-          callback.Get());
+          favicon::FaviconServerFetcherParams::CreateForMobile(GURL(kDummyUrl)),
+          /*may_page_url_be_private=*/true, /*should_trim_page_url_path=*/false,
+          TRAFFIC_ANNOTATION_FOR_TESTS, callback.Get());
 
   EXPECT_CALL(callback, Run(favicon_base::GoogleFaviconServerRequestStatus::
                                 FAILURE_CONNECTION_ERROR));
@@ -451,26 +424,24 @@ TEST_F(LargeIconServiceTest, ShouldReportUnavailableIfFetchFromServerFails) {
 
 TEST_F(LargeIconServiceTest, ShouldNotGetFromGoogleServerIfUnavailable) {
   ON_CALL(mock_favicon_service_,
-          WasUnableToDownloadFavicon(
-              GURL("https://t0.gstatic.com/faviconV2?client=chrome&nfrp=2"
-                   "&check_seen=true&size=61&min_size=42&max_size=256"
-                   "&fallback_opts=TYPE,SIZE,URL&url=http://www.example.com/")))
+          WasUnableToDownloadFavicon(GURL(base::StringPrintf(
+              "https://t0.gstatic.com/faviconV2?client=chrome&nfrp=2"
+              "&check_seen=true&size=%d&min_size=16&max_size=256"
+              "&fallback_opts=TYPE,SIZE,URL&url=http://www.example.com/",
+              kMobileSizeInDip * 2))))
       .WillByDefault(Return(true));
 
   EXPECT_CALL(mock_favicon_service_, UnableToDownloadFavicon(_)).Times(0);
-  EXPECT_CALL(*mock_image_fetcher_, FetchImageAndData_(_, _, _, _, _)).Times(0);
+  EXPECT_CALL(*mock_image_fetcher_, FetchImageAndData_(_, _, _, _)).Times(0);
   EXPECT_CALL(mock_favicon_service_, SetOnDemandFavicons(_, _, _, _, _))
       .Times(0);
 
   base::MockCallback<favicon_base::GoogleFaviconServerCallback> callback;
   large_icon_service_
       .GetLargeIconOrFallbackStyleFromGoogleServerSkippingLocalCache(
-          favicon::FaviconServerFetcherParams::CreateForMobile(
-              GURL(kDummyUrl),
-              /*min_source_size_in_pixel=*/42,
-              /*desired_size_in_pixel=*/61),
-          /*may_page_url_be_private=*/true, TRAFFIC_ANNOTATION_FOR_TESTS,
-          callback.Get());
+          favicon::FaviconServerFetcherParams::CreateForMobile(GURL(kDummyUrl)),
+          /*may_page_url_be_private=*/true, /*should_trim_page_url_path=*/false,
+          TRAFFIC_ANNOTATION_FOR_TESTS, callback.Get());
 
   EXPECT_CALL(callback, Run(favicon_base::GoogleFaviconServerRequestStatus::
                                 FAILURE_HTTP_ERROR_CACHED));
@@ -485,21 +456,21 @@ TEST_F(LargeIconServiceTest, ShouldNotGetFromGoogleServerIfCannotSet) {
   EXPECT_CALL(mock_favicon_service_,
               CanSetOnDemandFavicons(GURL(kDummyUrl),
                                      favicon_base::IconType::kTouchIcon, _))
-      .WillOnce(PostBoolReplyToArg2(false));
+      .WillOnce([](auto, auto, base::OnceCallback<void(bool)> callback) {
+        return base::ThreadTaskRunnerHandle::Get()->PostTask(
+            FROM_HERE, base::BindOnce(std::move(callback), false));
+      });
 
-  EXPECT_CALL(*mock_image_fetcher_, FetchImageAndData_(_, _, _, _, _)).Times(0);
+  EXPECT_CALL(*mock_image_fetcher_, FetchImageAndData_(_, _, _, _)).Times(0);
   EXPECT_CALL(mock_favicon_service_, SetOnDemandFavicons(_, _, _, _, _))
       .Times(0);
 
   base::MockCallback<favicon_base::GoogleFaviconServerCallback> callback;
   large_icon_service_
       .GetLargeIconOrFallbackStyleFromGoogleServerSkippingLocalCache(
-          favicon::FaviconServerFetcherParams::CreateForMobile(
-              GURL(kDummyUrl),
-              /*min_source_size_in_pixel=*/42,
-              /*desired_size_in_pixel=*/61),
-          /*may_page_url_be_private=*/true, TRAFFIC_ANNOTATION_FOR_TESTS,
-          callback.Get());
+          favicon::FaviconServerFetcherParams::CreateForMobile(GURL(kDummyUrl)),
+          /*may_page_url_be_private=*/true, /*should_trim_page_url_path=*/false,
+          TRAFFIC_ANNOTATION_FOR_TESTS, callback.Get());
 
   EXPECT_CALL(callback, Run(favicon_base::GoogleFaviconServerRequestStatus::
                                 FAILURE_ICON_EXISTS_IN_DB));
@@ -562,12 +533,20 @@ class LargeIconServiceGetterTest : public LargeIconServiceTest,
     }
   }
 
-  void InjectMockResult(
-      const GURL& page_url,
-      const favicon_base::FaviconRawBitmapResult& mock_result) {
+  // The parameter |mock_result| needs to be passed by value otherwise the
+  // lambda injected into the mock may capture a reference to a temporary,
+  // which would cause Undefined Behaviour.
+  void InjectMockResult(const GURL& page_url,
+                        favicon_base::FaviconRawBitmapResult mock_result) {
     ON_CALL(mock_favicon_service_,
             GetLargestRawFaviconForPageURL(page_url, _, _, _, _))
-        .WillByDefault(PostReply<5>(mock_result));
+        .WillByDefault([=](auto, auto, auto,
+                           favicon_base::FaviconRawBitmapCallback callback,
+                           base::CancelableTaskTracker* tracker) {
+          return tracker->PostTask(
+              base::ThreadTaskRunnerHandle::Get().get(), FROM_HERE,
+              base::BindOnce(std::move(callback), mock_result));
+        });
   }
 
  protected:
@@ -755,9 +734,9 @@ TEST_P(LargeIconServiceGetterTest, ShouldRecordMatchesDespiteDifferentUrls) {
 // Every test will appear with suffix /0 (param false) and /1 (param true), e.g.
 //  LargeIconServiceGetterTest.FallbackSinceTooPicky/0: get image.
 //  LargeIconServiceGetterTest.FallbackSinceTooPicky/1: get raw bitmap.
-INSTANTIATE_TEST_CASE_P(,  // Empty instatiation name.
-                        LargeIconServiceGetterTest,
-                        ::testing::Values(false, true));
+INSTANTIATE_TEST_SUITE_P(,  // Empty instatiation name.
+                         LargeIconServiceGetterTest,
+                         ::testing::Values(false, true));
 
 TEST(LargeIconServiceOrganizationNameTest, ShouldGetOrganizationNameForUma) {
   EXPECT_EQ("", LargeIconServiceImpl::GetOrganizationNameForUma(GURL()));
