@@ -15,18 +15,21 @@
 #include "components/offline_items_collection/core/android/offline_item_bridge.h"
 #include "components/offline_items_collection/core/android/offline_item_share_info_bridge.h"
 #include "components/offline_items_collection/core/android/offline_item_visuals_bridge.h"
+#include "components/offline_items_collection/core/jni_headers/OfflineContentAggregatorBridge_jni.h"
 #include "components/offline_items_collection/core/offline_item.h"
 #include "components/offline_items_collection/core/throttled_offline_content_provider.h"
-#include "jni/OfflineContentAggregatorBridge_jni.h"
 
 using base::android::AttachCurrentThread;
 using base::android::ConvertJavaStringToUTF8;
 using base::android::ConvertUTF8ToJavaString;
 using base::android::JavaParamRef;
-using base::android::ScopedJavaLocalRef;
 using base::android::ScopedJavaGlobalRef;
+using base::android::ScopedJavaLocalRef;
 
 namespace offline_items_collection {
+
+using GetVisualsOptions = OfflineContentProvider::GetVisualsOptions;
+
 namespace android {
 
 namespace {
@@ -66,6 +69,11 @@ void ForwardShareInfoToJavaCallback(
           env, std::move(shareInfo)));
 }
 
+void RenameItemCallback(ScopedJavaGlobalRef<jobject> j_callback,
+                        RenameResult result) {
+  base::android::RunIntCallbackAndroid(j_callback, static_cast<int>(result));
+}
+
 void RunGetAllItemsCallback(const base::android::JavaRef<jobject>& j_callback,
                             const std::vector<OfflineItem>& items) {
   JNIEnv* env = AttachCurrentThread();
@@ -202,6 +210,7 @@ void OfflineContentAggregatorBridge::GetVisualsForItem(
   provider_->GetVisualsForItem(
       JNI_OfflineContentAggregatorBridge_CreateContentId(env, j_namespace,
                                                          j_id),
+      GetVisualsOptions::IconOnly(),
       base::BindOnce(&GetVisualsForItemHelperCallback,
                      ScopedJavaGlobalRef<jobject>(env, j_callback)));
 }
@@ -219,6 +228,23 @@ void OfflineContentAggregatorBridge::GetShareInfoForItem(
                      ScopedJavaGlobalRef<jobject>(env, j_callback)));
 }
 
+void OfflineContentAggregatorBridge::RenameItem(
+    JNIEnv* env,
+    const JavaParamRef<jobject>& jobj,
+    const JavaParamRef<jstring>& j_namespace,
+    const JavaParamRef<jstring>& j_id,
+    const JavaParamRef<jstring>& j_name,
+    const JavaParamRef<jobject>& j_callback) {
+  base::OnceCallback<void(RenameResult)> callback = base::BindOnce(
+      &RenameItemCallback,
+      base::android::ScopedJavaGlobalRef<jobject>(env, j_callback));
+
+  provider_->RenameItem(JNI_OfflineContentAggregatorBridge_CreateContentId(
+                            env, j_namespace, j_id),
+                        ConvertJavaStringToUTF8(env, j_name),
+                        std::move(callback));
+}
+
 void OfflineContentAggregatorBridge::OnItemsAdded(
     const OfflineContentProvider::OfflineItemList& items) {
   if (java_ref_.is_null())
@@ -239,13 +265,16 @@ void OfflineContentAggregatorBridge::OnItemRemoved(const ContentId& id) {
       ConvertUTF8ToJavaString(env, id.id));
 }
 
-void OfflineContentAggregatorBridge::OnItemUpdated(const OfflineItem& item) {
+void OfflineContentAggregatorBridge::OnItemUpdated(
+    const OfflineItem& item,
+    const base::Optional<UpdateDelta>& update_delta) {
   if (java_ref_.is_null())
     return;
 
   JNIEnv* env = AttachCurrentThread();
   Java_OfflineContentAggregatorBridge_onItemUpdated(
-      env, java_ref_, OfflineItemBridge::CreateOfflineItem(env, item));
+      env, java_ref_, OfflineItemBridge::CreateOfflineItem(env, item),
+      OfflineItemBridge::CreateUpdateDelta(env, update_delta));
 }
 
 }  // namespace android
