@@ -19,23 +19,23 @@
 #include "components/offline_pages/core/archive_manager.h"
 #include "components/offline_pages/core/client_namespace_constants.h"
 #include "components/offline_pages/core/model/add_page_task.h"
-#include "components/offline_pages/core/model/cleanup_thumbnails_task.h"
+#include "components/offline_pages/core/model/cleanup_visuals_task.h"
 #include "components/offline_pages/core/model/delete_page_task.h"
 #include "components/offline_pages/core/model/get_pages_task.h"
-#include "components/offline_pages/core/model/get_thumbnail_task.h"
-#include "components/offline_pages/core/model/has_thumbnail_task.h"
+#include "components/offline_pages/core/model/get_visuals_task.h"
 #include "components/offline_pages/core/model/mark_page_accessed_task.h"
 #include "components/offline_pages/core/model/offline_page_model_utils.h"
 #include "components/offline_pages/core/model/persistent_page_consistency_check_task.h"
 #include "components/offline_pages/core/model/startup_maintenance_task.h"
-#include "components/offline_pages/core/model/store_thumbnail_task.h"
-#include "components/offline_pages/core/model/update_file_path_task.h"
+#include "components/offline_pages/core/model/store_visuals_task.h"
+#include "components/offline_pages/core/model/update_publish_id_task.h"
+#include "components/offline_pages/core/model/visuals_availability_task.h"
 #include "components/offline_pages/core/offline_clock.h"
+#include "components/offline_pages/core/offline_page_client_policy.h"
 #include "components/offline_pages/core/offline_page_feature.h"
 #include "components/offline_pages/core/offline_page_metadata_store.h"
 #include "components/offline_pages/core/offline_page_model.h"
 #include "components/offline_pages/core/offline_store_utils.h"
-#include "components/offline_pages/core/system_download_manager.h"
 #include "url/gurl.h"
 
 namespace offline_pages {
@@ -65,12 +65,6 @@ SavePageResult ArchiverResultToSavePageResult(ArchiverResult archiver_result) {
       return SavePageResult::ARCHIVE_CREATION_FAILED;
     case ArchiverResult::ERROR_CANCELED:
       return SavePageResult::CANCELLED;
-    case ArchiverResult::ERROR_SECURITY_CERTIFICATE:
-      return SavePageResult::SECURITY_CERTIFICATE_ERROR;
-    case ArchiverResult::ERROR_ERROR_PAGE:
-      return SavePageResult::ERROR_PAGE;
-    case ArchiverResult::ERROR_INTERSTITIAL_PAGE:
-      return SavePageResult::INTERSTITIAL_PAGE;
     case ArchiverResult::ERROR_SKIPPED:
       return SavePageResult::SKIPPED;
     case ArchiverResult::ERROR_DIGEST_CALCULATION_FAILED:
@@ -169,10 +163,6 @@ void OnUpdateFilePathDone(PublishPageCallback publish_done_callback,
 
 }  // namespace
 
-// static
-constexpr base::TimeDelta
-    OfflinePageModelTaskified::kInitialUpgradeSelectionDelay;
-
 // static
 constexpr base::TimeDelta OfflinePageModelTaskified::kMaintenanceTasksDelay;
 
@@ -182,21 +172,17 @@ constexpr base::TimeDelta OfflinePageModelTaskified::kClearStorageInterval;
 OfflinePageModelTaskified::OfflinePageModelTaskified(
     std::unique_ptr<OfflinePageMetadataStore> store,
     std::unique_ptr<ArchiveManager> archive_manager,
-    std::unique_ptr<SystemDownloadManager> download_manager,
+    std::unique_ptr<OfflinePageArchivePublisher> archive_publisher,
     const scoped_refptr<base::SequencedTaskRunner>& task_runner)
     : store_(std::move(store)),
       archive_manager_(std::move(archive_manager)),
-      download_manager_(std::move(download_manager)),
-      policy_controller_(new ClientPolicyController()),
+      archive_publisher_(std::move(archive_publisher)),
       task_queue_(this),
       skip_clearing_original_url_for_testing_(false),
       skip_maintenance_tasks_for_testing_(false),
-      task_runner_(task_runner),
-      weak_ptr_factory_(this) {
+      task_runner_(task_runner) {
   DCHECK_LT(kMaintenanceTasksDelay, OfflinePageMetadataStore::kClosingDelay);
   CreateArchivesDirectoryIfNeeded();
-  // TODO(fgorski): Call from here, when upgrade task is available:
-  // PostSelectItemsMarkedForUpgrade();
 }
 
 OfflinePageModelTaskified::~OfflinePageModelTaskified() {}
@@ -272,38 +258,13 @@ void OfflinePageModelTaskified::MarkPageAccessed(int64_t offline_id) {
   task_queue_.AddTask(std::move(task));
 }
 
-void OfflinePageModelTaskified::DeletePagesByOfflineId(
-    const std::vector<int64_t>& offline_ids,
+void OfflinePageModelTaskified::DeletePagesWithCriteria(
+    const PageCriteria& criteria,
     DeletePageCallback callback) {
-  auto task = DeletePageTask::CreateTaskMatchingOfflineIds(
-      store_.get(),
+  task_queue_.AddTask(DeletePageTask::CreateTaskWithCriteria(
+      store_.get(), criteria,
       base::BindOnce(&OfflinePageModelTaskified::OnDeleteDone,
-                     weak_ptr_factory_.GetWeakPtr(), std::move(callback)),
-      offline_ids);
-  task_queue_.AddTask(std::move(task));
-}
-
-void OfflinePageModelTaskified::DeletePagesByClientIds(
-    const std::vector<ClientId>& client_ids,
-    DeletePageCallback callback) {
-  auto task = DeletePageTask::CreateTaskMatchingClientIds(
-      store_.get(),
-      base::BindOnce(&OfflinePageModelTaskified::OnDeleteDone,
-                     weak_ptr_factory_.GetWeakPtr(), std::move(callback)),
-      client_ids);
-  task_queue_.AddTask(std::move(task));
-}
-
-void OfflinePageModelTaskified::DeletePagesByClientIdsAndOrigin(
-    const std::vector<ClientId>& client_ids,
-    const std::string& origin,
-    DeletePageCallback callback) {
-  auto task = DeletePageTask::CreateTaskMatchingClientIdsAndOrigin(
-      store_.get(),
-      base::BindOnce(&OfflinePageModelTaskified::OnDeleteDone,
-                     weak_ptr_factory_.GetWeakPtr(), std::move(callback)),
-      client_ids, origin);
-  task_queue_.AddTask(std::move(task));
+                     weak_ptr_factory_.GetWeakPtr(), std::move(callback))));
 }
 
 void OfflinePageModelTaskified::DeleteCachedPagesByURLPredicate(
@@ -313,90 +274,37 @@ void OfflinePageModelTaskified::DeleteCachedPagesByURLPredicate(
       store_.get(),
       base::BindOnce(&OfflinePageModelTaskified::OnDeleteDone,
                      weak_ptr_factory_.GetWeakPtr(), std::move(callback)),
-      policy_controller_.get(), predicate);
+      predicate);
   task_queue_.AddTask(std::move(task));
 }
 
 void OfflinePageModelTaskified::GetAllPages(
     MultipleOfflinePageItemCallback callback) {
-  auto task = GetPagesTask::CreateTaskMatchingAllPages(
-      store_.get(),
-      base::BindOnce(&ReportSavedPagesCount, std::move(callback)));
-  task_queue_.AddTask(std::move(task));
+  GetPagesWithCriteria(PageCriteria(), base::BindOnce(&ReportSavedPagesCount,
+                                                      std::move(callback)));
   ScheduleMaintenanceTasks();
 }
 
 void OfflinePageModelTaskified::GetPageByOfflineId(
     int64_t offline_id,
     SingleOfflinePageItemCallback callback) {
-  auto task = GetPagesTask::CreateTaskMatchingOfflineId(
-      store_.get(), std::move(callback), offline_id);
-  task_queue_.AddTask(std::move(task));
+  PageCriteria criteria;
+  criteria.offline_ids = std::vector<int64_t>{offline_id};
+  // Adapt multiple result to single result callback.
+  auto wrapped_callback = base::BindOnce(
+      [](SingleOfflinePageItemCallback callback,
+         const std::vector<OfflinePageItem>& pages) {
+        std::move(callback).Run(pages.empty() ? nullptr : &pages[0]);
+      },
+      std::move(callback));
+  GetPagesWithCriteria(criteria, std::move(wrapped_callback));
 }
 
-void OfflinePageModelTaskified::GetPageByGuid(
-    const std::string& guid,
-    SingleOfflinePageItemCallback callback) {
-  auto task = GetPagesTask::CreateTaskMatchingGuid(store_.get(),
-                                                   std::move(callback), guid);
-  task_queue_.AddTask(std::move(task));
-}
-
-void OfflinePageModelTaskified::GetPagesByClientIds(
-    const std::vector<ClientId>& client_ids,
+void OfflinePageModelTaskified::GetPagesWithCriteria(
+    const PageCriteria& criteria,
     MultipleOfflinePageItemCallback callback) {
-  auto task = GetPagesTask::CreateTaskMatchingClientIds(
-      store_.get(), std::move(callback), client_ids);
-  task_queue_.AddTask(std::move(task));
-}
-
-void OfflinePageModelTaskified::GetPagesByURL(
-    const GURL& url,
-    MultipleOfflinePageItemCallback callback) {
-  auto task = GetPagesTask::CreateTaskMatchingUrl(store_.get(),
-                                                  std::move(callback), url);
-  task_queue_.AddTask(std::move(task));
-}
-
-void OfflinePageModelTaskified::GetPagesByNamespace(
-    const std::string& name_space,
-    MultipleOfflinePageItemCallback callback) {
-  auto task = GetPagesTask::CreateTaskMatchingNamespace(
-      store_.get(), std::move(callback), name_space);
-  task_queue_.AddTask(std::move(task));
-}
-
-void OfflinePageModelTaskified::GetPagesRemovedOnCacheReset(
-    MultipleOfflinePageItemCallback callback) {
-  auto task = GetPagesTask::CreateTaskMatchingPagesRemovedOnCacheReset(
-      store_.get(), std::move(callback), policy_controller_.get());
-  task_queue_.AddTask(std::move(task));
-}
-
-void OfflinePageModelTaskified::GetPagesSupportedByDownloads(
-    MultipleOfflinePageItemCallback callback) {
-  auto task = GetPagesTask::CreateTaskMatchingPagesSupportedByDownloads(
-      store_.get(), std::move(callback), policy_controller_.get());
-  task_queue_.AddTask(std::move(task));
-}
-
-void OfflinePageModelTaskified::GetPagesByRequestOrigin(
-    const std::string& request_origin,
-    MultipleOfflinePageItemCallback callback) {
-  auto task = GetPagesTask::CreateTaskMatchingRequestOrigin(
-      store_.get(), std::move(callback), request_origin);
-  task_queue_.AddTask(std::move(task));
-}
-
-void OfflinePageModelTaskified::GetPageBySizeAndDigest(
-    int64_t file_size,
-    const std::string& digest,
-    SingleOfflinePageItemCallback callback) {
-  DCHECK_GT(file_size, 0);
-  DCHECK(!digest.empty());
-  auto task = GetPagesTask::CreateTaskMatchingSizeAndDigest(
-      store_.get(), std::move(callback), file_size, digest);
-  task_queue_.AddTask(std::move(task));
+  task_queue_.AddTask(std::make_unique<GetPagesTask>(store_.get(), criteria,
+                                                     std::move(callback)));
 }
 
 void OfflinePageModelTaskified::GetOfflineIdsForClientId(
@@ -405,38 +313,45 @@ void OfflinePageModelTaskified::GetOfflineIdsForClientId(
   // We're currently getting offline IDs by querying offline items based on
   // client ids, and then extract the offline IDs from the items. This is fine
   // since we're not expecting many pages with the same client ID.
-  auto task = GetPagesTask::CreateTaskMatchingClientIds(
-      store_.get(),
-      base::BindOnce(&WrapInMultipleItemsCallback, std::move(callback)),
-      {client_id});
-  task_queue_.AddTask(std::move(task));
+  PageCriteria criteria;
+  criteria.client_ids = std::vector<ClientId>{client_id};
+  GetPagesWithCriteria(criteria, base::BindOnce(&WrapInMultipleItemsCallback,
+                                                std::move(callback)));
 }
 
-void OfflinePageModelTaskified::StoreThumbnail(
-    const OfflinePageThumbnail& thumb) {
-  task_queue_.AddTask(std::make_unique<StoreThumbnailTask>(
-      store_.get(), thumb,
+void OfflinePageModelTaskified::StoreThumbnail(int64_t offline_id,
+                                               std::string thumbnail) {
+  task_queue_.AddTask(StoreVisualsTask::MakeStoreThumbnailTask(
+      store_.get(), offline_id, std::move(thumbnail),
       base::BindOnce(&OfflinePageModelTaskified::OnStoreThumbnailDone,
-                     weak_ptr_factory_.GetWeakPtr(), thumb)));
+                     weak_ptr_factory_.GetWeakPtr(), offline_id)));
 }
 
-void OfflinePageModelTaskified::GetThumbnailByOfflineId(
-    int64_t offline_id,
-    base::OnceCallback<void(std::unique_ptr<OfflinePageThumbnail>)> callback) {
-  task_queue_.AddTask(std::make_unique<GetThumbnailTask>(
-      store_.get(), offline_id, std::move(callback)));
+void OfflinePageModelTaskified::StoreFavicon(int64_t offline_id,
+                                             std::string favicon) {
+  task_queue_.AddTask(StoreVisualsTask::MakeStoreFaviconTask(
+      store_.get(), offline_id, std::move(favicon),
+      base::BindOnce(&OfflinePageModelTaskified::OnStoreFaviconDone,
+                     weak_ptr_factory_.GetWeakPtr(), offline_id)));
 }
 
-void OfflinePageModelTaskified::HasThumbnailForOfflineId(
+void OfflinePageModelTaskified::GetVisualsByOfflineId(
     int64_t offline_id,
-    base::OnceCallback<void(bool)> callback) {
-  task_queue_.AddTask(std::make_unique<HasThumbnailTask>(
+    base::OnceCallback<void(std::unique_ptr<OfflinePageVisuals>)> callback) {
+  task_queue_.AddTask(std::make_unique<GetVisualsTask>(store_.get(), offline_id,
+                                                       std::move(callback)));
+}
+
+void OfflinePageModelTaskified::GetVisualsAvailability(
+    int64_t offline_id,
+    base::OnceCallback<void(VisualsAvailability)> callback) {
+  task_queue_.AddTask(std::make_unique<VisualsAvailabilityTask>(
       store_.get(), offline_id, std::move(callback)));
 }
 
 const base::FilePath& OfflinePageModelTaskified::GetInternalArchiveDirectory(
     const std::string& name_space) const {
-  if (policy_controller_->IsRemovedOnCacheReset(name_space))
+  if (GetPolicy(name_space).lifetime_type == LifetimeType::TEMPORARY)
     return archive_manager_->GetTemporaryArchivesDir();
   return archive_manager_->GetPrivateArchivesDir();
 }
@@ -451,10 +366,6 @@ bool OfflinePageModelTaskified::IsArchiveInInternalDir(
          archive_manager_->GetPrivateArchivesDir().IsParent(file_path);
 }
 
-ClientPolicyController* OfflinePageModelTaskified::GetPolicyController() {
-  return policy_controller_.get();
-}
-
 OfflineEventLogger* OfflinePageModelTaskified::GetLogger() {
   return &offline_event_logger_;
 }
@@ -517,22 +428,18 @@ void OfflinePageModelTaskified::OnCreateArchiveDone(
   // the completion of certain action, i.e., authentication, in the middle.
   if (skip_clearing_original_url_for_testing_ ||
       save_page_params.original_url != offline_page.url) {
-    offline_page.original_url = save_page_params.original_url;
+    offline_page.original_url_if_different = save_page_params.original_url;
   }
 
-  if (IsOfflinePagesSharingEnabled() &&
-      policy_controller_->IsUserRequestedDownload(
-          offline_page.client_id.name_space)) {
-    // If the user intentionally downloaded the page, move it to a public place.
-    // Note: Moving the archiver instance into the callback so it won't be
-    // deleted.
-    OfflinePageArchiver* raw_archiver = archiver.get();
-    raw_archiver->PublishArchive(
-        offline_page, task_runner_, archive_manager_->GetPublicArchivesDir(),
-        download_manager_.get(),
+  if (GetPolicy(offline_page.client_id.name_space).lifetime_type ==
+      LifetimeType::PERSISTENT) {
+    // If the user intentionally downloaded the page (aka it belongs to a
+    // persistent namespace), move it to a public place.
+    archive_publisher_->PublishArchive(
+        offline_page, task_runner_,
         base::BindOnce(&OfflinePageModelTaskified::PublishArchiveDone,
-                       weak_ptr_factory_.GetWeakPtr(), std::move(archiver),
-                       std::move(callback), OfflineTimeNow()));
+                       weak_ptr_factory_.GetWeakPtr(), std::move(callback),
+                       OfflineTimeNow()));
     return;
   }
 
@@ -548,7 +455,6 @@ void OfflinePageModelTaskified::OnCreateArchiveDone(
 }
 
 void OfflinePageModelTaskified::PublishArchiveDone(
-    std::unique_ptr<OfflinePageArchiver> archiver,
     SavePageCallback save_page_callback,
     base::Time publish_start_time,
     const OfflinePageItem& offline_page,
@@ -569,8 +475,8 @@ void OfflinePageModelTaskified::PublishArchiveDone(
                           add_page_start_time - publish_start_time);
 
   OfflinePageItem page = offline_page;
-  page.file_path = publish_results.new_file_path;
-  page.system_download_id = publish_results.download_id;
+  page.file_path = publish_results.id.new_file_path;
+  page.system_download_id = publish_results.id.download_id;
 
   AddPage(page, base::BindOnce(
                     &OfflinePageModelTaskified::OnAddPageForSavePageDone,
@@ -580,39 +486,34 @@ void OfflinePageModelTaskified::PublishArchiveDone(
 
 void OfflinePageModelTaskified::PublishInternalArchive(
     const OfflinePageItem& offline_page,
-    std::unique_ptr<OfflinePageArchiver> archiver,
     PublishPageCallback publish_done_callback) {
-  // Note: the archiver instance must be kept alive until the final callback
-  // coming from it takes place.
-  OfflinePageArchiver* raw_archiver = archiver.get();
-  raw_archiver->PublishArchive(
-      offline_page, task_runner_, archive_manager_->GetPublicArchivesDir(),
-      download_manager_.get(),
+  archive_publisher_->PublishArchive(
+      offline_page, task_runner_,
       base::BindOnce(&OfflinePageModelTaskified::PublishInternalArchiveDone,
-                     weak_ptr_factory_.GetWeakPtr(), std::move(archiver),
+                     weak_ptr_factory_.GetWeakPtr(),
                      std::move(publish_done_callback)));
 }
 
 void OfflinePageModelTaskified::PublishInternalArchiveDone(
-    std::unique_ptr<OfflinePageArchiver> archiver,
     PublishPageCallback publish_done_callback,
     const OfflinePageItem& offline_page,
     PublishArchiveResult publish_results) {
-  base::FilePath file_path = publish_results.new_file_path;
   SavePageResult result = publish_results.move_result;
   // Call the callback with success == false if we failed to move the page.
   if (result != SavePageResult::SUCCESS) {
-    std::move(publish_done_callback).Run(file_path, result);
+    std::move(publish_done_callback)
+        .Run(publish_results.id.new_file_path, result);
     return;
   }
 
   // Update the OfflinePageModel with the new location for the page, which is
-  // found in move_results.new_file_path, and with the download ID found at
-  // move_results.download_id.  Return the updated offline_page to the callback.
-  auto task = std::make_unique<UpdateFilePathTask>(
-      store_.get(), offline_page.offline_id, file_path,
+  // found in move_results.id.new_file_path, and with the download ID found at
+  // move_results.id.download_id.  Return the updated offline_page to the
+  // callback.
+  auto task = std::make_unique<UpdatePublishIdTask>(
+      store_.get(), offline_page.offline_id, publish_results.id,
       base::BindOnce(&OnUpdateFilePathDone, std::move(publish_done_callback),
-                     file_path, result));
+                     publish_results.id.new_file_path, result));
   task_queue_.AddTask(std::move(task));
 }
 
@@ -637,8 +538,8 @@ void OfflinePageModelTaskified::OnAddPageForSavePageDone(
                                              successful_finish_time);
     // TODO(romax): Just keep the same with logic in OPMImpl (which was wrong).
     // This should be fixed once we have the new strategy for clearing pages.
-    if (policy_controller_->GetPolicy(page_attempted.client_id.name_space)
-            .pages_allowed_per_url != kUnlimitedPages) {
+    if (GetPolicy(page_attempted.client_id.name_space).pages_allowed_per_url !=
+        kUnlimitedPages) {
       RemovePagesMatchingUrlAndNamespace(page_attempted);
     }
     offline_event_logger_.RecordPageSaved(page_attempted.client_id.name_space,
@@ -661,47 +562,55 @@ void OfflinePageModelTaskified::OnAddPageDone(const OfflinePageItem& page,
 void OfflinePageModelTaskified::OnDeleteDone(
     DeletePageCallback callback,
     DeletePageResult result,
-    const std::vector<OfflinePageModel::DeletedPageInfo>& infos) {
+    const std::vector<OfflinePageItem>& deleted_items) {
   UMA_HISTOGRAM_ENUMERATION("OfflinePages.DeletePageResult", result);
-  std::vector<int64_t> system_download_ids;
+  std::vector<PublishedArchiveId> publish_ids;
 
   // Notify observers and run callback.
-  for (const auto& info : infos) {
+  for (const auto& item : deleted_items) {
     UMA_HISTOGRAM_ENUMERATION(
         "OfflinePages.DeletePageCount",
-        model_utils::ToNamespaceEnum(info.client_id.name_space));
-    offline_event_logger_.RecordPageDeleted(info.offline_id);
+        model_utils::ToNamespaceEnum(item.client_id.name_space));
+    offline_event_logger_.RecordPageDeleted(item.offline_id);
     for (Observer& observer : observers_)
-      observer.OfflinePageDeleted(info);
-    if (info.system_download_id != 0)
-      system_download_ids.push_back(info.system_download_id);
+      observer.OfflinePageDeleted(item);
+
+    publish_ids.emplace_back(item.system_download_id, item.file_path);
   }
 
   // Remove the page from the system download manager. We don't need to wait for
   // completion before calling the delete page callback.
-  task_runner_->PostTask(
-      FROM_HERE,
-      base::BindOnce(&OfflinePageModelTaskified::RemoveFromDownloadManager,
-                     download_manager_.get(), system_download_ids));
+  task_runner_->PostTask(FROM_HERE,
+                         base::BindOnce(&OfflinePageModelTaskified::Unpublish,
+                                        archive_publisher_.get(), publish_ids));
 
   if (!callback.is_null())
     std::move(callback).Run(result);
 }
 
-void OfflinePageModelTaskified::OnStoreThumbnailDone(
-    const OfflinePageThumbnail& thumbnail,
-    bool success) {
+void OfflinePageModelTaskified::OnStoreThumbnailDone(int64_t offline_id,
+                                                     bool success,
+                                                     std::string thumbnail) {
   if (success) {
     for (Observer& observer : observers_)
-      observer.ThumbnailAdded(this, thumbnail);
+      observer.ThumbnailAdded(this, offline_id, thumbnail);
   }
 }
 
-void OfflinePageModelTaskified::RemoveFromDownloadManager(
-    SystemDownloadManager* download_manager,
-    const std::vector<int64_t>& system_download_ids) {
-  if (system_download_ids.size() > 0)
-    download_manager->Remove(system_download_ids);
+void OfflinePageModelTaskified::OnStoreFaviconDone(int64_t offline_id,
+                                                   bool success,
+                                                   std::string favicon) {
+  if (success) {
+    for (Observer& observer : observers_)
+      observer.FaviconAdded(this, offline_id, favicon);
+  }
+}
+
+void OfflinePageModelTaskified::Unpublish(
+    OfflinePageArchivePublisher* publisher,
+    const std::vector<PublishedArchiveId>& publish_ids) {
+  if (!publish_ids.empty())
+    publisher->UnpublishArchives(publish_ids);
 }
 
 void OfflinePageModelTaskified::ScheduleMaintenanceTasks() {
@@ -730,20 +639,20 @@ void OfflinePageModelTaskified::RunMaintenanceTasks(base::Time now,
   // reporting storage usage UMA.
   if (first_run) {
     task_queue_.AddTask(std::make_unique<StartupMaintenanceTask>(
-        store_.get(), archive_manager_.get(), policy_controller_.get()));
+        store_.get(), archive_manager_.get()));
 
-    task_queue_.AddTask(std::make_unique<CleanupThumbnailsTask>(
+    task_queue_.AddTask(std::make_unique<CleanupVisualsTask>(
         store_.get(), OfflineTimeNow(), base::DoNothing()));
   }
 
   task_queue_.AddTask(std::make_unique<ClearStorageTask>(
-      store_.get(), archive_manager_.get(), policy_controller_.get(), now,
+      store_.get(), archive_manager_.get(), now,
       base::BindOnce(&OfflinePageModelTaskified::OnClearCachedPagesDone,
                      weak_ptr_factory_.GetWeakPtr())));
 
   // TODO(https://crbug.com/834902) This might need a better execution plan.
   task_queue_.AddTask(std::make_unique<PersistentPageConsistencyCheckTask>(
-      store_.get(), archive_manager_.get(), policy_controller_.get(), now,
+      store_.get(), archive_manager_.get(), now,
       base::BindOnce(
           &OfflinePageModelTaskified::OnPersistentPageConsistencyCheckDone,
           weak_ptr_factory_.GetWeakPtr())));
@@ -751,12 +660,8 @@ void OfflinePageModelTaskified::RunMaintenanceTasks(base::Time now,
 
 void OfflinePageModelTaskified::OnPersistentPageConsistencyCheckDone(
     bool success,
-    const std::vector<int64_t>& pages_deleted) {
-  // If there's no persistent page expired, save some effort by exiting early.
-  // TODO(https://crbug.com/834909): Use the temporary hidden bit in
-  // DownloadUIAdapter instead of calling remove directly.
-  if (pages_deleted.size() > 0)
-    download_manager_->Remove(pages_deleted);
+    const std::vector<PublishedArchiveId>& ids_of_deleted_pages) {
+  Unpublish(archive_publisher_.get(), ids_of_deleted_pages);
 }
 
 void OfflinePageModelTaskified::OnClearCachedPagesDone(
@@ -769,36 +674,6 @@ void OfflinePageModelTaskified::OnClearCachedPagesDone(
   }
 }
 
-void OfflinePageModelTaskified::PostSelectItemsMarkedForUpgrade() {
-  // TODO(fgorski): Make storage permission check. Here or later?
-  // TODO(fgorski): Check disk space here.
-  if (!IsOfflinePagesSharingEnabled())
-    return;
-
-  base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-      FROM_HERE,
-      base::BindRepeating(
-          &OfflinePageModelTaskified::SelectItemsMarkedForUpgrade,
-          weak_ptr_factory_.GetWeakPtr()),
-      kInitialUpgradeSelectionDelay);
-}
-
-void OfflinePageModelTaskified::SelectItemsMarkedForUpgrade() {
-  // TODO(fgorski): Add legacy Persistent path in archive manager to know which
-  // files still need upgrade.
-  auto task = GetPagesTask::CreateTaskSelectingItemsMarkedForUpgrade(
-      store_.get(),
-      base::BindRepeating(
-          &OfflinePageModelTaskified::OnSelectItemsMarkedForUpgradeDone,
-          weak_ptr_factory_.GetWeakPtr()));
-  task_queue_.AddTask(std::move(task));
-}
-
-void OfflinePageModelTaskified::OnSelectItemsMarkedForUpgradeDone(
-    const MultipleOfflinePageItemResult& pages_for_upgrade) {
-  // TODO(fgorski): Save the list of ID to feed them into the upgrade task.
-}
-
 void OfflinePageModelTaskified::RemovePagesMatchingUrlAndNamespace(
     const OfflinePageItem& page) {
   auto task = DeletePageTask::CreateTaskDeletingForPageLimit(
@@ -806,7 +681,7 @@ void OfflinePageModelTaskified::RemovePagesMatchingUrlAndNamespace(
       base::BindOnce(&OfflinePageModelTaskified::OnDeleteDone,
                      weak_ptr_factory_.GetWeakPtr(),
                      base::DoNothing::Once<DeletePageResult>()),
-      policy_controller_.get(), page);
+      page);
   task_queue_.AddTask(std::move(task));
 }
 
