@@ -16,10 +16,10 @@
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/metrics/histogram_tester.h"
 #include "build/build_config.h"
+#include "components/autofill/core/browser/field_types.h"
 #include "components/autofill/core/common/form_data.h"
 #include "components/autofill/core/common/form_field_data.h"
 #include "components/autofill/core/common/password_form.h"
-#include "components/autofill/core/common/submission_indicator_event.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "url/gurl.h"
@@ -28,7 +28,7 @@ using autofill::FieldPropertiesFlags;
 using autofill::FormData;
 using autofill::FormFieldData;
 using autofill::PasswordForm;
-using autofill::SubmissionIndicatorEvent;
+using autofill::mojom::SubmissionIndicatorEvent;
 using base::ASCIIToUTF16;
 
 namespace password_manager {
@@ -109,7 +109,7 @@ uint32_t GetUniqueId() {
 // Use to add a number suffix which is unique in the scope of the test.
 base::string16 StampUniqueSuffix(const char* base_str) {
   return ASCIIToUTF16(base_str) + ASCIIToUTF16("_") +
-         base::UintToString16(GetUniqueId());
+         base::NumberToString16(GetUniqueId());
 }
 
 // Describes which renderer IDs are expected for username/password fields
@@ -169,12 +169,12 @@ FormData GetFormDataAndExpectation(const FormParsingTestCase& test_case,
                                    ParseResultIds* save_result) {
   FormData form_data;
   form_data.action = GURL("http://example1.com");
-  form_data.origin = GURL("http://example2.com");
+  form_data.url = GURL("http://example2.com");
   form_data.submission_event = test_case.submission_event;
   for (const FieldDataDescription& field_description : test_case.fields) {
     FormFieldData field;
-    const uint32_t unique_id = GetUniqueId();
-    field.unique_renderer_id = unique_id;
+    const uint32_t renderer_id = GetUniqueId();
+    field.unique_renderer_id = renderer_id;
     field.id_attribute = StampUniqueSuffix("html_id");
     if (field_description.name == kNonimportantValue) {
       field.name = StampUniqueSuffix("html_name");
@@ -182,6 +182,9 @@ FormData GetFormDataAndExpectation(const FormParsingTestCase& test_case,
       field.name = ASCIIToUTF16(field_description.name);
     }
     field.name_attribute = field.name;
+#if defined(OS_IOS)
+    field.unique_id = StampUniqueSuffix("unique_id");
+#endif
     field.form_control_type = field_description.form_control_type;
     field.is_focusable = field_description.is_focusable;
     field.is_enabled = field_description.is_enabled;
@@ -198,16 +201,22 @@ FormData GetFormDataAndExpectation(const FormParsingTestCase& test_case,
       field.typed_value = ASCIIToUTF16(field_description.typed_value);
     form_data.fields.push_back(field);
     if (field_description.role == ElementRole::NONE) {
-      UpdateResultWithIdByRole(fill_result, unique_id,
+      UpdateResultWithIdByRole(fill_result, renderer_id,
                                field_description.role_filling);
-      UpdateResultWithIdByRole(save_result, unique_id,
+      UpdateResultWithIdByRole(save_result, renderer_id,
                                field_description.role_saving);
     } else {
-      UpdateResultWithIdByRole(fill_result, unique_id, field_description.role);
-      UpdateResultWithIdByRole(save_result, unique_id, field_description.role);
+      UpdateResultWithIdByRole(fill_result, renderer_id,
+                               field_description.role);
+      UpdateResultWithIdByRole(save_result, renderer_id,
+                               field_description.role);
     }
     if (field_description.prediction.type != autofill::MAX_VALID_FIELD_TYPE) {
-      (*predictions)[unique_id] = field_description.prediction;
+      predictions->push_back(field_description.prediction);
+      predictions->back().renderer_id = renderer_id;
+#if defined(OS_IOS)
+      predictions->back().unique_id = field.unique_id;
+#endif
     }
     if (field_description.predicted_username >= 0) {
       size_t index = static_cast<size_t>(field_description.predicted_username);
@@ -348,10 +357,10 @@ void CheckTestData(const std::vector<FormParsingTestCase>& test_cases) {
         EXPECT_FALSE(parsed_form) << "Expected no parsed results";
       } else {
         ASSERT_TRUE(parsed_form) << "Expected successful parsing";
-        EXPECT_EQ(PasswordForm::SCHEME_HTML, parsed_form->scheme);
+        EXPECT_EQ(PasswordForm::Scheme::kHtml, parsed_form->scheme);
         EXPECT_FALSE(parsed_form->preferred);
         EXPECT_FALSE(parsed_form->blacklisted_by_user);
-        EXPECT_EQ(PasswordForm::TYPE_MANUAL, parsed_form->type);
+        EXPECT_EQ(PasswordForm::Type::kManual, parsed_form->type);
 #if defined(OS_IOS)
         EXPECT_FALSE(parsed_form->has_renderer_ids);
 #else
@@ -368,7 +377,7 @@ void CheckTestData(const std::vector<FormParsingTestCase>& test_cases) {
 
         CheckPasswordFormFields(*parsed_form, form_data, expected_ids);
         CheckAllValuesUnique(parsed_form->all_possible_passwords);
-        CheckAllValuesUnique(parsed_form->other_possible_usernames);
+        CheckAllValuesUnique(parsed_form->all_possible_usernames);
         if (test_case.number_of_all_possible_passwords >= 0) {
           EXPECT_EQ(
               static_cast<size_t>(test_case.number_of_all_possible_passwords),
@@ -381,11 +390,11 @@ void CheckTestData(const std::vector<FormParsingTestCase>& test_cases) {
         if (test_case.number_of_all_possible_usernames >= 0) {
           EXPECT_EQ(
               static_cast<size_t>(test_case.number_of_all_possible_usernames),
-              parsed_form->other_possible_usernames.size());
+              parsed_form->all_possible_usernames.size());
         }
         if (test_case.all_possible_usernames) {
           EXPECT_EQ(*test_case.all_possible_usernames,
-                    parsed_form->other_possible_usernames);
+                    parsed_form->all_possible_usernames);
         }
         if (mode == FormDataParser::Mode::kSaving) {
           EXPECT_EQ(test_case.fallback_only, parsed_form->only_for_fallback);
@@ -409,13 +418,15 @@ void CheckTestData(const std::vector<FormParsingTestCase>& test_cases) {
 TEST(FormParserTest, NotPasswordForm) {
   CheckTestData({
       {
-          "No fields", {},
+          "No fields",
+          {},
       },
       {
           .description_for_logging = "No password fields",
           .fields =
               {
-                  {.form_control_type = "text"}, {.form_control_type = "text"},
+                  {.form_control_type = "text"},
+                  {.form_control_type = "text"},
               },
           .number_of_all_possible_passwords = 0,
           .number_of_all_possible_usernames = 0,
@@ -1088,7 +1099,6 @@ TEST(FormParserTest, ServerHints) {
                   {.role_saving = ElementRole::CURRENT_PASSWORD,
                    .form_control_type = "password"},
                   {.role_filling = ElementRole::NEW_PASSWORD,
-
                    .prediction = {.type = autofill::ACCOUNT_CREATION_PASSWORD},
                    .form_control_type = "password"},
                   {.role_filling = ElementRole::CONFIRMATION_PASSWORD,
@@ -1111,7 +1121,6 @@ TEST(FormParserTest, ServerHints) {
                .form_control_type = "password"},
           },
       },
-
   });
 }
 
@@ -1484,6 +1493,123 @@ TEST(FormParserTest, CVC) {
   });
 }
 
+// The parser should avoid identifying NOT_PASSWORD fields as passwords.
+TEST(FormParserTest, NotPasswordField) {
+  CheckTestData({
+      {
+          "Server hints: NOT_PASSWORD.",
+          {
+              {.role = ElementRole::USERNAME, .form_control_type = "text"},
+              {.form_control_type = "password",
+               .prediction = {.type = autofill::NOT_PASSWORD}},
+              {.role = ElementRole::CURRENT_PASSWORD,
+               .form_control_type = "password"},
+          },
+          .fallback_only = false,
+      },
+      {
+          "Server hints: NOT_PASSWORD on only password.",
+          {
+              {.role = ElementRole::USERNAME, .form_control_type = "text"},
+              {.role = ElementRole::CURRENT_PASSWORD,
+               .prediction = {.type = autofill::NOT_PASSWORD},
+               .form_control_type = "password"},
+          },
+          .fallback_only = true,
+      },
+  });
+}
+
+// The parser should avoid identifying NOT_USERNAME fields as usernames.
+TEST(FormParserTest, NotUsernameField) {
+  CheckTestData({
+      {
+          "Server hints: NOT_USERNAME.",
+          {{.role = ElementRole::USERNAME, .form_control_type = "text"},
+           {.role = ElementRole::NONE,
+            .form_control_type = "text",
+            .prediction = {.type = autofill::NOT_USERNAME}},
+           {.role = ElementRole::CURRENT_PASSWORD,
+            .form_control_type = "password",
+            .prediction = {.type = autofill::PASSWORD}}},
+          .fallback_only = false,
+      },
+      {
+          "Server hints: NOT_USERNAME on only username.",
+          {{.role = ElementRole::NONE,
+            .form_control_type = "text",
+            .prediction = {.type = autofill::NOT_USERNAME}},
+           {.role = ElementRole::CURRENT_PASSWORD,
+            .form_control_type = "password"}},
+          .fallback_only = false,
+      },
+  });
+}
+
+// The parser should avoid identifying NOT_USERNAME fields as usernames despite
+// autocomplete attribute.
+TEST(FormParserTest, NotUsernameFieldDespiteAutocompelteAtrribute) {
+  CheckTestData({
+      {
+          "Server hints: NOT_USERNAME.",
+          {{.role = ElementRole::USERNAME, .form_control_type = "text"},
+           {.form_control_type = "text",
+            .autocomplete_attribute = "username",
+            .prediction = {.type = autofill::NOT_USERNAME}},
+           {.role = ElementRole::CURRENT_PASSWORD,
+            .form_control_type = "password",
+            .prediction = {.type = autofill::PASSWORD}}},
+          .fallback_only = false,
+      },
+      {
+          "Server hints: NOT_USERNAME on only username.",
+          {
+              {.role = ElementRole::NONE,
+               .form_control_type = "text",
+               .autocomplete_attribute = "username",
+               .prediction = {.type = autofill::NOT_USERNAME}},
+              {.role = ElementRole::CURRENT_PASSWORD,
+               .form_control_type = "password"},
+          },
+          .fallback_only = false,
+      },
+  });
+}
+
+// The parser should avoid identifying NOT_PASSWORD fields as passwords.
+TEST(FormParserTest, NotPasswordFieldDespiteAutocompleteAttribute) {
+  CheckTestData({
+      {
+          "Server hints: NOT_PASSWORD.",
+          {
+              {.role = ElementRole::USERNAME, .form_control_type = "text"},
+              {.form_control_type = "password",
+               .prediction = {.type = autofill::NOT_PASSWORD},
+               .autocomplete_attribute = "current-password"},
+              {.form_control_type = "password",
+               .prediction = {.type = autofill::NOT_PASSWORD},
+               .autocomplete_attribute = "new-password"},
+              {.form_control_type = "password",
+               .prediction = {.type = autofill::NOT_PASSWORD},
+               .autocomplete_attribute = "password"},
+              {.role = ElementRole::CURRENT_PASSWORD,
+               .form_control_type = "password"},
+          },
+          .fallback_only = false,
+      },
+      {
+          "Server hints: NOT_PASSWORD on only password.",
+          {
+              {.role = ElementRole::USERNAME, .form_control_type = "text"},
+              {.role = ElementRole::CURRENT_PASSWORD,
+               .prediction = {.type = autofill::NOT_PASSWORD},
+               .form_control_type = "password"},
+          },
+          .fallback_only = true,
+      },
+  });
+}
+
 // Check that "readonly status" is reported accordingly.
 TEST(FormParserTest, ReadonlyStatus) {
   CheckTestData({
@@ -1898,8 +2024,7 @@ TEST(FormParserTest, HistogramsForUsernameDetectionMethod) {
     // saving mode.
     SCOPED_TRACE(histogram_test_case.parsing_data.description_for_logging);
     tester.ExpectUniqueSample("PasswordManager.UsernameDetectionMethod",
-                              histogram_test_case.expected_method,
-                              2);
+                              histogram_test_case.expected_method, 2);
   }
 }
 
