@@ -62,7 +62,7 @@ class BadLoginDatabase : public LoginDatabase {
 };
 
 PasswordFormData CreateTestPasswordFormData() {
-  PasswordFormData data = {PasswordForm::SCHEME_HTML,
+  PasswordFormData data = {PasswordForm::Scheme::kHtml,
                            "http://bar.example.com",
                            "http://bar.example.com/origin",
                            "http://bar.example.com/action",
@@ -158,9 +158,9 @@ base::FilePath PasswordStoreDefaultTestDelegate::test_login_db_file_path()
 
 }  // anonymous namespace
 
-INSTANTIATE_TYPED_TEST_CASE_P(Default,
-                              PasswordStoreOriginTest,
-                              PasswordStoreDefaultTestDelegate);
+INSTANTIATE_TYPED_TEST_SUITE_P(Default,
+                               PasswordStoreOriginTest,
+                               PasswordStoreDefaultTestDelegate);
 
 TEST(PasswordStoreDefaultTest, NonASCIIData) {
   PasswordStoreDefaultTestDelegate delegate;
@@ -168,7 +168,7 @@ TEST(PasswordStoreDefaultTest, NonASCIIData) {
 
   // Some non-ASCII password form data.
   static const PasswordFormData form_data[] = {
-      {PasswordForm::SCHEME_HTML, "http://foo.example.com",
+      {PasswordForm::Scheme::kHtml, "http://foo.example.com",
        "http://foo.example.com/origin", "http://foo.example.com/action",
        L"มีสีสัน", L"お元気ですか?", L"盆栽", L"أحب كرة", L"£éä국수çà", true, 1},
   };
@@ -276,7 +276,7 @@ TEST(PasswordStoreDefaultTest, OperationsOnABadDatabaseSilentlyFail) {
   delegate.FinishAsyncProcessing();
   testing::Mock::VerifyAndClearExpectations(&mock_consumer);
   EXPECT_CALL(mock_consumer, OnGetPasswordStoreResultsConstRef(IsEmpty()));
-  bad_store->GetBlacklistLogins(&mock_consumer);
+  bad_store->GetAllLogins(&mock_consumer);
   delegate.FinishAsyncProcessing();
   testing::Mock::VerifyAndClearExpectations(&mock_consumer);
 
@@ -296,8 +296,6 @@ TEST(PasswordStoreDefaultTest, OperationsOnABadDatabaseSilentlyFail) {
   bad_store->RemoveLoginsCreatedBetween(base::Time(), base::Time::Max(),
                                         run_loop.QuitClosure());
   run_loop.Run();
-
-  bad_store->RemoveLoginsSyncedBetween(base::Time(), base::Time::Max());
   delegate.FinishAsyncProcessing();
 
   // Ensure no notifications and no explosions during shutdown either.
