@@ -6,51 +6,20 @@
 
 namespace password_manager {
 
+// NOTE: It is strongly recommended to use UpperCamelCase style for feature
+//       names, e.g. "MyGreatFeature".
 namespace features {
 
-// Enable affiliation based matching, so that credentials stored for an Android
-// application will also be considered matches for, and be filled into
-// corresponding Web applications.
-const base::Feature kAffiliationBasedMatching = {
-    "AffiliationBasedMatching", base::FEATURE_ENABLED_BY_DEFAULT};
+// Enables the editing of passwords in chrome://settings/passwords, i.e. the
+// Desktop passwords settings page.
+const base::Feature kEditPasswordsInDesktopSettings = {
+    "EditPasswordsInDesktopSettings", base::FEATURE_DISABLED_BY_DEFAULT};
 
 // Recovers lost passwords on Mac by deleting the ones that cannot be decrypted
 // with the present encryption key from the Keychain.
 const base::Feature kDeleteCorruptedPasswords = {
     "DeleteCorruptedPasswords", base::FEATURE_DISABLED_BY_DEFAULT};
 
-// Use HTML based username detector.
-const base::Feature kHtmlBasedUsernameDetector = {
-    "HtmlBaseUsernameDetector", base::FEATURE_ENABLED_BY_DEFAULT};
-
-// Controls whether password requirements can be overridden for domains
-// (as opposed to only relying on the autofill server).
-const base::Feature kPasswordGenerationRequirementsDomainOverrides = {
-    "PasswordGenerationRequirementsDomainOverrides",
-    base::FEATURE_ENABLED_BY_DEFAULT};
-
-// Disallow autofilling of the sync credential.
-const base::Feature kProtectSyncCredential = {
-    "protect-sync-credential", base::FEATURE_DISABLED_BY_DEFAULT};
-
-// Disallow autofilling of the sync credential only for transactional reauth
-// pages.
-const base::Feature kProtectSyncCredentialOnReauth = {
-    "ProtectSyncCredentialOnReauth", base::FEATURE_DISABLED_BY_DEFAULT};
-
-// Controls the ability to import passwords from Chrome's settings page.
-const base::Feature kPasswordImport = {"PasswordImport",
-                                       base::FEATURE_DISABLED_BY_DEFAULT};
-
-// Adds password-related features to the keyboard accessory on mobile devices.
-const base::Feature kPasswordsKeyboardAccessory = {
-    "PasswordsKeyboardAccessory", base::FEATURE_DISABLED_BY_DEFAULT};
-
-// Deletes entries from local database on Mac which cannot be decrypted when
-// merging data with Sync.
-const base::Feature kRecoverPasswordsForSyncUsers = {
-    "RecoverPasswordsForSyncUsers", base::FEATURE_ENABLED_BY_DEFAULT};
-
 // Enables the experiment for the password manager to only fill on account
 // selection, rather than autofilling on page load, with highlighting of fields.
 const base::Feature kFillOnAccountSelect = {"fill-on-account-select",
@@ -61,29 +30,69 @@ const base::Feature kFillOnAccountSelect = {"fill-on-account-select",
 const base::Feature kFillOnAccountSelectHttp = {
     "FillOnAccountSelectHttp", base::FEATURE_DISABLED_BY_DEFAULT};
 
+// Forces password generation to ask user to update the password instead of
+// overwriting silently.
+const base::Feature kGenerationNoOverwrites = {
+    "GenerationNoOverwrites", base::FEATURE_DISABLED_BY_DEFAULT};
+
 // Integration with Google's Password Manager for signed-in and sync users.
 const base::Feature kGooglePasswordManager = {
     "google-password-manager", base::FEATURE_DISABLED_BY_DEFAULT};
 
+// Analyses the credentials submitted by user for leak detection.
+const base::Feature kLeakDetection = {"PasswordLeakDetection",
+                                      base::FEATURE_DISABLED_BY_DEFAULT};
+
+// Controls whether to offer manual password generation in the accessory sheet
+// on Android.
+const base::Feature kManualPasswordGenerationAndroid{
+    "ManualPasswordGenerationAndroid", base::FEATURE_DISABLED_BY_DEFAULT};
+
+// Performs a one-off migration (with retries) from a native backend into
+// logindb. Passwords are served from the new location.
+const base::Feature kMigrateLinuxToLoginDB = {"migrate-linux-to-logindb",
+                                              base::FEATURE_ENABLED_BY_DEFAULT};
+
 // Enables new password form parsing mechanism for filling passwords, details in
 // https://goo.gl/QodPH1
 const base::Feature kNewPasswordFormParsing = {
-    "new-password-form-parsing", base::FEATURE_DISABLED_BY_DEFAULT};
+    "new-password-form-parsing", base::FEATURE_ENABLED_BY_DEFAULT};
 
 // Enables new password form parsing mechanism for saving passwords, details in
 // https://goo.gl/QodPH1
 const base::Feature kNewPasswordFormParsingForSaving = {
-    "new-password-form-parsing-for-saving", base::FEATURE_DISABLED_BY_DEFAULT};
+    "new-password-form-parsing-for-saving", base::FEATURE_ENABLED_BY_DEFAULT};
 
 // Enables new password form parsing mechanism for saving passwords and disables
 // the old parser, details in https://goo.gl/QodPH1
 const base::Feature kOnlyNewParser = {"only-new-password-form-parsing",
-                                      base::FEATURE_DISABLED_BY_DEFAULT};
+                                      base::FEATURE_ENABLED_BY_DEFAULT};
 
-// Performs a one-off migration (with retries) from a native backend into
-// logindb. Passwords are served from the new location.
-const base::Feature kMigrateLinuxToLoginDB = {
-    "migrate-linux-to-logindb", base::FEATURE_DISABLED_BY_DEFAULT};
+// Enables editing saved passwords for Android.
+const base::Feature kPasswordEditingAndroid = {
+    "PasswordEditingAndroid", base::FEATURE_DISABLED_BY_DEFAULT};
+
+// Controls the ability to import passwords from Chrome's settings page.
+const base::Feature kPasswordImport = {"PasswordImport",
+                                       base::FEATURE_DISABLED_BY_DEFAULT};
+
+// Controls whether the password manager onboarding experience is shown
+// on Android.
+const base::Feature kPasswordManagerOnboardingAndroid = {
+    "PasswordManagerOnboardingAndroid", base::FEATURE_DISABLED_BY_DEFAULT};
+
+// Deletes entries from local database on Mac which cannot be decrypted when
+// merging data with Sync.
+const base::Feature kRecoverPasswordsForSyncUsers = {
+    "RecoverPasswordsForSyncUsers", base::FEATURE_ENABLED_BY_DEFAULT};
+
+// Enables the touch to fill feature for Android.
+const base::Feature kTouchToFillAndroid = {"TouchToFillAndroid",
+                                           base::FEATURE_DISABLED_BY_DEFAULT};
+
+// Enables support of filling and saving on username first flow.
+const base::Feature kUsernameFirstFlow = {"UsernameFirstFlow",
+                                          base::FEATURE_DISABLED_BY_DEFAULT};
 
 // Field trial identifier for password generation requirements.
 const char* kGenerationRequirementsFieldTrial =
