@@ -8,11 +8,12 @@
 
 #include "base/android/jni_string.h"
 #include "base/android/scoped_java_ref.h"
+#include "base/bind.h"
+#include "components/payments/content/android/jni_headers/PaymentManifestDownloader_jni.h"
 #include "components/payments/content/developer_console_logger.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/storage_partition.h"
 #include "content/public/browser/web_contents.h"
-#include "jni/PaymentManifestDownloader_jni.h"
 #include "services/network/public/cpp/shared_url_loader_factory.h"
 #include "url/gurl.h"
 
@@ -28,11 +29,14 @@ class DownloadCallback {
   ~DownloadCallback() {}
 
   void OnPaymentMethodManifestDownload(const GURL& url_after_redirects,
-                                       const std::string& content) {
+                                       const std::string& content,
+                                       const std::string& error_message) {
     JNIEnv* env = base::android::AttachCurrentThread();
 
     if (content.empty()) {
-      Java_ManifestDownloadCallback_onManifestDownloadFailure(env, jcallback_);
+      Java_ManifestDownloadCallback_onManifestDownloadFailure(
+          env, jcallback_,
+          base::android::ConvertUTF8ToJavaString(env, error_message));
     } else {
       Java_ManifestDownloadCallback_onPaymentMethodManifestDownloadSuccess(
           env, jcallback_,
@@ -41,11 +45,14 @@ class DownloadCallback {
   }
 
   void OnWebAppManifestDownload(const GURL& url_after_redirects,
-                                const std::string& content) {
+                                const std::string& content,
+                                const std::string& error_message) {
     JNIEnv* env = base::android::AttachCurrentThread();
 
     if (content.empty()) {
-      Java_ManifestDownloadCallback_onManifestDownloadFailure(env, jcallback_);
+      Java_ManifestDownloadCallback_onManifestDownloadFailure(
+          env, jcallback_,
+          base::android::ConvertUTF8ToJavaString(env, error_message));
     } else {
       Java_ManifestDownloadCallback_onWebAppManifestDownloadSuccess(
           env, jcallback_,
