@@ -7,6 +7,7 @@
 #include <map>
 #include <memory>
 #include <string>
+#include <vector>
 
 #include "base/feature_list.h"
 #include "base/metrics/field_trial.h"
@@ -14,6 +15,7 @@
 #include "base/strings/string_util.h"
 #include "base/test/scoped_feature_list.h"
 #include "build/build_config.h"
+#include "components/previews/core/previews_constants.h"
 #include "components/previews/core/previews_features.h"
 #include "components/variations/variations_associated_data.h"
 
@@ -26,22 +28,6 @@ namespace {
 const char kClientSidePreviewsFieldTrial[] = "ClientSidePreviews";
 const char kEnabled[] = "Enabled";
 
-// Verifies that we can enable offline previews via comand line.
-TEST(PreviewsExperimentsTest, TestCommandLineOfflinePage) {
-  EXPECT_TRUE(params::IsOfflinePreviewsEnabled());
-
-  std::unique_ptr<base::FeatureList> feature_list =
-      std::make_unique<base::FeatureList>();
-
-  // The feature is explicitly enabled on the command-line.
-  feature_list->InitializeFromCommandLine("", "OfflinePreviews");
-  base::FeatureList::ClearInstanceForTesting();
-  base::FeatureList::SetInstance(std::move(feature_list));
-
-  EXPECT_FALSE(params::IsOfflinePreviewsEnabled());
-  base::FeatureList::ClearInstanceForTesting();
-}
-
 // Verifies that the default params are correct, and that custom params can be
 // set, for both the previews blacklist and offline previews.
 TEST(PreviewsExperimentsTest, TestParamsForBlackListAndOffline) {
@@ -101,52 +87,107 @@ TEST(PreviewsExperimentsTest, TestParamsForBlackListAndOffline) {
   variations::testing::ClearAllVariationParams();
 }
 
-#if defined(OS_ANDROID)
-
-TEST(PreviewsExperimentsTest, TestClientLoFiEnabledByDefaultOnAndroid) {
-  EXPECT_TRUE(params::IsClientLoFiEnabled());
-}
-
-#else  // !defined(OS_ANDROID)
-
-TEST(PreviewsExperimentsTest, TestClientLoFiDisabledByDefaultOnNonAndroid) {
-  EXPECT_FALSE(params::IsClientLoFiEnabled());
-}
-
-#endif  // defined(OS_ANDROID)
-
-TEST(PreviewsExperimentsTest, TestClientLoFiExplicitlyDisabled) {
+TEST(PreviewsExperimentsTest, TestDefaultShouldExcludeMediaSuffix) {
   base::test::ScopedFeatureList scoped_feature_list;
-  scoped_feature_list.InitAndDisableFeature(features::kClientLoFi);
-  EXPECT_FALSE(params::IsClientLoFiEnabled());
+  scoped_feature_list.InitAndEnableFeature(features::kExcludedMediaSuffixes);
+
+  EXPECT_FALSE(
+      params::ShouldExcludeMediaSuffix(GURL("http://chromium.org/path/")));
+
+  std::vector<std::string> default_suffixes = {
+      ".apk", ".avi",  ".gif", ".gifv", ".jpeg", ".jpg", ".mp3",
+      ".mp4", ".mpeg", ".pdf", ".png",  ".webm", ".webp"};
+  for (const std::string& suffix : default_suffixes) {
+    GURL url("http://chromium.org/path/" + suffix);
+    EXPECT_TRUE(params::ShouldExcludeMediaSuffix(url));
+  }
 }
 
-TEST(PreviewsExperimentsTest, TestClientLoFiExplicitlyEnabled) {
-  base::test::ScopedFeatureList scoped_feature_list;
-  scoped_feature_list.InitAndEnableFeature(features::kClientLoFi);
-  EXPECT_TRUE(params::IsClientLoFiEnabled());
-}
+TEST(PreviewsExperimentsTest, TestShouldExcludeMediaSuffix) {
+  struct TestCase {
+    std::string msg;
+    bool enable_feature;
+    std::string varaiation_value;
+    std::vector<std::string> urls;
+    bool want_return;
+  };
+  const TestCase kTestCases[]{
+      {
+          .msg = "Feature disabled, should always return false",
+          .enable_feature = false,
+          .varaiation_value = "",
+          .urls = {"http://chromium.org/video.mp4"},
+          .want_return = false,
+      },
+      {
+          .msg = "Default values are overridden by variations",
+          .enable_feature = true,
+          .varaiation_value = ".html",
+          .urls = {"http://chromium.org/video.mp4",
+                   "http://chromium.org/image.png",
+                   "http://chromium.org/image.jpg",
+                   "http://chromium.org/audio.mp3"},
+          .want_return = false,
+      },
+      {
+          .msg = "Variation value whitespace should be trimmed",
+          .enable_feature = true,
+          .varaiation_value = " .mp4 , \t .png\n",
+          .urls = {"http://chromium.org/video.mp4",
+                   "http://chromium.org/image.png"},
+          .want_return = true,
+      },
+      {
+          .msg = "Variation value empty values should be excluded",
+          .enable_feature = true,
+          .varaiation_value = ".mp4,,.png,",
+          .urls = {"http://chromium.org/video.mp4",
+                   "http://chromium.org/image.png"},
+          .want_return = true,
+      },
+      {
+          .msg = "URLs should be compared case insensitive",
+          .enable_feature = true,
+          .varaiation_value = ".MP4,.png,",
+          .urls = {"http://chromium.org/video.mP4",
+                   "http://chromium.org/image.PNG"},
+          .want_return = true,
+      },
+      {
+          .msg = "Query params and fragments don't matter",
+          .enable_feature = true,
+          .varaiation_value = ".mp4,.png,",
+          .urls = {"http://chromium.org/video.mp4?hello=world",
+                   "http://chromium.org/image.png#test"},
+          .want_return = true,
+      },
+      {
+          .msg = "Query params and fragments shouldn't be considered",
+          .enable_feature = true,
+          .varaiation_value = ".mp4,.png,",
+          .urls = {"http://chromium.org/?video=video.mp4",
+                   "http://chromium.org/#image.png"},
+          .want_return = false,
+      },
+  };
+  for (const TestCase& test_case : kTestCases) {
+    SCOPED_TRACE(test_case.msg);
 
-TEST(PreviewsExperimentsTest, TestEnableClientLoFiWithDefaultParams) {
-  base::test::ScopedFeatureList scoped_feature_list;
-  scoped_feature_list.InitAndEnableFeature(features::kClientLoFi);
+    base::test::ScopedFeatureList scoped_feature_list;
+    if (test_case.enable_feature) {
+      scoped_feature_list.InitAndEnableFeatureWithParameters(
+          features::kExcludedMediaSuffixes,
+          {{"excluded_path_suffixes", test_case.varaiation_value}});
+    } else {
+      scoped_feature_list.InitAndDisableFeature(
+          features::kExcludedMediaSuffixes);
+    }
 
-  EXPECT_TRUE(params::IsClientLoFiEnabled());
-  EXPECT_EQ(0, params::ClientLoFiVersion());
-  EXPECT_EQ(net::EFFECTIVE_CONNECTION_TYPE_2G,
-            params::GetECTThresholdForPreview(PreviewsType::LOFI));
-}
-
-TEST(PreviewsExperimentsTest, TestEnableClientLoFiWithCustomParams) {
-  base::test::ScopedFeatureList scoped_feature_list;
-  scoped_feature_list.InitAndEnableFeatureWithParameters(
-      features::kClientLoFi,
-      {{"version", "10"}, {"max_allowed_effective_connection_type", "3G"}});
-
-  EXPECT_TRUE(params::IsClientLoFiEnabled());
-  EXPECT_EQ(10, params::ClientLoFiVersion());
-  EXPECT_EQ(net::EFFECTIVE_CONNECTION_TYPE_3G,
-            params::GetECTThresholdForPreview(PreviewsType::LOFI));
+    for (const std::string& url : test_case.urls) {
+      EXPECT_EQ(test_case.want_return,
+                params::ShouldExcludeMediaSuffix(GURL(url)));
+    }
+  }
 }
 
 }  // namespace
