@@ -7,11 +7,11 @@
 #include <utility>
 
 #include "base/base64url.h"
+#include "base/bind.h"
 #include "base/macros.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/rand_util.h"
 #include "base/timer/timer.h"
-#include "components/data_use_measurement/core/data_use_user_data.h"
 #include "components/safe_browsing/db/safebrowsing.pb.h"
 #include "net/base/load_flags.h"
 #include "net/http/http_response_headers.h"
@@ -20,6 +20,10 @@
 #include "services/network/public/cpp/shared_url_loader_factory.h"
 #include "services/network/public/cpp/simple_url_loader.h"
 
+#if !defined(FULL_SAFE_BROWSING)
+#include "base/system/sys_info.h"
+#endif
+
 using base::Time;
 using base::TimeDelta;
 
@@ -77,6 +81,15 @@ static const int kV4TimerStartIntervalSecMax = 300;
 // Maximum time, in seconds, to wait for a response to an update request.
 static const int kV4TimerUpdateWaitSecMax = 15 * 60;  // 15 minutes
 
+// The default number of entries, per threat type, in the safe browsing
+// database on low end (low RAM) devices. This value is also used as the default
+// for GMS Safe Browsing on low end devices.
+static const int kLowEndDefaultDBEntryCount = 1 << 10;
+
+// Malware threat DB coverage drops off too much below 4096 entries, so we use
+// this values instead of the default above.
+static const int kLowEndMalwareDBEntryCount = 1 << 12;
+
 ChromeClientInfo::SafeBrowsingReportingPopulation GetReportingLevelProtoValue(
     ExtendedReportingLevel reporting_level) {
   switch (reporting_level) {
@@ -242,6 +255,13 @@ std::string V4UpdateProtocolManager::GetBase64SerializedUpdateRequestProto() {
     list_update_request->mutable_constraints()->add_supported_compressions(RAW);
     list_update_request->mutable_constraints()->add_supported_compressions(
         RICE);
+
+#if !defined(FULL_SAFE_BROWSING)
+    if (base::SysInfo::IsLowEndDevice()) {
+      list_update_request->mutable_constraints()->set_max_database_entries(
+          GetLowEndDBEntryCount(list_update_request->threat_type()));
+    }
+#endif
   }
 
   if (!extended_reporting_level_callback_.is_null()) {
@@ -260,6 +280,15 @@ std::string V4UpdateProtocolManager::GetBase64SerializedUpdateRequestProto() {
   return req_base64;
 }
 
+int V4UpdateProtocolManager::GetLowEndDBEntryCount(ThreatType threat_type) {
+  switch (threat_type) {
+    case ThreatType::MALWARE_THREAT:
+      return kLowEndMalwareDBEntryCount;
+    default:
+      return kLowEndDefaultDBEntryCount;
+  }
+}
+
 bool V4UpdateProtocolManager::ParseUpdateResponse(
     const std::string& data,
     ParsedServerResponse* parsed_server_response) {
