@@ -5,14 +5,20 @@
 #ifndef COMPONENTS_SAFE_BROWSING_PASSWORD_PROTECTION_PASSWORD_PROTECTION_REQUEST_H_
 #define COMPONENTS_SAFE_BROWSING_PASSWORD_PROTECTION_PASSWORD_PROTECTION_REQUEST_H_
 
+#include <vector>
+
 #include "base/macros.h"
 #include "base/memory/ref_counted.h"
 #include "base/task/cancelable_task_tracker.h"
+#include "base/time/time.h"
+#include "components/password_manager/core/browser/password_manager_metrics_util.h"
+#include "components/safe_browsing/common/safe_browsing.mojom.h"
 #include "components/safe_browsing/password_protection/metrics_util.h"
 #include "components/safe_browsing/password_protection/password_protection_service.h"
+#include "components/safe_browsing/proto/csd.pb.h"
 #include "content/public/browser/browser_thread.h"
-
-#include <vector>
+#include "content/public/browser/web_contents_observer.h"
+#include "third_party/skia/include/core/SkBitmap.h"
 
 class GURL;
 
@@ -24,6 +30,8 @@ namespace safe_browsing {
 
 class PasswordProtectionNavigationThrottle;
 
+using password_manager::metrics_util::PasswordType;
+
 // A request for checking if an unfamiliar login form or a password reuse event
 // is safe. PasswordProtectionRequest objects are owned by
 // PasswordProtectionService indicated by |password_protection_service_|.
@@ -37,20 +45,23 @@ class PasswordProtectionNavigationThrottle;
 // (3) |   IO   | Check whitelist and return the result back to UI thread.
 // (4) |   UI   | If whitelisted, check verdict cache; else quit request.
 // (5) |   UI   | If verdict cached, quit request; else prepare request proto.
-// (6) |   UI   | Start a timeout task, and send network request.
-// (7) |   UI   | On receiving response, handle response and finish.
+// (6) |   UI   | Collect features related to the DOM of the page.
+// (7) |   UI   | If appropriate, compute visual features of the page.
+// (7) |   UI   | Start a timeout task, and send network request.
+// (8) |   UI   | On receiving response, handle response and finish.
 //     |        | On request timeout, cancel request.
 //     |        | On deletion of |password_protection_service_|, cancel request.
-class PasswordProtectionRequest
-    : public base::RefCountedThreadSafe<
-          PasswordProtectionRequest,
-          content::BrowserThread::DeleteOnUIThread> {
+class PasswordProtectionRequest : public base::RefCountedThreadSafe<
+                                      PasswordProtectionRequest,
+                                      content::BrowserThread::DeleteOnUIThread>,
+                                  public content::WebContentsObserver {
  public:
   PasswordProtectionRequest(content::WebContents* web_contents,
                             const GURL& main_frame_url,
                             const GURL& password_form_action,
                             const GURL& password_form_frame_url,
-                            ReusedPasswordType reused_password_type,
+                            const std::string& username,
+                            PasswordType password_type,
                             const std::vector<std::string>& matching_origins,
                             LoginReputationClientRequest::TriggerType type,
                             bool password_field_exists,
@@ -84,9 +95,9 @@ class PasswordProtectionRequest
     return trigger_type_;
   }
 
-  ReusedPasswordType reused_password_type() const {
-    return reused_password_type_;
-  }
+  const std::string username() const { return username_; }
+
+  PasswordType password_type() const { return password_type_; }
 
   bool is_modal_warning_showing() const { return is_modal_warning_showing_; }
 
@@ -106,6 +117,9 @@ class PasswordProtectionRequest
   // Cancels navigation if there is modal warning showing, resumes it otherwise.
   void HandleDeferredNavigations();
 
+  // WebContentsObserver implementation
+  void WebContentsDestroyed() override;
+
  protected:
   friend class base::RefCountedThreadSafe<PasswordProtectionRequest>;
 
@@ -113,8 +127,9 @@ class PasswordProtectionRequest
   friend struct content::BrowserThread::DeleteOnThread<
       content::BrowserThread::UI>;
   friend class base::DeleteHelper<PasswordProtectionRequest>;
+  friend class PasswordProtectionServiceTest;
   friend class ChromePasswordProtectionServiceTest;
-  virtual ~PasswordProtectionRequest();
+  ~PasswordProtectionRequest() override;
 
   // Start checking the whitelist.
   void CheckWhitelist();
@@ -134,6 +149,16 @@ class PasswordProtectionRequest
   // Fill |request_proto_| with appropriate values.
   void FillRequestProto();
 
+  // Collects visual features from the current login page.
+  void CollectVisualFeatures();
+
+  // Processes the screenshot of the login page into visual features.
+  void OnScreenshotTaken(const SkBitmap& bitmap);
+
+  // Called when the visual feature extraction is complete.
+  void OnVisualFeatureCollectionDone(
+      std::unique_ptr<VisualFeatures> visual_features);
+
   // Initiates network request to Safe Browsing backend.
   void SendRequest();
 
@@ -144,6 +169,9 @@ class PasswordProtectionRequest
   void Finish(RequestOutcome outcome,
               std::unique_ptr<LoginReputationClientResponse> response);
 
+  // Called when the DOM feature extraction is complete.
+  void OnGetDomFeatures(const std::string& verdict);
+
   // WebContents of the password protection event.
   content::WebContents* web_contents_;
 
@@ -156,8 +184,13 @@ class PasswordProtectionRequest
   // Frame url of the detected password form.
   const GURL password_form_frame_url_;
 
+  // The username of the reused password hash. The username can be an email or
+  // a username for a non-GAIA or saved-password reuse. No validation has been
+  // done on it.
+  const std::string username_;
+
   // Type of the reused password.
-  const ReusedPasswordType reused_password_type_;
+  const PasswordType password_type_;
 
   // Domains from the Password Manager that match this password.
   // Should be non-empty if |reused_password_type_| == SAVED_PASSWORD.
@@ -200,7 +233,18 @@ class PasswordProtectionRequest
   // If a request is sent, this is the token returned by the WebUI.
   int web_ui_token_;
 
-  base::WeakPtrFactory<PasswordProtectionRequest> weakptr_factory_;
+  // When we start extracting visual features.
+  base::TimeTicks visual_feature_start_time_;
+
+  // The Mojo pipe used for extracting DOM features from the renderer.
+  safe_browsing::mojom::PhishingDetectorPtr phishing_detector_;
+
+  // When we start extracting DOM features. Used to compute the duration of DOM
+  // feature extraction, which is logged at
+  // PasswordProtection.DomFeatureExtractionDuration.
+  base::TimeTicks dom_feature_start_time_;
+
+  base::WeakPtrFactory<PasswordProtectionRequest> weakptr_factory_{this};
   DISALLOW_COPY_AND_ASSIGN(PasswordProtectionRequest);
 };
 
