@@ -2,11 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "base/message_loop/message_loop.h"
+#include "base/task/single_thread_task_executor.h"
 #include "components/services/leveldb/leveldb_app.h"
 #include "services/service_manager/public/cpp/service_executable/service_main.h"
 
 void ServiceMain(service_manager::mojom::ServiceRequest request) {
-  base::MessageLoop message_loop;
+  base::SingleThreadTaskExecutor main_thread_task_executor;
   leveldb::LevelDBApp(std::move(request)).RunUntilTermination();
 }
