@@ -5,6 +5,7 @@
 #include "components/ui_devtools/ui_devtools_unittest_utils.h"
 
 #include "base/strings/string_util.h"
+#include "components/ui_devtools/devtools_protocol_encoding.h"
 
 namespace ui_devtools {
 
@@ -29,10 +30,22 @@ int FakeFrontendChannel::CountProtocolNotificationMessage(
                     protocol_notification_messages_.end(), message);
 }
 
+namespace {
+std::string SerializeToJSON(std::unique_ptr<protocol::Serializable> message) {
+  std::vector<uint8_t> cbor = message->serializeToBinary();
+  std::string json;
+  ::inspector_protocol_encoding::Status status =
+      ConvertCBORToJSON(::inspector_protocol_encoding::SpanFrom(cbor), &json);
+  DCHECK(status.ok()) << status.ToASCIIString();
+  return json;
+}
+}  // namespace
+
 void FakeFrontendChannel::sendProtocolNotification(
     std::unique_ptr<protocol::Serializable> message) {
   EXPECT_TRUE(allow_notifications_);
-  protocol_notification_messages_.push_back(message->serialize());
+  protocol_notification_messages_.push_back(
+      SerializeToJSON(std::move(message)));
 }
 
 }  // namespace ui_devtools
