@@ -12,15 +12,25 @@
 
 #include "base/i18n/rtl.h"
 #include "build/build_config.h"
+#include "components/viz/common/surfaces/surface_id.h"
 #include "content/browser/webui/web_ui_impl.h"
 #include "content/common/content_export.h"
 #include "content/common/frame_message_enums.h"
+#include "content/public/browser/media_stream_request.h"
 #include "content/public/browser/site_instance.h"
+#include "content/public/browser/visibility.h"
 #include "content/public/common/javascript_dialog_type.h"
-#include "content/public/common/media_stream_request.h"
+#include "content/public/common/resource_load_info.mojom.h"
+#include "content/public/common/resource_type.h"
 #include "mojo/public/cpp/bindings/scoped_interface_endpoint_handle.h"
+#include "net/cert/cert_status_flags.h"
 #include "net/http/http_response_headers.h"
-#include "services/device/public/interfaces/wake_lock.mojom.h"
+#include "services/device/public/mojom/geolocation_context.mojom.h"
+#include "services/device/public/mojom/wake_lock.mojom.h"
+#include "services/metrics/public/cpp/ukm_source_id.h"
+#include "third_party/blink/public/common/frame/blocked_navigation_types.h"
+#include "third_party/blink/public/common/mediastream/media_stream_request.h"
+#include "third_party/blink/public/mojom/devtools/console_message.mojom.h"
 #include "ui/base/window_open_disposition.h"
 
 #if defined(OS_WIN)
@@ -29,7 +39,7 @@
 
 #if defined(OS_ANDROID)
 #include "base/android/scoped_java_ref.h"
-#include "services/device/public/interfaces/nfc.mojom.h"
+#include "services/device/public/mojom/nfc.mojom.h"
 #endif
 
 class GURL;
@@ -38,19 +48,24 @@ namespace IPC {
 class Message;
 }
 
-namespace device {
-class GeolocationContext;
-}
-
 namespace gfx {
 class Rect;
+class Size;
 }
 
 namespace url {
 class Origin;
 }
 
+namespace blink {
+struct WebFullscreenOptions;
+namespace mojom {
+class FileChooserParams;
+}
+}
+
 namespace content {
+class FileSelectListener;
 class FrameTreeNode;
 class InterstitialPage;
 class PageState;
@@ -61,7 +76,7 @@ class WebContents;
 struct AXEventNotificationDetails;
 struct AXLocationChangeNotificationDetails;
 struct ContextMenuParams;
-struct FileChooserParams;
+struct GlobalRequestID;
 
 namespace mojom {
 class CreateNewWindowParams;
@@ -81,20 +96,42 @@ class CONTENT_EXPORT RenderFrameHostDelegate {
       const std::string& interface_name,
       mojo::ScopedInterfaceEndpointHandle handle) {}
 
+  // Allows the delegate to filter incoming interface requests.
+  virtual void OnInterfaceRequest(
+      RenderFrameHost* render_frame_host,
+      const std::string& interface_name,
+      mojo::ScopedMessagePipeHandle* interface_pipe) {}
+
+  // Notification from the renderer host that a suspicious navigation of the
+  // main frame has been blocked. Allows the delegate to provide some UI to let
+  // the user know about the blocked navigation and give them the option to
+  // recover from it.
+  // |blocked_url| is the blocked navigation target, |initiator_url| is the URL
+  // of the frame initiating the navigation, |reason| specifies why the
+  // navigation was blocked.
+  virtual void OnDidBlockNavigation(const GURL& blocked_url,
+                                    const GURL& initiator_url,
+                                    blink::NavigationBlockedReason reason) {}
+
   // Gets the last committed URL. See WebContents::GetLastCommittedURL for a
   // description of the semantics.
-  virtual const GURL& GetMainFrameLastCommittedURL() const;
+  virtual const GURL& GetMainFrameLastCommittedURL();
 
   // A message was added to to the console.
-  virtual bool DidAddMessageToConsole(int32_t level,
-                                      const base::string16& message,
-                                      int32_t line_no,
-                                      const base::string16& source_id);
+  virtual bool DidAddMessageToConsole(
+      blink::mojom::ConsoleMessageLevel log_level,
+      const base::string16& message,
+      int32_t line_no,
+      const base::string16& source_id);
 
-  // Informs the delegate whenever a RenderFrameHost is created.
+  // Called when a RenderFrame for |render_frame_host| is created in the
+  // renderer process. Use |RenderFrameDeleted| to listen for when this
+  // RenderFrame goes away.
   virtual void RenderFrameCreated(RenderFrameHost* render_frame_host) {}
 
-  // Informs the delegate whenever a RenderFrameHost is deleted.
+  // Called when a RenderFrame for |render_frame_host| is deleted or the
+  // renderer process in which it runs it has died. Use |RenderFrameCreated| to
+  // listen for when RenderFrame objects are created.
   virtual void RenderFrameDeleted(RenderFrameHost* render_frame_host) {}
 
   // A context menu should be shown, to be built using the context information
@@ -106,7 +143,6 @@ class CONTENT_EXPORT RenderFrameHostDelegate {
   virtual void RunJavaScriptDialog(RenderFrameHost* render_frame_host,
                                    const base::string16& message,
                                    const base::string16& default_prompt,
-                                   const GURL& frame_url,
                                    JavaScriptDialogType type,
                                    IPC::Message* reply_msg) {}
 
@@ -115,8 +151,24 @@ class CONTENT_EXPORT RenderFrameHostDelegate {
                                       IPC::Message* reply_msg) {}
 
   // Called when a file selection is to be done.
-  virtual void RunFileChooser(RenderFrameHost* render_frame_host,
-                              const FileChooserParams& params) {}
+  // Overrides of this function must call either listener->FileSelected() or
+  // listener->FileSelectionCanceled().
+  virtual void RunFileChooser(
+      RenderFrameHost* render_frame_host,
+      std::unique_ptr<content::FileSelectListener> listener,
+      const blink::mojom::FileChooserParams& params);
+
+  // Request to enumerate a directory.  This is equivalent to running the file
+  // chooser in directory-enumeration mode and having the user select the given
+  // directory.
+  // Overrides of this function must call either listener->FileSelected() or
+  // listener->FileSelectionCanceled().
+  virtual void EnumerateDirectory(RenderFrameHost* render_frame_host,
+                                  std::unique_ptr<FileSelectListener> listener,
+                                  const base::FilePath& directory_path);
+
+  // The pending page load was canceled, so the address bar should be updated.
+  virtual void DidCancelLoading() {}
 
   // Another page accessed the top-level initial empty document, which means it
   // is no longer safe to display a pending URL without risking a URL spoof.
@@ -126,6 +178,20 @@ class CONTENT_EXPORT RenderFrameHostDelegate {
   virtual void DidChangeName(RenderFrameHost* render_frame_host,
                              const std::string& name) {}
 
+  // The sticky user activation bit has been set on the frame. This will not be
+  // called for new RenderFrameHosts whose underlying FrameTreeNode was already
+  // activated.
+  virtual void DidReceiveFirstUserActivation(
+      RenderFrameHost* render_frame_host) {}
+
+  // The display style of the frame has changed.
+  virtual void DidChangeDisplayState(RenderFrameHost* render_frame_host,
+                                     bool is_display_none) {}
+
+  // The size of the frame has changed.
+  virtual void FrameSizeChanged(RenderFrameHost* render_frame_host,
+                                const gfx::Size& frame_size) {}
+
   // The onload handler in the frame has completed. Only called for the top-
   // level frame.
   virtual void DocumentOnLoadCompleted(RenderFrameHost* render_frame_host) {}
@@ -156,30 +222,30 @@ class CONTENT_EXPORT RenderFrameHostDelegate {
   // The render frame has requested access to media devices listed in
   // |request|, and the client should grant or deny that permission by
   // calling |callback|.
-  virtual void RequestMediaAccessPermission(
-      const MediaStreamRequest& request,
-      const MediaResponseCallback& callback);
+  virtual void RequestMediaAccessPermission(const MediaStreamRequest& request,
+                                            MediaResponseCallback callback);
 
   // Checks if we have permission to access the microphone or camera. Note that
   // this does not query the user. |type| must be MEDIA_DEVICE_AUDIO_CAPTURE
   // or MEDIA_DEVICE_VIDEO_CAPTURE.
-  // TODO(guidou): use url::Origin for |security_origin|. See crbug.com/683115.
-  virtual bool CheckMediaAccessPermission(const GURL& security_origin,
-                                          MediaStreamType type);
+  virtual bool CheckMediaAccessPermission(RenderFrameHost* render_frame_host,
+                                          const url::Origin& security_origin,
+                                          blink::mojom::MediaStreamType type);
 
   // Returns the ID of the default device for the given media device |type|.
   // If the returned value is an empty string, it means that there is no
   // default device for the given |type|.
-  virtual std::string GetDefaultMediaDeviceID(MediaStreamType type);
+  virtual std::string GetDefaultMediaDeviceID(
+      blink::mojom::MediaStreamType type);
 
   // Get the accessibility mode for the WebContents that owns this frame.
-  virtual AccessibilityMode GetAccessibilityMode() const;
+  virtual ui::AXMode GetAccessibilityMode();
 
   // Called when accessibility events or location changes are received
   // from a render frame, when the accessibility mode has the
-  // AccessibilityMode::kWebContents flag set.
+  // ui::AXMode::kWebContents flag set.
   virtual void AccessibilityEventReceived(
-      const std::vector<AXEventNotificationDetails>& details) {}
+      const AXEventNotificationDetails& details) {}
   virtual void AccessibilityLocationChangesReceived(
       const std::vector<AXLocationChangeNotificationDetails>& details) {}
 
@@ -190,10 +256,7 @@ class CONTENT_EXPORT RenderFrameHostDelegate {
       int browser_plugin_instance_id);
 
   // Gets the GeolocationContext associated with this delegate.
-  virtual device::GeolocationContext* GetGeolocationContext();
-
-  // Gets the WakeLock that serves wake lock requests from the renderer.
-  virtual device::mojom::WakeLock* GetRendererWakeLock();
+  virtual device::mojom::GeolocationContext* GetGeolocationContext();
 
 #if defined(OS_ANDROID)
   // Gets an NFC implementation within the context of this delegate.
@@ -202,7 +265,9 @@ class CONTENT_EXPORT RenderFrameHostDelegate {
 
   // Notification that the frame wants to go into fullscreen mode.
   // |origin| represents the origin of the frame that requests fullscreen.
-  virtual void EnterFullscreenMode(const GURL& origin) {}
+  virtual void EnterFullscreenMode(const GURL& origin,
+                                   const blink::WebFullscreenOptions& options) {
+  }
 
   // Notification that the frame wants to go out of fullscreen mode.
   // |will_cause_resize| indicates whether the fullscreen change causes a
@@ -210,6 +275,18 @@ class CONTENT_EXPORT RenderFrameHostDelegate {
   // browser fullscreen.
   virtual void ExitFullscreenMode(bool will_cause_resize) {}
 
+  // Notification that this frame has changed fullscreen state.
+  virtual void FullscreenStateChanged(RenderFrameHost* rfh,
+                                      bool is_fullscreen) {}
+
+#if defined(OS_ANDROID)
+  // Updates information to determine whether a user gesture should carryover to
+  // future navigations. This is needed so navigations within a certain
+  // timeframe of a request initiated by a gesture will be treated as if they
+  // were initiated by a gesture too, otherwise the navigation may be blocked.
+  virtual void UpdateUserGestureCarryoverInfo() {}
+#endif
+
   // Let the delegate decide whether postMessage should be delivered to
   // |target_rfh| from a source frame in the given SiteInstance.  This defaults
   // to false and overrides the RenderFrameHost's decision if true.
@@ -234,6 +311,9 @@ class CONTENT_EXPORT RenderFrameHostDelegate {
   // possibly changing focus in distinct but related inner/outer WebContents.
   virtual void SetFocusedFrame(FrameTreeNode* node, SiteInstance* source) {}
 
+  // The frame called |window.focus()|.
+  virtual void DidCallFocus() {}
+
   // Searches the WebContents for a focused frame, potentially in an inner
   // WebContents. If this WebContents has no focused frame, returns |nullptr|.
   // If there is no inner WebContents at the focused tree node, returns its
@@ -285,6 +365,7 @@ class CONTENT_EXPORT RenderFrameHostDelegate {
       int32_t main_frame_route_id,
       int32_t main_frame_widget_route_id,
       const mojom::CreateNewWindowParams& params,
+      bool has_user_gesture,
       SessionStorageNamespace* session_storage_namespace) {}
 
   // Show a previously created page with the specified disposition and bounds.
@@ -314,6 +395,9 @@ class CONTENT_EXPORT RenderFrameHostDelegate {
                                                  const url::Origin& origin,
                                                  const GURL& resource_url);
 
+  // Opens a new view-source tab for the last committed document in |frame|.
+  virtual void ViewSource(RenderFrameHostImpl* frame) {}
+
 #if defined(OS_ANDROID)
   virtual base::android::ScopedJavaLocalRef<jobject>
   GetJavaRenderFrameHostDelegate();
@@ -321,7 +405,49 @@ class CONTENT_EXPORT RenderFrameHostDelegate {
 
   // Whether the delegate is being destroyed, in which case the RenderFrameHost
   // should not be asked to create a RenderFrame.
-  virtual bool IsBeingDestroyed() const;
+  virtual bool IsBeingDestroyed();
+
+  // Notifies that the render frame started loading a subresource.
+  virtual void SubresourceResponseStarted(const GURL& url,
+                                          net::CertStatus cert_status) {}
+
+  // Notifies that the render finished loading a subresource for the frame
+  // associated with |render_frame_host|.
+  virtual void ResourceLoadComplete(
+      RenderFrameHost* render_frame_host,
+      const GlobalRequestID& request_id,
+      mojom::ResourceLoadInfoPtr resource_load_info) {}
+
+  // Request to print a frame that is in a different process than its parent.
+  virtual void PrintCrossProcessSubframe(const gfx::Rect& rect,
+                                         int document_cookie,
+                                         RenderFrameHost* render_frame_host) {}
+
+  // Updates the Picture-in-Picture controller with the relevant viz::SurfaceId
+  // of the video to be in Picture-in-Picture mode.
+  virtual void UpdatePictureInPictureSurfaceId(const viz::SurfaceId& surface_id,
+                                               const gfx::Size& natural_size) {}
+
+  // Returns the visibility of the delegate.
+  virtual Visibility GetVisibility();
+
+  // Get the UKM source ID for current content. This is used for providing
+  // data about the content to the URL-keyed metrics service.
+  // Note: This is also exposed by the RenderWidgetHostDelegate.
+  virtual ukm::SourceId GetUkmSourceIdForLastCommittedSource() const;
+
+  // Notify observers if WebAudio AudioContext has started (or stopped) playing
+  // audible sounds.
+  virtual void AudioContextPlaybackStarted(RenderFrameHost* host,
+                                           int context_id) {}
+  virtual void AudioContextPlaybackStopped(RenderFrameHost* host,
+                                           int context_id) {}
+
+  // Returns the main frame of the inner delegate that is attached to this
+  // delegate using |frame_tree_node|. Returns nullptr if no such inner delegate
+  // exists.
+  virtual RenderFrameHostImpl* GetMainFrameForInnerDelegate(
+      FrameTreeNode* frame_tree_node);
 
  protected:
   virtual ~RenderFrameHostDelegate() {}
