@@ -4,37 +4,55 @@
 
 #include "content/browser/frame_host/render_frame_message_filter.h"
 
+#include <memory>
+#include <string>
+#include <utility>
+#include <vector>
+
+#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/debug/alias.h"
+#include "base/feature_list.h"
 #include "base/macros.h"
 #include "base/strings/string_util.h"
+#include "base/syslog_logging.h"
+#include "base/task/post_task.h"
+#include "base/unguessable_token.h"
 #include "build/build_config.h"
+#include "components/download/public/common/download_url_parameters.h"
 #include "content/browser/bad_message.h"
 #include "content/browser/blob_storage/chrome_blob_storage_context.h"
 #include "content/browser/child_process_security_policy_impl.h"
-#include "content/browser/download/download_stats.h"
+#include "content/browser/frame_host/ipc_utils.h"
 #include "content/browser/frame_host/render_frame_host_impl.h"
 #include "content/browser/gpu/gpu_data_manager_impl.h"
 #include "content/browser/renderer_host/render_widget_helper.h"
 #include "content/browser/resource_context_impl.h"
+#include "content/browser/storage_partition_impl.h"
 #include "content/common/content_constants_internal.h"
 #include "content/common/frame_messages.h"
 #include "content/common/frame_owner_properties.h"
 #include "content/common/view_messages.h"
 #include "content/public/browser/browser_context.h"
+#include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/download_manager.h"
-#include "content/public/browser/download_url_parameters.h"
+#include "content/public/browser/storage_partition.h"
 #include "content/public/common/content_constants.h"
+#include "content/public/common/content_features.h"
 #include "gpu/GLES2/gl2extchromium.h"
-#include "net/base/registry_controlled_domains/registry_controlled_domain.h"
-#include "net/cookies/cookie_options.h"
-#include "net/cookies/cookie_store.h"
+#include "mojo/public/cpp/bindings/callback_helpers.h"
+#include "mojo/public/cpp/system/message_pipe.h"
 #include "net/traffic_annotation/network_traffic_annotation.h"
 #include "net/url_request/url_request_context.h"
 #include "net/url_request/url_request_context_getter.h"
-#include "ppapi/features/features.h"
+#include "ppapi/buildflags/buildflags.h"
+#include "services/network/public/cpp/features.h"
+#include "services/service_manager/public/mojom/interface_provider.mojom.h"
 #include "storage/browser/blob/blob_storage_context.h"
+#include "third_party/blink/public/common/frame/frame_owner_element_type.h"
+#include "third_party/blink/public/common/frame/frame_policy.h"
+#include "third_party/blink/public/mojom/web_feature/web_feature.mojom.h"
 #include "url/gurl.h"
 #include "url/origin.h"
 
@@ -53,19 +71,21 @@ namespace content {
 
 namespace {
 
-#if BUILDFLAG(ENABLE_PLUGINS)
-const int kPluginsRefreshThresholdInSeconds = 3;
-#endif
-
-void CreateChildFrameOnUI(int process_id,
-                          int parent_routing_id,
-                          blink::WebTreeScopeType scope,
-                          const std::string& frame_name,
-                          const std::string& frame_unique_name,
-                          blink::WebSandboxFlags sandbox_flags,
-                          const ParsedFeaturePolicyHeader& container_policy,
-                          const FrameOwnerProperties& frame_owner_properties,
-                          int new_routing_id) {
+void CreateChildFrameOnUI(
+    int process_id,
+    int parent_routing_id,
+    blink::WebTreeScopeType scope,
+    const std::string& frame_name,
+    const std::string& frame_unique_name,
+    bool is_created_by_script,
+    const base::UnguessableToken& devtools_frame_token,
+    const blink::FramePolicy& frame_policy,
+    const FrameOwnerProperties& frame_owner_properties,
+    blink::FrameOwnerElementType owner_type,
+    int new_routing_id,
+    mojo::ScopedMessagePipeHandle interface_provider_request_handle,
+    mojo::ScopedMessagePipeHandle document_interface_broker_content_handle,
+    mojo::ScopedMessagePipeHandle document_interface_broker_blink_handle) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   RenderFrameHostImpl* render_frame_host =
       RenderFrameHostImpl::FromID(process_id, parent_routing_id);
@@ -73,12 +93,25 @@ void CreateChildFrameOnUI(int process_id,
   // processing a subframe creation message.
   if (render_frame_host) {
     render_frame_host->OnCreateChildFrame(
-        new_routing_id, scope, frame_name, frame_unique_name, sandbox_flags,
-        container_policy, frame_owner_properties);
+        new_routing_id,
+        service_manager::mojom::InterfaceProviderRequest(
+            std::move(interface_provider_request_handle)),
+        blink::mojom::DocumentInterfaceBrokerRequest(
+            std::move(document_interface_broker_content_handle)),
+        blink::mojom::DocumentInterfaceBrokerRequest(
+            std::move(document_interface_broker_blink_handle)),
+        scope, frame_name, frame_unique_name, is_created_by_script,
+        devtools_frame_token, frame_policy, frame_owner_properties, owner_type);
   }
 }
 
-void DownloadUrlOnUIThread(std::unique_ptr<DownloadUrlParameters> parameters) {
+// |blob_data_handle| is only here for the legacy code path. With network
+// service enabled |blob_url_token| should be provided and will be used instead
+// to download the correct blob.
+void DownloadUrlOnUIThread(
+    std::unique_ptr<download::DownloadUrlParameters> parameters,
+    std::unique_ptr<storage::BlobDataHandle> blob_data_handle,
+    blink::mojom::BlobURLTokenPtrInfo blob_url_token) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
   RenderProcessHost* render_process_host =
@@ -87,10 +120,44 @@ void DownloadUrlOnUIThread(std::unique_ptr<DownloadUrlParameters> parameters) {
     return;
 
   BrowserContext* browser_context = render_process_host->GetBrowserContext();
+
+  scoped_refptr<network::SharedURLLoaderFactory> blob_url_loader_factory;
+  if (blob_url_token) {
+    blob_url_loader_factory =
+        ChromeBlobStorageContext::URLLoaderFactoryForToken(
+            browser_context,
+            blink::mojom::BlobURLTokenPtr(std::move(blob_url_token)));
+  }
+
   DownloadManager* download_manager =
       BrowserContext::GetDownloadManager(browser_context);
-  RecordDownloadSource(INITIATED_BY_RENDERER);
-  download_manager->DownloadUrl(std::move(parameters));
+  parameters->set_download_source(download::DownloadSource::FROM_RENDERER);
+  download_manager->DownloadUrl(std::move(parameters),
+                                std::move(blob_data_handle),
+                                std::move(blob_url_loader_factory));
+}
+
+// With network service disabled the downloads code wouldn't know what to do
+// with a BlobURLToken, so this method is used to convert from a token to a
+// BlobDataHandle to be passed on to the rest of the downloads system.
+void DownloadBlobURLFromToken(
+    std::unique_ptr<download::DownloadUrlParameters> params,
+    blink::mojom::BlobURLTokenPtr,
+    const base::WeakPtr<storage::BlobStorageContext>& context,
+    const base::UnguessableToken& token) {
+  std::unique_ptr<storage::BlobDataHandle> blob_handle;
+  GURL blob_url;
+  if (context) {
+    std::string uuid;
+    if (context->registry().GetTokenMapping(token, &blob_url, &uuid) &&
+        blob_url == params->url()) {
+      blob_handle = context->GetBlobDataFromUUID(uuid);
+    }
+  }
+  base::PostTaskWithTraits(
+      FROM_HERE, {BrowserThread::UI},
+      base::BindOnce(&DownloadUrlOnUIThread, std::move(params),
+                     std::move(blob_handle), nullptr));
 }
 
 // Common functionality for converting a sync renderer message to a callback
@@ -173,11 +240,8 @@ class RenderFrameMessageFilter::OpenChannelToPpapiPluginCallback
       public PpapiPluginProcessHost::PluginClient {
  public:
   OpenChannelToPpapiPluginCallback(RenderFrameMessageFilter* filter,
-                                   ResourceContext* context,
                                    IPC::Message* reply_msg)
-      : RenderMessageCompletionCallback(filter, reply_msg),
-        context_(context) {
-  }
+      : RenderMessageCompletionCallback(filter, reply_msg) {}
 
   void GetPpapiChannelInfo(base::ProcessHandle* renderer_handle,
                            int* renderer_id) override {
@@ -197,11 +261,6 @@ class RenderFrameMessageFilter::OpenChannelToPpapiPluginCallback
   }
 
   bool Incognito() override { return filter()->incognito_; }
-
-  ResourceContext* GetResourceContext() override { return context_; }
-
- private:
-  ResourceContext* context_;
 };
 
 #endif  // ENABLE_PLUGINS
@@ -210,15 +269,17 @@ RenderFrameMessageFilter::RenderFrameMessageFilter(
     int render_process_id,
     PluginServiceImpl* plugin_service,
     BrowserContext* browser_context,
-    net::URLRequestContextGetter* request_context,
+    StoragePartition* storage_partition,
     RenderWidgetHelper* render_widget_helper)
     : BrowserMessageFilter(FrameMsgStart),
-      BrowserAssociatedInterface<mojom::RenderFrameMessageFilter>(this, this),
 #if BUILDFLAG(ENABLE_PLUGINS)
       plugin_service_(plugin_service),
-      profile_data_directory_(browser_context->GetPath()),
+      profile_data_directory_(storage_partition->GetPath()),
 #endif  // ENABLE_PLUGINS
-      request_context_(request_context),
+      request_context_(
+          base::FeatureList::IsEnabled(network::features::kNetworkService)
+              ? nullptr
+              : storage_partition->GetURLRequestContext()),
       resource_context_(browser_context->GetResourceContext()),
       render_widget_helper_(render_widget_helper),
       incognito_(browser_context->IsOffTheRecord()),
@@ -230,19 +291,19 @@ RenderFrameMessageFilter::~RenderFrameMessageFilter() {
   DCHECK_CURRENTLY_ON(BrowserThread::IO);
 }
 
+void RenderFrameMessageFilter::ClearResourceContext() {
+  resource_context_ = nullptr;
+}
+
 bool RenderFrameMessageFilter::OnMessageReceived(const IPC::Message& message) {
   bool handled = true;
   IPC_BEGIN_MESSAGE_MAP(RenderFrameMessageFilter, message)
     IPC_MESSAGE_HANDLER(FrameHostMsg_CreateChildFrame, OnCreateChildFrame)
-    IPC_MESSAGE_HANDLER(FrameHostMsg_CookiesEnabled, OnCookiesEnabled)
     IPC_MESSAGE_HANDLER(FrameHostMsg_DownloadUrl, OnDownloadUrl)
     IPC_MESSAGE_HANDLER(FrameHostMsg_SaveImageFromDataURL,
                         OnSaveImageFromDataURL)
     IPC_MESSAGE_HANDLER(FrameHostMsg_Are3DAPIsBlocked, OnAre3DAPIsBlocked)
-    IPC_MESSAGE_HANDLER_GENERIC(FrameHostMsg_RenderProcessGone,
-                                OnRenderProcessGone())
 #if BUILDFLAG(ENABLE_PLUGINS)
-    IPC_MESSAGE_HANDLER_DELAY_REPLY(FrameHostMsg_GetPlugins, OnGetPlugins)
     IPC_MESSAGE_HANDLER(FrameHostMsg_GetPluginInfo, OnGetPluginInfo)
     IPC_MESSAGE_HANDLER_DELAY_REPLY(FrameHostMsg_OpenChannelToPepperPlugin,
                                     OnOpenChannelToPepperPlugin)
@@ -265,13 +326,16 @@ void RenderFrameMessageFilter::OnDestruct() const {
   BrowserThread::DeleteOnIOThread::Destruct(this);
 }
 
-void RenderFrameMessageFilter::DownloadUrl(int render_view_id,
-                                           int render_frame_id,
-                                           const GURL& url,
-                                           const Referrer& referrer,
-                                           const url::Origin& initiator,
-                                           const base::string16& suggested_name,
-                                           const bool use_prompt) const {
+void RenderFrameMessageFilter::DownloadUrl(
+    int render_view_id,
+    int render_frame_id,
+    const GURL& url,
+    const Referrer& referrer,
+    const url::Origin& initiator,
+    const base::string16& suggested_name,
+    const bool use_prompt,
+    const bool follow_cross_origin_redirects,
+    blink::mojom::BlobURLTokenPtrInfo blob_url_token) const {
   if (!resource_context_)
     return;
 
@@ -291,7 +355,7 @@ void RenderFrameMessageFilter::DownloadUrl(int render_view_id,
           destination: WEBSITE
         }
         policy {
-          cookies_allowed: true
+          cookies_allowed: YES
           cookies_store: "user"
           setting: "This feature cannot be disabled by settings."
           chrome_policy {
@@ -300,79 +364,104 @@ void RenderFrameMessageFilter::DownloadUrl(int render_view_id,
             }
           }
         })");
-  std::unique_ptr<DownloadUrlParameters> parameters(new DownloadUrlParameters(
-      url, render_process_id_, render_view_id, render_frame_id,
-      request_context_.get(), traffic_annotation));
+  std::unique_ptr<download::DownloadUrlParameters> parameters(
+      new download::DownloadUrlParameters(url, render_process_id_,
+                                          render_view_id, render_frame_id,
+                                          traffic_annotation));
   parameters->set_content_initiated(true);
   parameters->set_suggested_name(suggested_name);
   parameters->set_prompt(use_prompt);
-  parameters->set_referrer(referrer);
+  parameters->set_follow_cross_origin_redirects(follow_cross_origin_redirects);
+  parameters->set_referrer(referrer.url);
+  parameters->set_referrer_policy(
+      Referrer::ReferrerPolicyForUrlRequest(referrer.policy));
   parameters->set_initiator(initiator);
 
+  // If network service is enabled we should always have a |blob_url_token|,
+  // which will be used to download the correct blob. But in the legacy
+  // non-network service code path we still need to look up the BlobDataHandle
+  // for the URL here, to make sure the correct blob ends up getting downloaded.
+  std::unique_ptr<storage::BlobDataHandle> blob_data_handle;
   if (url.SchemeIsBlob()) {
     ChromeBlobStorageContext* blob_context =
         GetChromeBlobStorageContextForResourceContext(resource_context_);
-    parameters->set_blob_data_handle(
-        blob_context->context()->GetBlobDataFromPublicURL(url));
+
+    // With network service disabled the downloads code wouldn't know what to do
+    // with the BlobURLToken (or the resulting URLLoaderFactory). So for that
+    // case convert the token to a BlobDataHandle before passing it of to the
+    // rest of the downloads system.
+    if (blob_url_token &&
+        !base::FeatureList::IsEnabled(network::features::kNetworkService)) {
+      blink::mojom::BlobURLTokenPtr blob_url_token_ptr(
+          std::move(blob_url_token));
+      auto* raw_token = blob_url_token_ptr.get();
+      raw_token->GetToken(mojo::WrapCallbackWithDefaultInvokeIfNotRun(
+          base::BindOnce(&DownloadBlobURLFromToken, std::move(parameters),
+                         std::move(blob_url_token_ptr),
+                         blob_context->context()->AsWeakPtr()),
+          base::UnguessableToken()));
+      return;
+    }
+
+    blob_data_handle = blob_context->context()->GetBlobDataFromPublicURL(url);
     // Don't care if the above fails. We are going to let the download go
     // through and allow it to be interrupted so that the embedder can deal.
   }
 
-  BrowserThread::PostTask(
-      BrowserThread::UI, FROM_HERE,
-      base::Bind(&DownloadUrlOnUIThread, base::Passed(&parameters)));
+  base::PostTaskWithTraits(
+      FROM_HERE, {BrowserThread::UI},
+      base::BindOnce(&DownloadUrlOnUIThread, std::move(parameters),
+                     std::move(blob_data_handle), std::move(blob_url_token)));
 }
 
 void RenderFrameMessageFilter::OnCreateChildFrame(
     const FrameHostMsg_CreateChildFrame_Params& params,
-    int* new_routing_id) {
-  *new_routing_id = render_widget_helper_->GetNextRoutingID();
-  BrowserThread::PostTask(
-      BrowserThread::UI, FROM_HERE,
-      base::Bind(&CreateChildFrameOnUI, render_process_id_,
-                 params.parent_routing_id, params.scope, params.frame_name,
-                 params.frame_unique_name, params.sandbox_flags,
-                 params.container_policy, params.frame_owner_properties,
-                 *new_routing_id));
-}
+    FrameHostMsg_CreateChildFrame_Params_Reply* params_reply) {
+  params_reply->child_routing_id = render_widget_helper_->GetNextRoutingID();
 
-void RenderFrameMessageFilter::OnCookiesEnabled(
-    int render_frame_id,
-    const GURL& url,
-    const GURL& first_party_for_cookies,
-    bool* cookies_enabled) {
-  // TODO(ananta): If this render frame is associated with an automation
-  // channel, aka ChromeFrame then we need to retrieve cookie settings from the
-  // external host.
-  *cookies_enabled = GetContentClient()->browser()->AllowGetCookie(
-      url, first_party_for_cookies, net::CookieList(), resource_context_,
-      render_process_id_, render_frame_id);
-}
+  service_manager::mojom::InterfaceProviderPtr interface_provider;
+  auto interface_provider_request(mojo::MakeRequest(&interface_provider));
+  params_reply->new_interface_provider =
+      interface_provider.PassInterface().PassHandle().release();
 
-void RenderFrameMessageFilter::CheckPolicyForCookies(
-    int render_frame_id,
-    const GURL& url,
-    const GURL& first_party_for_cookies,
-    GetCookiesCallback callback,
-    const net::CookieList& cookie_list) {
-  net::URLRequestContext* context = GetRequestContextForURL(url);
-  // Check the policy for get cookies, and pass cookie_list to the
-  // TabSpecificContentSetting for logging purpose.
-  if (context &&
-      GetContentClient()->browser()->AllowGetCookie(
-          url, first_party_for_cookies, cookie_list, resource_context_,
-          render_process_id_, render_frame_id)) {
-    std::move(callback).Run(net::CookieStore::BuildCookieLine(cookie_list));
-  } else {
-    std::move(callback).Run(std::string());
-  }
+  blink::mojom::DocumentInterfaceBrokerPtrInfo
+      document_interface_broker_content;
+  auto document_interface_broker_request_content(
+      mojo::MakeRequest(&document_interface_broker_content));
+  params_reply->document_interface_broker_content_handle =
+      document_interface_broker_content.PassHandle().release();
+
+  blink::mojom::DocumentInterfaceBrokerPtrInfo document_interface_broker_blink;
+  auto document_interface_broker_request_blink(
+      mojo::MakeRequest(&document_interface_broker_blink));
+  params_reply->document_interface_broker_blink_handle =
+      document_interface_broker_blink.PassHandle().release();
+
+  params_reply->devtools_frame_token = base::UnguessableToken::Create();
+
+  base::PostTaskWithTraits(
+      FROM_HERE, {BrowserThread::UI},
+      base::BindOnce(
+          &CreateChildFrameOnUI, render_process_id_, params.parent_routing_id,
+          params.scope, params.frame_name, params.frame_unique_name,
+          params.is_created_by_script, params_reply->devtools_frame_token,
+          params.frame_policy, params.frame_owner_properties,
+          params.frame_owner_element_type, params_reply->child_routing_id,
+          interface_provider_request.PassMessagePipe(),
+          document_interface_broker_request_content.PassMessagePipe(),
+          document_interface_broker_request_blink.PassMessagePipe()));
 }
 
 void RenderFrameMessageFilter::OnDownloadUrl(
     const FrameHostMsg_DownloadUrl_Params& params) {
+  blink::mojom::BlobURLTokenPtrInfo blob_url_token;
+  if (!VerifyDownloadUrlParams(render_process_id_, params, &blob_url_token))
+    return;
+
   DownloadUrl(params.render_view_id, params.render_frame_id, params.url,
               params.referrer, params.initiator_origin, params.suggested_name,
-              false);
+              false, params.follow_cross_origin_redirects,
+              std::move(blob_url_token));
 }
 
 void RenderFrameMessageFilter::OnSaveImageFromDataURL(
@@ -388,7 +477,7 @@ void RenderFrameMessageFilter::OnSaveImageFromDataURL(
     return;
 
   DownloadUrl(render_view_id, render_frame_id, data_url, Referrer(),
-              url::Origin(), base::string16(), true);
+              url::Origin(), base::string16(), true, true, nullptr);
 }
 
 void RenderFrameMessageFilter::OnAre3DAPIsBlocked(int render_frame_id,
@@ -399,132 +488,8 @@ void RenderFrameMessageFilter::OnAre3DAPIsBlocked(int render_frame_id,
       top_origin_url, render_process_id_, render_frame_id, requester);
 }
 
-void RenderFrameMessageFilter::OnRenderProcessGone() {
-  // FrameHostMessage_RenderProcessGone is a synthetic IPC message used by
-  // RenderProcessHostImpl to clean things up after a crash (it's injected
-  // downstream of this filter). Allowing it to proceed would enable a renderer
-  // to fake its own death; instead, actually kill the renderer.
-  bad_message::ReceivedBadMessage(
-      this, bad_message::RFMF_RENDERER_FAKED_ITS_OWN_DEATH);
-}
-
-void RenderFrameMessageFilter::SetCookie(int32_t render_frame_id,
-                                         const GURL& url,
-                                         const GURL& first_party_for_cookies,
-                                         const std::string& cookie) {
-  ChildProcessSecurityPolicyImpl* policy =
-      ChildProcessSecurityPolicyImpl::GetInstance();
-  if (!policy->CanAccessDataForOrigin(render_process_id_, url)) {
-    bad_message::ReceivedBadMessage(this,
-                                    bad_message::RFMF_SET_COOKIE_BAD_ORIGIN);
-    return;
-  }
-
-  net::CookieOptions options;
-  if (GetContentClient()->browser()->AllowSetCookie(
-          url, first_party_for_cookies, cookie, resource_context_,
-          render_process_id_, render_frame_id, options)) {
-    net::URLRequestContext* context = GetRequestContextForURL(url);
-    // Pass a null callback since we don't care about when the 'set' completes.
-    context->cookie_store()->SetCookieWithOptionsAsync(
-        url, cookie, options, net::CookieStore::SetCookiesCallback());
-  }
-}
-
-void RenderFrameMessageFilter::GetCookies(int render_frame_id,
-                                          const GURL& url,
-                                          const GURL& first_party_for_cookies,
-                                          GetCookiesCallback callback) {
-  ChildProcessSecurityPolicyImpl* policy =
-      ChildProcessSecurityPolicyImpl::GetInstance();
-  if (!policy->CanAccessDataForOrigin(render_process_id_, url)) {
-    bad_message::ReceivedBadMessage(this,
-                                    bad_message::RFMF_GET_COOKIES_BAD_ORIGIN);
-    std::move(callback).Run(std::string());
-    return;
-  }
-
-  net::CookieOptions options;
-  if (net::registry_controlled_domains::SameDomainOrHost(
-          url, first_party_for_cookies,
-          net::registry_controlled_domains::INCLUDE_PRIVATE_REGISTRIES)) {
-    // TODO(mkwst): This check ought to further distinguish between frames
-    // initiated in a strict or lax same-site context.
-    options.set_same_site_cookie_mode(
-        net::CookieOptions::SameSiteCookieMode::INCLUDE_STRICT_AND_LAX);
-  } else {
-    options.set_same_site_cookie_mode(
-        net::CookieOptions::SameSiteCookieMode::DO_NOT_INCLUDE);
-  }
-
-  // If we crash here, figure out what URL the renderer was requesting.
-  // http://crbug.com/99242
-  char url_buf[128];
-  base::strlcpy(url_buf, url.spec().c_str(), arraysize(url_buf));
-  base::debug::Alias(url_buf);
-
-  net::URLRequestContext* context = GetRequestContextForURL(url);
-  context->cookie_store()->GetCookieListWithOptionsAsync(
-      url, options,
-      base::Bind(&RenderFrameMessageFilter::CheckPolicyForCookies, this,
-                 render_frame_id, url, first_party_for_cookies,
-                 base::Passed(&callback)));
-}
-
 #if BUILDFLAG(ENABLE_PLUGINS)
 
-void RenderFrameMessageFilter::OnGetPlugins(
-    bool refresh,
-    const url::Origin& main_frame_origin,
-    IPC::Message* reply_msg) {
-  // Don't refresh if the specified threshold has not been passed.  Note that
-  // this check is performed before off-loading to the file thread.  The reason
-  // we do this is that some pages tend to request that the list of plugins be
-  // refreshed at an excessive rate.  This instigates disk scanning, as the list
-  // is accumulated by doing multiple reads from disk.  This effect is
-  // multiplied when we have several pages requesting this operation.
-  if (refresh) {
-    const base::TimeDelta threshold = base::TimeDelta::FromSeconds(
-        kPluginsRefreshThresholdInSeconds);
-    const base::TimeTicks now = base::TimeTicks::Now();
-    if (now - last_plugin_refresh_time_ >= threshold) {
-      // Only refresh if the threshold hasn't been exceeded yet.
-      PluginServiceImpl::GetInstance()->RefreshPlugins();
-      last_plugin_refresh_time_ = now;
-    }
-  }
-
-  PluginServiceImpl::GetInstance()->GetPlugins(
-      base::Bind(&RenderFrameMessageFilter::GetPluginsCallback, this, reply_msg,
-                 main_frame_origin));
-}
-
-void RenderFrameMessageFilter::GetPluginsCallback(
-    IPC::Message* reply_msg,
-    const url::Origin& main_frame_origin,
-    const std::vector<WebPluginInfo>& all_plugins) {
-  // Filter the plugin list.
-  PluginServiceFilter* filter = PluginServiceImpl::GetInstance()->GetFilter();
-  std::vector<WebPluginInfo> plugins;
-
-  int child_process_id = -1;
-  int routing_id = MSG_ROUTING_NONE;
-  // In this loop, copy the WebPluginInfo (and do not use a reference) because
-  // the filter might mutate it.
-  for (WebPluginInfo plugin : all_plugins) {
-    // TODO(crbug.com/621724): Pass an url::Origin instead of a GURL.
-    if (!filter ||
-        filter->IsPluginAvailable(child_process_id, routing_id,
-                                  resource_context_, main_frame_origin.GetURL(),
-                                  main_frame_origin, &plugin)) {
-      plugins.push_back(plugin);
-    }
-  }
-
-  FrameHostMsg_GetPlugins::WriteReplyParams(reply_msg, plugins);
-  Send(reply_msg);
-}
-
 void RenderFrameMessageFilter::OnGetPluginInfo(
     int render_frame_id,
     const GURL& url,
@@ -533,6 +498,9 @@ void RenderFrameMessageFilter::OnGetPluginInfo(
     bool* found,
     WebPluginInfo* info,
     std::string* actual_mime_type) {
+  if (!resource_context_)
+    return;
+
   bool allow_wildcard = true;
   *found = plugin_service_->GetPluginInfo(
       render_process_id_, render_frame_id, resource_context_, url,
@@ -542,12 +510,11 @@ void RenderFrameMessageFilter::OnGetPluginInfo(
 
 void RenderFrameMessageFilter::OnOpenChannelToPepperPlugin(
     const base::FilePath& path,
+    const base::Optional<url::Origin>& origin_lock,
     IPC::Message* reply_msg) {
   plugin_service_->OpenChannelToPpapiPlugin(
-      render_process_id_,
-      path,
-      profile_data_directory_,
-      new OpenChannelToPpapiPluginCallback(this, resource_context_, reply_msg));
+      render_process_id_, path, profile_data_directory_, origin_lock,
+      new OpenChannelToPpapiPluginCallback(this, reply_msg));
 }
 
 void RenderFrameMessageFilter::OnDidCreateOutOfProcessPepperInstance(
@@ -597,8 +564,7 @@ void RenderFrameMessageFilter::OnOpenChannelToPpapiBroker(
     int routing_id,
     const base::FilePath& path) {
   plugin_service_->OpenChannelToPpapiBroker(
-      render_process_id_,
-      path,
+      render_process_id_, routing_id, path,
       new OpenChannelToPpapiBrokerCallback(this, routing_id));
 }
 
@@ -613,17 +579,4 @@ void RenderFrameMessageFilter::OnPluginInstanceThrottleStateChange(
 
 #endif  // ENABLE_PLUGINS
 
-net::URLRequestContext* RenderFrameMessageFilter::GetRequestContextForURL(
-    const GURL& url) {
-  DCHECK_CURRENTLY_ON(BrowserThread::IO);
-
-  net::URLRequestContext* context =
-      GetContentClient()->browser()->OverrideRequestContextForURL(
-          url, resource_context_);
-  if (!context)
-    context = request_context_->GetURLRequestContext();
-
-  return context;
-}
-
 }  // namespace content
