@@ -13,8 +13,8 @@
 #include <vector>
 
 #include "base/callback.h"
+#include "base/containers/id_map.h"
 #include "base/files/file_path.h"
-#include "base/id_map.h"
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
 #include "base/observer_list_threadsafe.h"
@@ -24,34 +24,27 @@
 #include "content/browser/service_worker/service_worker_registration_status.h"
 #include "content/browser/service_worker/service_worker_storage.h"
 #include "content/common/content_export.h"
-#include "content/common/worker_url_loader_factory_provider.mojom.h"
 #include "content/public/browser/service_worker_context.h"
+#include "third_party/blink/public/mojom/service_worker/service_worker_registration.mojom.h"
 
 class GURL;
 
 namespace base {
 class FilePath;
-class SingleThreadTaskRunner;
 }
 
 namespace storage {
-class BlobStorageContext;
 class QuotaManagerProxy;
 class SpecialStoragePolicy;
 }
 
 namespace content {
 
-class EmbeddedWorkerRegistry;
 class ServiceWorkerContextCoreObserver;
 class ServiceWorkerContextWrapper;
-class ServiceWorkerDatabaseTaskManager;
-class ServiceWorkerDispatcherHost;
 class ServiceWorkerJobCoordinator;
-class ServiceWorkerNavigationHandleCore;
 class ServiceWorkerProviderHost;
 class ServiceWorkerRegistration;
-class ServiceWorkerStorage;
 class URLLoaderFactoryGetter;
 
 // This class manages data associated with service workers.
@@ -60,29 +53,33 @@ class URLLoaderFactoryGetter;
 // is the root of the containment hierarchy for service worker data
 // associated with a particular partition.
 class CONTENT_EXPORT ServiceWorkerContextCore
-    : NON_EXPORTED_BASE(public ServiceWorkerVersion::Listener) {
+    : public ServiceWorkerVersion::Observer {
  public:
-  using BoolCallback = base::Callback<void(bool)>;
-  using StatusCallback = base::Callback<void(ServiceWorkerStatusCode status)>;
+  using BoolCallback = base::OnceCallback<void(bool)>;
+  using StatusCallback =
+      base::OnceCallback<void(blink::ServiceWorkerStatusCode status)>;
   using RegistrationCallback =
-      base::Callback<void(ServiceWorkerStatusCode status,
-                          const std::string& status_message,
-                          int64_t registration_id)>;
-  using UpdateCallback = base::Callback<void(ServiceWorkerStatusCode status,
-                                             const std::string& status_message,
-                                             int64_t registration_id)>;
+      base::OnceCallback<void(blink::ServiceWorkerStatusCode status,
+                              const std::string& status_message,
+                              int64_t registration_id)>;
+  using UpdateCallback =
+      base::OnceCallback<void(blink::ServiceWorkerStatusCode status,
+                              const std::string& status_message,
+                              int64_t registration_id)>;
   using UnregistrationCallback =
-      base::Callback<void(ServiceWorkerStatusCode status)>;
-  using ProviderMap = IDMap<std::unique_ptr<ServiceWorkerProviderHost>>;
-  using ProcessToProviderMap = IDMap<std::unique_ptr<ProviderMap>>;
-
+      base::OnceCallback<void(blink::ServiceWorkerStatusCode status)>;
+  using ProviderByIdMap =
+      std::map<int, std::unique_ptr<ServiceWorkerProviderHost>>;
   using ProviderByClientUUIDMap =
       std::map<std::string, ServiceWorkerProviderHost*>;
 
   // Directory for ServiceWorkerStorage and ServiceWorkerCacheManager.
   static const base::FilePath::CharType kServiceWorkerDirectory[];
 
-  // Iterates over ServiceWorkerProviderHost objects in a ProcessToProviderMap.
+  // Iterates over ServiceWorkerProviderHost objects in the ProviderByIdMap.
+  // Note: As ProviderHostIterator is operating on a member of
+  // ServiceWorkerContextCore, users must ensure the ServiceWorkerContextCore
+  // instance always outlives the ProviderHostIterator one.
   class CONTENT_EXPORT ProviderHostIterator {
    public:
     ~ProviderHostIterator();
@@ -93,16 +90,13 @@ class CONTENT_EXPORT ServiceWorkerContextCore
    private:
     friend class ServiceWorkerContextCore;
     using ProviderHostPredicate =
-        base::Callback<bool(ServiceWorkerProviderHost*)>;
-    ProviderHostIterator(ProcessToProviderMap* map,
-                         const ProviderHostPredicate& predicate);
-    void Initialize();
-    bool ForwardUntilMatchingProviderHost();
+        base::RepeatingCallback<bool(ServiceWorkerProviderHost*)>;
+    ProviderHostIterator(ProviderByIdMap* map, ProviderHostPredicate predicate);
+    void ForwardUntilMatchingProviderHost();
 
-    ProcessToProviderMap* map_;
+    ProviderByIdMap* const map_;
     ProviderHostPredicate predicate_;
-    std::unique_ptr<ProcessToProviderMap::iterator> process_iterator_;
-    std::unique_ptr<ProviderMap::iterator> provider_host_iterator_;
+    ProviderByIdMap::iterator provider_host_iterator_;
 
     DISALLOW_COPY_AND_ASSIGN(ProviderHostIterator);
   };
@@ -113,20 +107,18 @@ class CONTENT_EXPORT ServiceWorkerContextCore
   // ServiceWorkerContextWrapper. When Notify() of |observer_list| is called in
   // ServiceWorkerContextCore, the methods of ServiceWorkerContextCoreObserver
   // will be called on the thread which called AddObserver() of |observer_list|.
-  // |blob_context| and |url_loader_factory_getter| are used only
-  // when IsServicificationEnabled is true.
   ServiceWorkerContextCore(
       const base::FilePath& user_data_directory,
-      std::unique_ptr<ServiceWorkerDatabaseTaskManager>
-          database_task_runner_manager,
-      const scoped_refptr<base::SingleThreadTaskRunner>& disk_cache_thread,
+      scoped_refptr<base::SequencedTaskRunner> database_task_runner,
       storage::QuotaManagerProxy* quota_manager_proxy,
       storage::SpecialStoragePolicy* special_storage_policy,
-      base::WeakPtr<storage::BlobStorageContext> blob_context,
       URLLoaderFactoryGetter* url_loader_factory_getter,
+      std::unique_ptr<blink::URLLoaderFactoryBundleInfo>
+          non_network_loader_factory_bundle_info_for_update_check,
       base::ObserverListThreadSafe<ServiceWorkerContextCoreObserver>*
           observer_list,
       ServiceWorkerContextWrapper* wrapper);
+  // TODO(https://crbug.com/877356): Remove this copy mechanism.
   ServiceWorkerContextCore(
       ServiceWorkerContextCore* old_context,
       ServiceWorkerContextWrapper* wrapper);
@@ -134,7 +126,7 @@ class CONTENT_EXPORT ServiceWorkerContextCore
 
   void OnStorageWiped();
 
-  // ServiceWorkerVersion::Listener overrides.
+  // ServiceWorkerVersion::Observer overrides.
   void OnRunningStateChanged(ServiceWorkerVersion* version) override;
   void OnVersionStateChanged(ServiceWorkerVersion* version) override;
   void OnDevToolsRoutingIdChanged(ServiceWorkerVersion* version) override;
@@ -145,51 +137,45 @@ class CONTENT_EXPORT ServiceWorkerContextCore
                        int column_number,
                        const GURL& source_url) override;
   void OnReportConsoleMessage(ServiceWorkerVersion* version,
-                              int source_identifier,
-                              int message_level,
+                              blink::mojom::ConsoleMessageSource source,
+                              blink::mojom::ConsoleMessageLevel message_level,
                               const base::string16& message,
                               int line_number,
                               const GURL& source_url) override;
   void OnControlleeAdded(ServiceWorkerVersion* version,
-                         ServiceWorkerProviderHost* provider_host) override;
+                         const std::string& client_uuid,
+                         const ServiceWorkerClientInfo& client_info) override;
   void OnControlleeRemoved(ServiceWorkerVersion* version,
-                           ServiceWorkerProviderHost* provider_host) override;
+                           const std::string& client_uuid) override;
+  void OnNoControllees(ServiceWorkerVersion* version) override;
 
   ServiceWorkerContextWrapper* wrapper() const { return wrapper_; }
   ServiceWorkerStorage* storage() { return storage_.get(); }
   ServiceWorkerProcessManager* process_manager();
-  EmbeddedWorkerRegistry* embedded_worker_registry() {
-    return embedded_worker_registry_.get();
-  }
   ServiceWorkerJobCoordinator* job_coordinator() {
     return job_coordinator_.get();
   }
 
-  // Maintains DispatcherHosts to exchange service worker related messages
-  // through them. The DispatcherHosts are not owned by this class.
-  void AddDispatcherHost(int process_id,
-                         ServiceWorkerDispatcherHost* dispatcher_host);
-  ServiceWorkerDispatcherHost* GetDispatcherHost(int process_id);
-  void RemoveDispatcherHost(int process_id);
-
   // The context class owns the set of ProviderHosts.
   void AddProviderHost(
       std::unique_ptr<ServiceWorkerProviderHost> provider_host);
-  ServiceWorkerProviderHost* GetProviderHost(int process_id, int provider_id);
-  void RemoveProviderHost(int process_id, int provider_id);
-  void RemoveAllProviderHostsForProcess(int process_id);
-  std::unique_ptr<ProviderHostIterator> GetProviderHostIterator();
+  ServiceWorkerProviderHost* GetProviderHost(int provider_id);
+  void RemoveProviderHost(int provider_id);
 
-  // Returns a ProviderHost iterator for all ServiceWorker clients for
-  // the |origin|.  This only returns ProviderHosts that are of CONTROLLEE
-  // and belong to the |origin|.
+  // Returns a ProviderHost iterator for all service worker clients for the
+  // |origin|. If |include_reserved_clients| is false, this only returns clients
+  // that are execution ready (i.e., for windows, the document has been
+  // created and for workers, the final response after redirects has been
+  // delivered).
   std::unique_ptr<ProviderHostIterator> GetClientProviderHostIterator(
-      const GURL& origin);
+      const GURL& origin,
+      bool include_reserved_clients);
 
   // Runs the callback with true if there is a ProviderHost for |origin| of type
-  // SERVICE_WORKER_PROVIDER_FOR_WINDOW which is a main (top-level) frame.
+  // blink::mojom::ServiceWorkerProviderType::kForWindow which is a main
+  // (top-level) frame. Reserved clients are ignored.
   void HasMainFrameProviderHost(const GURL& origin,
-                                const BoolCallback& callback) const;
+                                BoolCallback callback) const;
 
   // Maintains a map from Client UUID to ProviderHost.
   // (Note: instead of maintaining 2 maps we might be able to uniformly use
@@ -201,29 +187,34 @@ class CONTENT_EXPORT ServiceWorkerContextCore
   ServiceWorkerProviderHost* GetProviderHostByClientID(
       const std::string& client_uuid);
 
-  // Non-null |provider_host| must be given if this is called from a document.
-  void RegisterServiceWorker(const GURL& script_url,
-                             const ServiceWorkerRegistrationOptions& options,
-                             ServiceWorkerProviderHost* provider_host,
-                             const RegistrationCallback& callback);
-  void UnregisterServiceWorker(const GURL& pattern,
-                               const UnregistrationCallback& callback);
+  void RegisterServiceWorker(
+      const GURL& script_url,
+      const blink::mojom::ServiceWorkerRegistrationOptions& options,
+      RegistrationCallback callback);
+  void UnregisterServiceWorker(const GURL& scope,
+                               UnregistrationCallback callback);
 
-  // Callback is called issued after all unregistrations occur.  The Status
-  // is populated as SERVICE_WORKER_OK if all succeed, or SERVICE_WORKER_FAILED
-  // if any did not succeed.
-  void UnregisterServiceWorkers(const GURL& origin,
-                                const UnregistrationCallback& callback);
+  // Callback is called after all deletions occured. The status code is
+  // blink::ServiceWorkerStatusCode::kOk if all succeed, or
+  // SERVICE_WORKER_FAILED if any did not succeed.
+  void DeleteForOrigin(const GURL& origin, StatusCallback callback);
+
+  // Performs internal storage cleanup. Operations to the storage in the past
+  // (e.g. deletion) are usually recorded in disk for a certain period until
+  // compaction happens. This method wipes them out to ensure that the deleted
+  // entries and other traces like log files are removed.
+  void PerformStorageCleanup(base::OnceClosure callback);
 
   // Updates the service worker. If |force_bypass_cache| is true or 24 hours
   // have passed since the last update, bypasses the browser cache.
   void UpdateServiceWorker(ServiceWorkerRegistration* registration,
                            bool force_bypass_cache);
+  // |callback| is called when the promise for
+  // ServiceWorkerRegistration.update() would be resolved.
   void UpdateServiceWorker(ServiceWorkerRegistration* registration,
                            bool force_bypass_cache,
                            bool skip_script_comparison,
-                           ServiceWorkerProviderHost* provider_host,
-                           const UpdateCallback& callback);
+                           UpdateCallback callback);
 
   // Used in DevTools to update the service worker registrations without
   // consulting the browser cache while loading the controlled page. The
@@ -251,15 +242,6 @@ class CONTENT_EXPORT ServiceWorkerContextCore
     return live_versions_;
   }
 
-  // PlzNavigate
-  // Methods to manage the map keeping track of all
-  // ServiceWorkerNavigationHandleCores registered for ongoing navigations.
-  void AddNavigationHandleCore(int service_worker_provider_id,
-                               ServiceWorkerNavigationHandleCore* handle);
-  void RemoveNavigationHandleCore(int service_worker_provider_id);
-  ServiceWorkerNavigationHandleCore* GetNavigationHandleCore(
-      int service_worker_provider_id);
-
   std::vector<ServiceWorkerRegistrationInfo> GetAllLiveRegistrationInfo();
   std::vector<ServiceWorkerVersionInfo> GetAllLiveVersionInfo();
 
@@ -268,52 +250,29 @@ class CONTENT_EXPORT ServiceWorkerContextCore
   void ProtectVersion(const scoped_refptr<ServiceWorkerVersion>& version);
   void UnprotectVersion(int64_t version_id);
 
-  // Returns new context-local unique ID.
-  int GetNewServiceWorkerHandleId();
-  int GetNewRegistrationHandleId();
-
   void ScheduleDeleteAndStartOver() const;
 
   // Deletes all files on disk and restarts the system. This leaves the system
   // in a disabled state until it's done.
-  void DeleteAndStartOver(const StatusCallback& callback);
+  void DeleteAndStartOver(StatusCallback callback);
 
-  // Methods to support cross site navigations.
-  std::unique_ptr<ServiceWorkerProviderHost> TransferProviderHostOut(
-      int process_id,
-      int provider_id);
-  void TransferProviderHostIn(
-      int new_process_id,
-      int new_host_id,
-      std::unique_ptr<ServiceWorkerProviderHost> provider_host);
+  void ClearAllServiceWorkersForTest(base::OnceClosure callback);
 
-  void ClearAllServiceWorkersForTest(const base::Closure& callback);
-
-  // Determines if there is a ServiceWorker registration that matches |url|, and
-  // if |other_url| falls inside the scope of the same registration. See
-  // ServiceWorkerContext::CheckHasServiceWorker for more details.
+  // Determines if there is a ServiceWorker registration that matches
+  // |url|. See ServiceWorkerContext::CheckHasServiceWorker for more
+  // details.
   void CheckHasServiceWorker(
       const GURL& url,
-      const GURL& other_url,
       const ServiceWorkerContext::CheckHasServiceWorkerCallback callback);
 
   void UpdateVersionFailureCount(int64_t version_id,
-                                 ServiceWorkerStatusCode status);
+                                 blink::ServiceWorkerStatusCode status);
   // Returns the count of consecutive start worker failures for the given
   // version. The count resets to zero when the worker successfully starts.
   int GetVersionFailureCount(int64_t version_id);
 
-  // Binds the ServiceWorkerWorkerClient of a dedicated (or shared) worker to
-  // the parent frame's ServiceWorkerProviderHost. (This is used only when
-  // off-main-thread-fetch is enabled.)
-  void BindWorkerFetchContext(
-      int render_process_id,
-      int service_worker_provider_id,
-      mojom::ServiceWorkerWorkerClientAssociatedPtrInfo client_ptr_info);
-
-  base::WeakPtr<storage::BlobStorageContext> blob_storage_context() {
-    return blob_storage_context_;
-  }
+  // Called by ServiceWorkerStorage when StoreRegistration() succeeds.
+  void NotifyRegistrationStored(int64_t registration_id, const GURL& scope);
 
   URLLoaderFactoryGetter* loader_factory_getter() {
     return loader_factory_getter_.get();
@@ -323,6 +282,17 @@ class CONTENT_EXPORT ServiceWorkerContextCore
     return weak_factory_.GetWeakPtr();
   }
 
+  int GetNextEmbeddedWorkerId();
+
+  // Called when ServiceWorkerImportedScriptUpdateCheck is enabled.
+  // Returns a factory bundle suitable for the browser process to use to fetch
+  // a non-installed service worker main script or imported script during an
+  // update check. It must not be sent to a renderer process. The bundle does
+  // not support reconnection to the network service, so it should be used for
+  // only a single service worker update check.
+  scoped_refptr<blink::URLLoaderFactoryBundle>
+  GetLoaderFactoryBundleForUpdateCheck();
+
  private:
   friend class ServiceWorkerContextCoreTest;
   FRIEND_TEST_ALL_PREFIXES(ServiceWorkerContextCoreTest, FailureInfo);
@@ -332,54 +302,55 @@ class CONTENT_EXPORT ServiceWorkerContextCore
 
   struct FailureInfo {
     int count;
-    ServiceWorkerStatusCode last_failure;
+    blink::ServiceWorkerStatusCode last_failure;
   };
 
-  ProviderMap* GetProviderMapForProcess(int process_id) {
-    return providers_->Lookup(process_id);
-  }
-
-  void RegistrationComplete(const GURL& pattern,
-                            const RegistrationCallback& callback,
-                            ServiceWorkerStatusCode status,
+  void RegistrationComplete(const GURL& scope,
+                            RegistrationCallback callback,
+                            blink::ServiceWorkerStatusCode status,
                             const std::string& status_message,
                             ServiceWorkerRegistration* registration);
 
-  void UpdateComplete(const UpdateCallback& callback,
-                      ServiceWorkerStatusCode status,
+  void UpdateComplete(UpdateCallback callback,
+                      blink::ServiceWorkerStatusCode status,
                       const std::string& status_message,
                       ServiceWorkerRegistration* registration);
 
-  void UnregistrationComplete(const GURL& pattern,
-                              const UnregistrationCallback& callback,
+  void UnregistrationComplete(const GURL& scope,
+                              UnregistrationCallback callback,
                               int64_t registration_id,
-                              ServiceWorkerStatusCode status);
+                              blink::ServiceWorkerStatusCode status);
 
-  void DidGetAllRegistrationsForUnregisterForOrigin(
-      const UnregistrationCallback& result,
-      const GURL& origin,
-      ServiceWorkerStatusCode status,
-      const std::vector<ServiceWorkerRegistrationInfo>& registrations);
+  bool IsValidRegisterRequest(const GURL& script_url,
+                              const GURL& scope_url,
+                              std::string* out_error) const;
+
+  void DidGetRegistrationsForDeleteForOrigin(
+      base::OnceCallback<void(blink::ServiceWorkerStatusCode)> callback,
+      blink::ServiceWorkerStatusCode status,
+      const std::vector<scoped_refptr<ServiceWorkerRegistration>>&
+          registrations);
 
   void DidFindRegistrationForCheckHasServiceWorker(
-      const GURL& other_url,
-      const ServiceWorkerContext::CheckHasServiceWorkerCallback callback,
-      ServiceWorkerStatusCode status,
+      ServiceWorkerContext::CheckHasServiceWorkerCallback callback,
+      blink::ServiceWorkerStatusCode status,
       scoped_refptr<ServiceWorkerRegistration> registration);
   void OnRegistrationFinishedForCheckHasServiceWorker(
-      const ServiceWorkerContext::CheckHasServiceWorkerCallback callback,
+      ServiceWorkerContext::CheckHasServiceWorkerCallback callback,
       scoped_refptr<ServiceWorkerRegistration> registration);
 
   // It's safe to store a raw pointer instead of a scoped_refptr to |wrapper_|
   // because the Wrapper::Shutdown call that hops threads to destroy |this| uses
   // Bind() to hold a reference to |wrapper_| until |this| is fully destroyed.
   ServiceWorkerContextWrapper* wrapper_;
-  std::map<int /* process_id */, ServiceWorkerDispatcherHost*>
-      dispatcher_hosts_;
-  std::unique_ptr<ProcessToProviderMap> providers_;
+
+  // |providers_| owns the provider hosts.
+  std::unique_ptr<ProviderByIdMap> providers_;
+
+  // |provider_by_uuid_| contains raw pointers to hosts owned by |providers_|.
   std::unique_ptr<ProviderByClientUUIDMap> provider_by_uuid_;
+
   std::unique_ptr<ServiceWorkerStorage> storage_;
-  scoped_refptr<EmbeddedWorkerRegistry> embedded_worker_registry_;
   std::unique_ptr<ServiceWorkerJobCoordinator> job_coordinator_;
   std::map<int64_t, ServiceWorkerRegistration*> live_registrations_;
   std::map<int64_t, ServiceWorkerVersion*> live_versions_;
@@ -387,25 +358,23 @@ class CONTENT_EXPORT ServiceWorkerContextCore
 
   std::map<int64_t /* version_id */, FailureInfo> failure_counts_;
 
-  // PlzNavigate
-  // Map of ServiceWorkerNavigationHandleCores used for navigation requests.
-  std::map<int, ServiceWorkerNavigationHandleCore*>
-      navigation_handle_cores_map_;
-
-  // IsServicificationEnabled
-  base::WeakPtr<storage::BlobStorageContext> blob_storage_context_;
   scoped_refptr<URLLoaderFactoryGetter> loader_factory_getter_;
 
+  scoped_refptr<blink::URLLoaderFactoryBundle>
+      loader_factory_bundle_for_update_check_;
+
   bool force_update_on_page_load_;
-  int next_handle_id_;
-  int next_registration_handle_id_;
   // Set in RegisterServiceWorker(), cleared in ClearAllServiceWorkersForTest().
   // This is used to avoid unnecessary disk read operation in tests. This value
   // is false if Chrome was relaunched after service workers were registered.
   bool was_service_worker_registered_;
-  scoped_refptr<base::ObserverListThreadSafe<ServiceWorkerContextCoreObserver>>
-      observer_list_;
-  base::WeakPtrFactory<ServiceWorkerContextCore> weak_factory_;
+  using ServiceWorkerContextObserverList =
+      base::ObserverListThreadSafe<ServiceWorkerContextCoreObserver>;
+  const scoped_refptr<ServiceWorkerContextObserverList> observer_list_;
+
+  int next_embedded_worker_id_ = 0;
+
+  base::WeakPtrFactory<ServiceWorkerContextCore> weak_factory_{this};
 
   DISALLOW_COPY_AND_ASSIGN(ServiceWorkerContextCore);
 };
