@@ -17,8 +17,8 @@
 #include "base/rand_util.h"
 #include "base/system/sys_info.h"
 #include "base/task/post_task.h"
-#include "base/task/task_scheduler/task_scheduler.h"
 #include "base/task/task_traits.h"
+#include "base/task/thread_pool/thread_pool.h"
 #include "base/trace_event/trace_event.h"
 #include "build/build_config.h"
 #include "gin/per_isolate_data.h"
@@ -47,7 +47,7 @@ class ConvertableToTraceFormatWrapper final
     : public base::trace_event::ConvertableToTraceFormat {
  public:
   explicit ConvertableToTraceFormatWrapper(
-      std::unique_ptr<v8::ConvertableToTraceFormat>& inner)
+      std::unique_ptr<v8::ConvertableToTraceFormat> inner)
       : inner_(std::move(inner)) {}
   ~ConvertableToTraceFormatWrapper() override = default;
   void AppendAsTraceFormat(std::string* out) const final {
@@ -266,6 +266,34 @@ base::LazyInstance<PageAllocator>::Leaky g_page_allocator =
 
 }  // namespace
 
+}  // namespace gin
+
+namespace base {
+
+namespace trace_event {
+
+// Allow std::unique_ptr<v8::ConvertableToTraceFormat> to be a valid
+// initialization value for trace macros.
+template <>
+struct TraceValue::Helper<
+    std::unique_ptr<v8::ConvertableToTraceFormat>> {
+  static constexpr unsigned char kType = TRACE_VALUE_TYPE_CONVERTABLE;
+  static inline void SetValue(
+      TraceValue* v,
+      std::unique_ptr<v8::ConvertableToTraceFormat> value) {
+    // NOTE: |as_convertable| is an owning pointer, so using new here
+    // is acceptable.
+    v->as_convertable =
+        new gin::ConvertableToTraceFormatWrapper(std::move(value));
+  }
+};
+
+} // namespace trace_event
+
+} // namespace base
+
+namespace gin {
+
 class V8Platform::TracingControllerImpl : public v8::TracingController {
  public:
   TracingControllerImpl() = default;
@@ -288,22 +316,45 @@ class V8Platform::TracingControllerImpl : public v8::TracingController {
       const uint64_t* arg_values,
       std::unique_ptr<v8::ConvertableToTraceFormat>* arg_convertables,
       unsigned int flags) override {
-    std::unique_ptr<base::trace_event::ConvertableToTraceFormat>
-        convertables[2];
-    if (num_args > 0 && arg_types[0] == TRACE_VALUE_TYPE_CONVERTABLE) {
-      convertables[0].reset(
-          new ConvertableToTraceFormatWrapper(arg_convertables[0]));
-    }
-    if (num_args > 1 && arg_types[1] == TRACE_VALUE_TYPE_CONVERTABLE) {
-      convertables[1].reset(
-          new ConvertableToTraceFormatWrapper(arg_convertables[1]));
-    }
+    base::trace_event::TraceArguments args(
+        num_args, arg_names, arg_types,
+        reinterpret_cast<const unsigned long long*>(arg_values),
+        arg_convertables);
     DCHECK_LE(num_args, 2);
     base::trace_event::TraceEventHandle handle =
         TRACE_EVENT_API_ADD_TRACE_EVENT_WITH_BIND_ID(
-            phase, category_enabled_flag, name, scope, id, bind_id, num_args,
-            arg_names, arg_types, (const long long unsigned int*)arg_values,
-            convertables, flags);
+            phase, category_enabled_flag, name, scope, id, bind_id, &args,
+            flags);
+    uint64_t result;
+    memcpy(&result, &handle, sizeof(result));
+    return result;
+  }
+  uint64_t AddTraceEventWithTimestamp(
+      char phase,
+      const uint8_t* category_enabled_flag,
+      const char* name,
+      const char* scope,
+      uint64_t id,
+      uint64_t bind_id,
+      int32_t num_args,
+      const char** arg_names,
+      const uint8_t* arg_types,
+      const uint64_t* arg_values,
+      std::unique_ptr<v8::ConvertableToTraceFormat>* arg_convertables,
+      unsigned int flags,
+      int64_t timestampMicroseconds) override {
+    base::trace_event::TraceArguments args(
+        num_args, arg_names, arg_types,
+        reinterpret_cast<const unsigned long long*>(arg_values),
+        arg_convertables);
+    DCHECK_LE(num_args, 2);
+    base::TimeTicks timestamp =
+        base::TimeTicks() +
+        base::TimeDelta::FromMicroseconds(timestampMicroseconds);
+    base::trace_event::TraceEventHandle handle =
+        TRACE_EVENT_API_ADD_TRACE_EVENT_WITH_THREAD_ID_AND_TIMESTAMP(
+            phase, category_enabled_flag, name, scope, id, bind_id,
+            TRACE_EVENT_API_CURRENT_THREAD_ID, timestamp, &args, flags);
     uint64_t result;
     memcpy(&result, &handle, sizeof(result));
     return result;
@@ -358,11 +409,11 @@ int V8Platform::NumberOfWorkerThreads() {
   // V8Platform assumes the scheduler uses the same set of workers for default
   // and user blocking tasks.
   const int num_foreground_workers =
-      base::TaskScheduler::GetInstance()
+      base::ThreadPoolInstance::Get()
           ->GetMaxConcurrentNonBlockedTasksWithTraitsDeprecated(
               kDefaultTaskTraits);
   DCHECK_EQ(num_foreground_workers,
-            base::TaskScheduler::GetInstance()
+            base::ThreadPoolInstance::Get()
                 ->GetMaxConcurrentNonBlockedTasksWithTraitsDeprecated(
                     kBlockingTaskTraits));
   return std::max(1, num_foreground_workers);
