@@ -15,16 +15,17 @@
 #include <vector>
 
 #include "base/at_exit.h"
+#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/compiler_specific.h"
 #include "base/files/scoped_file.h"
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
-#include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/task_scheduler/task_scheduler.h"
+#include "base/task/single_thread_task_executor.h"
+#include "base/task/thread_pool/thread_pool.h"
 #include "base/threading/thread.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/time/default_clock.h"
@@ -39,22 +40,23 @@
 #include "google_apis/gcm/engine/gservices_settings.h"
 #include "google_apis/gcm/engine/mcs_client.h"
 #include "google_apis/gcm/monitoring/fake_gcm_stats_recorder.h"
+#include "mojo/core/embedder/embedder.h"
 #include "net/cert/cert_verifier.h"
-#include "net/cert/ct_policy_enforcer.h"
-#include "net/cert/multi_log_ct_verifier.h"
 #include "net/dns/host_resolver.h"
 #include "net/http/http_auth_handler_factory.h"
 #include "net/http/http_auth_preferences.h"
 #include "net/http/http_auth_scheme.h"
-#include "net/http/http_network_session.h"
-#include "net/http/http_server_properties_impl.h"
-#include "net/http/transport_security_state.h"
 #include "net/log/file_net_log_observer.h"
-#include "net/socket/client_socket_factory.h"
-#include "net/socket/ssl_client_socket.h"
-#include "net/ssl/channel_id_service.h"
-#include "net/ssl/default_channel_id_store.h"
+#include "net/proxy_resolution/proxy_resolution_service.h"
+#include "net/url_request/url_request_context.h"
+#include "net/url_request/url_request_context_builder.h"
 #include "net/url_request/url_request_test_util.h"
+#include "services/network/network_context.h"
+#include "services/network/public/cpp/shared_url_loader_factory.h"
+#include "services/network/public/cpp/weak_wrapper_shared_url_loader_factory.h"
+#include "services/network/public/mojom/proxy_resolving_socket.mojom.h"
+#include "services/network/public/mojom/url_loader_factory.mojom.h"
+#include "services/network/test/test_network_connection_tracker.h"
 
 #if defined(OS_MACOSX)
 #include "base/mac/scoped_nsautorelease_pool.h"
@@ -138,40 +140,6 @@ void MessageSentCallback(int64_t user_serial_number,
             << " Message send status: " << status;
 }
 
-// Needed to use a real host resolver.
-class MyTestURLRequestContext : public net::TestURLRequestContext {
- public:
-  MyTestURLRequestContext() : TestURLRequestContext(true) {
-    context_storage_.set_host_resolver(
-        net::HostResolver::CreateDefaultResolver(NULL));
-    context_storage_.set_transport_security_state(
-        base::MakeUnique<net::TransportSecurityState>());
-    Init();
-  }
-
-  ~MyTestURLRequestContext() override {}
-};
-
-class MyTestURLRequestContextGetter : public net::TestURLRequestContextGetter {
- public:
-  explicit MyTestURLRequestContextGetter(
-      const scoped_refptr<base::SingleThreadTaskRunner>& io_task_runner)
-      : TestURLRequestContextGetter(io_task_runner) {}
-
-  net::TestURLRequestContext* GetURLRequestContext() override {
-    // Construct |context_| lazily so it gets constructed on the right
-    // thread (the IO thread).
-    if (!context_)
-      context_.reset(new MyTestURLRequestContext());
-    return context_.get();
-  }
-
- private:
-  ~MyTestURLRequestContextGetter() override {}
-
-  std::unique_ptr<MyTestURLRequestContext> context_;
-};
-
 // A cert verifier that access all certificates.
 class MyTestCertVerifier : public net::CertVerifier {
  public:
@@ -179,45 +147,34 @@ class MyTestCertVerifier : public net::CertVerifier {
   ~MyTestCertVerifier() override {}
 
   int Verify(const RequestParams& params,
-             net::CRLSet* crl_set,
              net::CertVerifyResult* verify_result,
-             const net::CompletionCallback& callback,
+             net::CompletionOnceCallback callback,
              std::unique_ptr<Request>* out_req,
              const net::NetLogWithSource& net_log) override {
     return net::OK;
   }
+  void SetConfig(const Config& config) override {}
 };
 
 class MCSProbeAuthPreferences : public net::HttpAuthPreferences {
  public:
-  MCSProbeAuthPreferences()
-      : HttpAuthPreferences(std::vector<std::string>()
-#if defined(OS_POSIX) && !defined(OS_ANDROID)
-                                ,
-                            std::string()
-#endif
-#if defined(OS_CHROMEOS)
-                                ,
-                            true
-#endif
-                            ) {
-  }
-  bool IsSupportedScheme(const std::string& scheme) const override {
-    return scheme == std::string(net::kBasicAuthScheme);
-  }
+  MCSProbeAuthPreferences() {}
+  ~MCSProbeAuthPreferences() override {}
+
   bool NegotiateDisableCnameLookup() const override { return false; }
   bool NegotiateEnablePort() const override { return false; }
   bool CanUseDefaultCredentials(const GURL& auth_origin) const override {
     return false;
   }
-  bool CanDelegate(const GURL& auth_origin) const override { return false; }
+  net::HttpAuth::DelegationType GetDelegationType(
+      const GURL& auth_origin) const override {
+    return net::HttpAuth::DelegationType::kNone;
+  }
 };
 
 class MCSProbe {
  public:
-  MCSProbe(
-      const base::CommandLine& command_line,
-      scoped_refptr<net::URLRequestContextGetter> url_request_context_getter);
+  explicit MCSProbe(const base::CommandLine& command_line);
   ~MCSProbe();
 
   void Start();
@@ -226,9 +183,10 @@ class MCSProbe {
   uint64_t secret() const { return secret_; }
 
  private:
+  void RequestProxyResolvingSocketFactory(
+      network::mojom::ProxyResolvingSocketFactoryRequest request);
   void CheckIn();
   void InitializeNetworkState();
-  void BuildNetworkSession();
 
   void LoadCallback(std::unique_ptr<GCMStore::LoadResult> load_result);
   void UpdateCallback(bool success);
@@ -249,20 +207,13 @@ class MCSProbe {
   int server_port_;
 
   // Network state.
-  scoped_refptr<net::URLRequestContextGetter> url_request_context_getter_;
+  std::unique_ptr<network::TestNetworkConnectionTracker>
+      network_connection_tracker_;
+  std::unique_ptr<net::URLRequestContext> url_request_context_;
   net::NetLog net_log_;
   std::unique_ptr<net::FileNetLogObserver> logger_;
-  std::unique_ptr<net::HostResolver> host_resolver_;
-  std::unique_ptr<net::CertVerifier> cert_verifier_;
-  std::unique_ptr<net::ChannelIDService> system_channel_id_service_;
-  std::unique_ptr<net::TransportSecurityState> transport_security_state_;
-  std::unique_ptr<net::CTVerifier> cert_transparency_verifier_;
-  std::unique_ptr<net::CTPolicyEnforcer> ct_policy_enforcer_;
   MCSProbeAuthPreferences http_auth_preferences_;
   std::unique_ptr<net::HttpAuthHandlerFactory> http_auth_handler_factory_;
-  std::unique_ptr<net::HttpServerPropertiesImpl> http_server_properties_;
-  std::unique_ptr<net::HttpNetworkSession> network_session_;
-  std::unique_ptr<net::ProxyService> proxy_service_;
 
   FakeGCMStatsRecorder recorder_;
   std::unique_ptr<GCMStore> gcm_store_;
@@ -273,19 +224,25 @@ class MCSProbe {
 
   base::Thread file_thread_;
 
+  std::unique_ptr<network::NetworkContext> network_context_;
+  network::mojom::NetworkContextPtr network_context_pipe_;
+  network::mojom::URLLoaderFactoryPtr url_loader_factory_;
+  scoped_refptr<network::SharedURLLoaderFactory> shared_url_loader_factory_;
+
   std::unique_ptr<base::RunLoop> run_loop_;
 };
 
-MCSProbe::MCSProbe(
-    const base::CommandLine& command_line,
-    scoped_refptr<net::URLRequestContextGetter> url_request_context_getter)
+MCSProbe::MCSProbe(const base::CommandLine& command_line)
     : command_line_(command_line),
       gcm_store_path_(base::FilePath(FILE_PATH_LITERAL("gcm_store"))),
       android_id_(0),
       secret_(0),
       server_port_(0),
-      url_request_context_getter_(url_request_context_getter),
+      network_connection_tracker_(
+          network::TestNetworkConnectionTracker::CreateInstance()),
       file_thread_("FileThread") {
+  network_connection_tracker_->SetConnectionType(
+      network::mojom::ConnectionType::CONNECTION_ETHERNET);
   if (command_line.HasSwitch(kRMQFileName)) {
     gcm_store_path_ = command_line.GetSwitchValuePath(kRMQFileName);
   }
@@ -317,27 +274,24 @@ MCSProbe::~MCSProbe() {
 void MCSProbe::Start() {
   file_thread_.Start();
   InitializeNetworkState();
-  BuildNetworkSession();
-  std::vector<GURL> endpoints(1,
-                              GURL("https://" +
-                                   net::HostPortPair(server_host_,
-                                                     server_port_).ToString()));
-  connection_factory_.reset(
-      new ConnectionFactoryImpl(endpoints,
-                                kDefaultBackoffPolicy,
-                                network_session_.get(),
-                                NULL,
-                                &net_log_,
-                                &recorder_));
-  gcm_store_.reset(
-      new GCMStoreImpl(gcm_store_path_, file_thread_.task_runner(),
-                       base::WrapUnique<Encryptor>(new FakeEncryptor)));
-  mcs_client_.reset(new MCSClient("probe",
-                                  &clock_,
-                                  connection_factory_.get(),
-                                  gcm_store_.get(),
-                                  &recorder_));
-  run_loop_.reset(new base::RunLoop());
+  std::vector<GURL> endpoints(
+      1, GURL("https://" +
+              net::HostPortPair(server_host_, server_port_).ToString()));
+
+  connection_factory_ = std::make_unique<ConnectionFactoryImpl>(
+      endpoints, kDefaultBackoffPolicy,
+      base::BindRepeating(&MCSProbe::RequestProxyResolvingSocketFactory,
+                          base::Unretained(this)),
+      base::ThreadTaskRunnerHandle::Get(), &recorder_,
+      network_connection_tracker_.get());
+  gcm_store_ = std::make_unique<GCMStoreImpl>(
+      gcm_store_path_, file_thread_.task_runner(),
+      std::make_unique<FakeEncryptor>());
+
+  mcs_client_ = std::make_unique<MCSClient>(
+      "probe", &clock_, connection_factory_.get(), gcm_store_.get(),
+      base::ThreadTaskRunnerHandle::Get(), &recorder_);
+  run_loop_ = std::make_unique<base::RunLoop>();
   gcm_store_->Load(GCMStore::CREATE_IF_MISSING,
                    base::Bind(&MCSProbe::LoadCallback,
                               base::Unretained(this)));
@@ -381,57 +335,54 @@ void MCSProbe::InitializeNetworkState() {
     base::FilePath log_path = command_line_.GetSwitchValuePath(kLogFileSwitch);
     logger_ = net::FileNetLogObserver::CreateUnbounded(log_path, nullptr);
     net::NetLogCaptureMode capture_mode =
-        net::NetLogCaptureMode::IncludeCookiesAndCredentials();
+        net::NetLogCaptureMode::kIncludeSensitive;
     logger_->StartObserving(&net_log_, capture_mode);
   }
 
-  host_resolver_ = net::HostResolver::CreateDefaultResolver(&net_log_);
-
-  if (command_line_.HasSwitch(kIgnoreCertSwitch)) {
-    cert_verifier_.reset(new MyTestCertVerifier());
-  } else {
-    cert_verifier_ = net::CertVerifier::CreateDefault();
-  }
-  system_channel_id_service_.reset(
-      new net::ChannelIDService(new net::DefaultChannelIDStore(NULL)));
-
-  transport_security_state_.reset(new net::TransportSecurityState());
-  cert_transparency_verifier_.reset(new net::MultiLogCTVerifier());
-  ct_policy_enforcer_.reset(new net::CTPolicyEnforcer());
   http_auth_handler_factory_ = net::HttpAuthHandlerRegistryFactory::Create(
-      &http_auth_preferences_, host_resolver_.get());
-  http_server_properties_.reset(new net::HttpServerPropertiesImpl());
-  proxy_service_ = net::ProxyService::CreateDirectWithNetLog(&net_log_);
-}
+      &http_auth_preferences_, std::vector<std::string>{net::kBasicAuthScheme});
 
-void MCSProbe::BuildNetworkSession() {
-  net::HttpNetworkSession::Params session_params;
-  session_params.ignore_certificate_errors = true;
-  session_params.testing_fixed_http_port = 0;
-  session_params.testing_fixed_https_port = 0;
+  net::URLRequestContextBuilder builder;
+  builder.set_net_log(&net_log_);
+  builder.set_host_resolver(
+      net::HostResolver::CreateStandaloneResolver(&net_log_));
+  builder.set_shared_http_auth_handler_factory(
+      http_auth_handler_factory_.get());
+  builder.set_proxy_resolution_service(
+      net::ProxyResolutionService::CreateDirect());
 
-  net::HttpNetworkSession::Context session_context;
-  session_context.host_resolver = host_resolver_.get();
-  session_context.cert_verifier = cert_verifier_.get();
-  session_context.channel_id_service = system_channel_id_service_.get();
-  session_context.transport_security_state = transport_security_state_.get();
-  session_context.cert_transparency_verifier =
-      cert_transparency_verifier_.get();
-  session_context.ct_policy_enforcer = ct_policy_enforcer_.get();
-  session_context.ssl_config_service = new net::SSLConfigServiceDefaults();
-  session_context.http_auth_handler_factory = http_auth_handler_factory_.get();
-  session_context.http_server_properties = http_server_properties_.get();
-  session_context.net_log = &net_log_;
-  session_context.proxy_service = proxy_service_.get();
+  if (command_line_.HasSwitch(kIgnoreCertSwitch))
+    builder.SetCertVerifier(std::make_unique<MyTestCertVerifier>());
 
-  network_session_.reset(
-      new net::HttpNetworkSession(session_params, session_context));
+  url_request_context_ = builder.Build();
+
+  // Wrap it up with network service APIs.
+  network_context_ = std::make_unique<network::NetworkContext>(
+      nullptr /* network_service */, mojo::MakeRequest(&network_context_pipe_),
+      url_request_context_.get(),
+      /*cors_exempt_header_list=*/std::vector<std::string>());
+  auto url_loader_factory_params =
+      network::mojom::URLLoaderFactoryParams::New();
+  url_loader_factory_params->process_id = network::mojom::kBrowserProcessId;
+  url_loader_factory_params->is_corb_enabled = false;
+  network_context_->CreateURLLoaderFactory(
+      mojo::MakeRequest(&url_loader_factory_),
+      std::move(url_loader_factory_params));
+  shared_url_loader_factory_ =
+      base::MakeRefCounted<network::WeakWrapperSharedURLLoaderFactory>(
+          url_loader_factory_.get());
 }
 
 void MCSProbe::ErrorCallback() {
   LOG(INFO) << "MCS error happened";
 }
 
+void MCSProbe::RequestProxyResolvingSocketFactory(
+    network::mojom::ProxyResolvingSocketFactoryRequest request) {
+  return network_context_->CreateProxyResolvingSocketFactory(
+      std::move(request));
+}
+
 void MCSProbe::CheckIn() {
   LOG(INFO) << "Check-in request initiated.";
   checkin_proto::ChromeBuildProto chrome_build_proto;
@@ -441,19 +392,15 @@ void MCSProbe::CheckIn() {
       checkin_proto::ChromeBuildProto::CHANNEL_CANARY);
   chrome_build_proto.set_chrome_version(kChromeVersion);
 
-  CheckinRequest::RequestInfo request_info(0,
-                                           0,
+  CheckinRequest::RequestInfo request_info(0, 0,
                                            std::map<std::string, std::string>(),
-                                           std::string(),
-                                           chrome_build_proto);
+                                           std::string(), chrome_build_proto);
 
-  checkin_request_.reset(new CheckinRequest(
-      GServicesSettings().GetCheckinURL(),
-      request_info,
-      kDefaultBackoffPolicy,
+  checkin_request_ = std::make_unique<CheckinRequest>(
+      GServicesSettings().GetCheckinURL(), request_info, kDefaultBackoffPolicy,
       base::Bind(&MCSProbe::OnCheckInCompleted, base::Unretained(this)),
-      url_request_context_getter_.get(),
-      &recorder_));
+      shared_url_loader_factory_, base::ThreadTaskRunnerHandle::Get(),
+      &recorder_);
   checkin_request_->Start();
 }
 
@@ -493,26 +440,25 @@ int MCSProbeMain(int argc, char* argv[]) {
 
   base::CommandLine::Init(argc, argv);
   logging::LoggingSettings settings;
-  settings.logging_dest = logging::LOG_TO_SYSTEM_DEBUG_LOG;
+  settings.logging_dest =
+      logging::LOG_TO_SYSTEM_DEBUG_LOG | logging::LOG_TO_STDERR;
   logging::InitLogging(settings);
 
-  base::MessageLoopForIO message_loop;
-  base::TaskScheduler::CreateAndStartWithDefaultParams("MCSProbe");
+  mojo::core::Init();
 
-  // For check-in and creating registration ids.
-  const scoped_refptr<MyTestURLRequestContextGetter> context_getter =
-      new MyTestURLRequestContextGetter(base::ThreadTaskRunnerHandle::Get());
+  base::SingleThreadTaskExecutor io_task_executor(base::MessagePump::Type::IO);
+  base::ThreadPoolInstance::CreateAndStartWithDefaultParams("MCSProbe");
 
   const base::CommandLine& command_line =
       *base::CommandLine::ForCurrentProcess();
 
-  MCSProbe mcs_probe(command_line, context_getter);
+  MCSProbe mcs_probe(command_line);
   mcs_probe.Start();
 
   base::RunLoop run_loop;
   run_loop.Run();
 
-  base::TaskScheduler::GetInstance()->Shutdown();
+  base::ThreadPoolInstance::Get()->Shutdown();
 
   return 0;
 }
